/**
 * Copyright(c) 1996 DTAI, Incorporated (http://www.dtai.com)
 *
 *                        All rights reserved
 *
 * Permission to use, copy, modify and distribute this material for
 * any purpose and without fee is hereby granted, provided that the
 * above copyright notice and this permission notice appear in all
 * copies, and that the name of DTAI, Incorporated not be used in
 * advertising or publicity pertaining to this material without the
 * specific, prior written permission of an authorized representative of
 * DTAI, Incorporated.
 *
 * DTAI, INCORPORATED MAKES NO REPRESENTATIONS AND EXTENDS NO WARRANTIES,
 * EXPRESS OR IMPLIED, WITH RESPECT TO THE SOFTWARE, INCLUDING, BUT
 * NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 * FITNESS FOR ANY PARTICULAR PURPOSE, AND THE WARRANTY AGAINST
 * INFRINGEMENT OF PATENTS OR OTHER INTELLECTUAL PROPERTY RIGHTS.  THE
 * SOFTWARE IS PROVIDED "AS IS", AND IN NO EVENT SHALL DTAI, INCORPORATED OR
 * ANY OF ITS AFFILIATES BE LIABLE FOR ANY DAMAGES, INCLUDING ANY
 * LOST PROFITS OR OTHER INCIDENTAL OR CONSEQUENTIAL DAMAGES RELATING
 * TO THE SOFTWARE.
 */

package COM.dtai.data;

import java.util.*;
import java.net.*;

/**
 * Data - interface for data management for prompts
 *
 * @see		COM.dtai.data.DataString
 * @see		COM.dtai.data.DataURLString
 * @see		COM.dtai.data.DataInteger
 * @see		COM.dtai.data.Prompt
 * @see		COM.dtai.data.ValueObserver
 * @version	1.0b1
 * @author	DTAI, Incorporated
 *
 * $Id: Data.java,v 1.2 1996/08/13 06:29:18 kadel Exp $
 *
 * $Source: /cvs/java/classes/COM/dtai/data/Data.java,v $
 */

public interface Data
{
	/**
	 * convert the Data of arbitrary type to String form, potentially to
	 * then put in a TextField.
	 *
	 * @return the string form
	 */
    public String toString();

	/**
	 * convert the string back into the appropriate data type stored in this
	 * object.
	 *
	 * @param str the string to convert from
	 * @exception Exception Check out the detail message, or test for instanceof
	 *              NumberFormatException if the Data type is a number, or
	 *              MalformedURLException if the Data type is a URL, or
	 *              IndexOutOfBoundsException if outside a min/max range.
	 */
    public void setFromString( String str )
	throws Exception;

	/**
	 * clears the Data to some default, or semantically "clear" value.  For example,
	 * a number to zero, or a string to the empty string.
	 */
    public void clear();

	/**
	 * Call this to notify ValueObservers that the value changed.
	 *
	 * @see ValueObserver
	 */
    public void valueChanged();

	/**
	 * This is called by objects like Prompt to accept the current value, or
	 * return false if there was a problem.
	 *
	 * @return true if value was accepted
	 * @see Prompt
	 */
    public boolean acceptChange();

	/**
	 * compares this Data object with another that should be of the same type
	 * (usually for ordering purposes).
	 *
	 * @param rhs the other Data object
	 * @return 0 if lhs == rhs, < 0 if lhs < rhs, or > 0 if lhs > rhs.
	 * @exception ClassCastException classes were not the same
	 */
    public int compareTo( Data rhs ) throws ClassCastException;

	/**
	 * add a ValueObserver to this object so that the observer will be notified
	 * when the value changes
	 *
	 * @param observer the object to be notified
	 * @param arg an object to be passed back to the observer with the notification
	 */
    public void addValueObserver( ValueObserver observer, Object arg );

	/**
	 * returns the maximum character width this object should normally take up if
	 * converted to text format
	 *
	 * @return the number of characters
	 */
    public int getMaxWidth();

	/**
	 * possibly different than max width, this is the number of characters the
	 * String value of this object might use in a column of a table.  (A NamedData
	 * object might return the largest of getMaxWidth() and the String length of
	 * its name.
	 *
	 * @return the maximum column width
	 * @see NamedData
	 */
    public int getColumnWidth();
}
