/**
 * Copyright(c) 1996 DTAI, Incorporated (http://www.dtai.com)
 *
 *                        All rights reserved
 *
 * Permission to use, copy, modify and distribute this material for
 * any purpose and without fee is hereby granted, provided that the
 * above copyright notice and this permission notice appear in all
 * copies, and that the name of DTAI, Incorporated not be used in
 * advertising or publicity pertaining to this material without the
 * specific, prior written permission of an authorized representative of
 * DTAI, Incorporated.
 *
 * DTAI, INCORPORATED MAKES NO REPRESENTATIONS AND EXTENDS NO WARRANTIES,
 * EXPRESS OR IMPLIED, WITH RESPECT TO THE SOFTWARE, INCLUDING, BUT
 * NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 * FITNESS FOR ANY PARTICULAR PURPOSE, AND THE WARRANTY AGAINST
 * INFRINGEMENT OF PATENTS OR OTHER INTELLECTUAL PROPERTY RIGHTS.  THE
 * SOFTWARE IS PROVIDED "AS IS", AND IN NO EVENT SHALL DTAI, INCORPORATED OR
 * ANY OF ITS AFFILIATES BE LIABLE FOR ANY DAMAGES, INCLUDING ANY
 * LOST PROFITS OR OTHER INCIDENTAL OR CONSEQUENTIAL DAMAGES RELATING
 * TO THE SOFTWARE.
 */

package COM.dtai.data;

import java.util.*;
import java.net.*;

/**
 * DataInteger - manages an integer for prompts
 *
 * @version	1.0b1
 * @author	DTAI, Incorporated
 *
 * $Id: DataInteger.java,v 1.2 1996/08/13 06:29:18 kadel Exp $
 *
 * $Source: /cvs/java/classes/COM/dtai/data/DataInteger.java,v $
 */

public class DataInteger extends DataSuper
{
    private int value;
    private int min = Integer.MIN_VALUE;
    private int max = Integer.MAX_VALUE;

	/**
	 * constructs a DataInteger with a max width of 10 and value zero
	 *
	 * @param PARAMNAME DESCRIPTION
	 * @return DESCRIPTION
	 * @exception CLASSNAME DESCRIPTION
	 * @see CLASSNAME#METHOD
	 */
    public DataInteger () {
		setMaxWidth( 10 );
		value = 0;
    }

	/**
	 * constructs a DataInteger with a maximum width of 10 and the given value
	 *
	 * @param PARAMNAME DESCRIPTION
	 * @return DESCRIPTION
	 * @exception CLASSNAME DESCRIPTION
	 * @see CLASSNAME#METHOD
	 */
    public DataInteger ( int value ) {
        setMaxWidth( 10 );
        this.value = value;
    }

	/**
	 * sets a minimum value boundary
	 *
	 * @param PARAMNAME DESCRIPTION
	 * @return DESCRIPTION
	 * @exception CLASSNAME DESCRIPTION
	 * @see CLASSNAME#METHOD
	 */
    public void setMin( int min ) {
        this.min = min;
    }

	/**
	 * sets a maximum value boundary
	 *
	 * @param PARAMNAME DESCRIPTION
	 * @return DESCRIPTION
	 * @exception CLASSNAME DESCRIPTION
	 * @see CLASSNAME#METHOD
	 */
    public void setMax( int max ) {
        this.max = max;
    }

	/**
	 * returns the value after converting it to a String
	 *
	 * @param PARAMNAME DESCRIPTION
	 * @return DESCRIPTION
	 * @exception CLASSNAME DESCRIPTION
	 * @see CLASSNAME#METHOD
	 */
    public String toString() {
        return ( Integer.toString( value ) );
    }

	/**
	 * sets the current value to zero
	 *
	 * @param PARAMNAME DESCRIPTION
	 * @return DESCRIPTION
	 * @exception CLASSNAME DESCRIPTION
	 * @see CLASSNAME#METHOD
	 */
    public void clear() {
        if ( ( 0 < min ) || ( 0 > max ) ) {
            setIntValue( min );
        }
        else {
            setIntValue( 0 );
        }
    }

	/**
	 * converts the given String to an integer and sets the value
	 *
	 * @param PARAMNAME DESCRIPTION
	 * @return DESCRIPTION
	 * @exception Exception Check out the detail message, or test for instanceof
	 *              NumberFormatException if the Data type is a number, or
	 *              MalformedURLException if the Data type is a URL, or
	 *              IndexOutOfBoundsException if outside a min/max range.
	 * @see CLASSNAME#METHOD
	 */
    public void setFromString( String str )
		throws NumberFormatException, IndexOutOfBoundsException {

        try {
            int newval = Integer.parseInt( str );
            if ( ( newval < min ) ||
                 ( newval > max ) ) {
                throw new IndexOutOfBoundsException( "Value out of range "+min+" to "+max );
            }
            value = newval;
            valueChanged();
        }
        catch ( NumberFormatException nfe ) {
            throw new NumberFormatException( "This value is not a valid Integer: "+str );
        }
    }

	/**
	 * compares this DataInteger to another DataInteger
	 *
	 * @param PARAMNAME DESCRIPTION
	 * @return DESCRIPTION
	 * @exception CLASSNAME DESCRIPTION
	 * @see CLASSNAME#METHOD
	 */
	public int compareTo( Data rhs ) throws ClassCastException {
        if ( value < ((DataInteger)rhs).value ) {
            return -1;
        }
        else if ( value > ((DataInteger)rhs).value ) {
            return 1;
        }
        else {
            return 0;
        }
    }

	/**
	 * returns the value as an integer
	 *
	 * @param PARAMNAME DESCRIPTION
	 * @return DESCRIPTION
	 * @exception CLASSNAME DESCRIPTION
	 * @see CLASSNAME#METHOD
	 */
    public int getIntValue () {
        return value;
    }

	/**
	 * sets the value to the given integer
	 *
	 * @param PARAMNAME DESCRIPTION
	 * @return DESCRIPTION
	 * @exception CLASSNAME DESCRIPTION
	 * @see CLASSNAME#METHOD
	 */
    public void setIntValue ( int value ) {
        this.value = value;
        valueChanged();
    }
}
