/**
 * Copyright(c) 1996 DTAI, Incorporated (http://www.dtai.com)
 *
 *                        All rights reserved
 *
 * Permission to use, copy, modify and distribute this material for
 * any purpose and without fee is hereby granted, provided that the
 * above copyright notice and this permission notice appear in all
 * copies, and that the name of DTAI, Incorporated not be used in
 * advertising or publicity pertaining to this material without the
 * specific, prior written permission of an authorized representative of
 * DTAI, Incorporated.
 *
 * DTAI, INCORPORATED MAKES NO REPRESENTATIONS AND EXTENDS NO WARRANTIES,
 * EXPRESS OR IMPLIED, WITH RESPECT TO THE SOFTWARE, INCLUDING, BUT
 * NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 * FITNESS FOR ANY PARTICULAR PURPOSE, AND THE WARRANTY AGAINST
 * INFRINGEMENT OF PATENTS OR OTHER INTELLECTUAL PROPERTY RIGHTS.  THE
 * SOFTWARE IS PROVIDED "AS IS", AND IN NO EVENT SHALL DTAI, INCORPORATED OR
 * ANY OF ITS AFFILIATES BE LIABLE FOR ANY DAMAGES, INCLUDING ANY
 * LOST PROFITS OR OTHER INCIDENTAL OR CONSEQUENTIAL DAMAGES RELATING
 * TO THE SOFTWARE.
 */

package COM.dtai.data;

import java.util.*;
import java.net.*;

/**
 * DataSuper - superclass for Data objects
 *
 * @see		COM.dtai.data.DataString
 * @see		COM.dtai.data.DataURLString
 * @see		COM.dtai.data.DataInteger
 * @see		COM.dtai.data.Prompt
 * @see		COM.dtai.data.ValueObserver
 * @version	1.0b1
 * @author	DTAI, Incorporated
 *
 * $Id: DataSuper.java,v 1.1 1996/07/29 21:12:38 kadel Exp $
 *
 * $Source: /cvs/java/classes/COM/dtai/data/DataSuper.java,v $
 */

public abstract class DataSuper implements Data
{
    private Vector valueObservers;
    private int max_width;

    protected DataSuper() {
    }

    protected void setMaxWidth( int max_width ) {
        this.max_width = max_width;
    }

	/**
	 * By default, calls setFromString( "" )
	 * 
	 * @param PARAMNAME DESCRIPTION
	 * @return DESCRIPTION
	 * @exception CLASSNAME DESCRIPTION
	 * @see CLASSNAME#METHOD
	 */
    public void clear() {
        try {
            setFromString( "" );
        }
        catch ( Exception exp ) {
        }
    }

	/**
	 * Loops through the ValueObservers and calls their valueChanged callbacks.
	 * 
	 * @param PARAMNAME DESCRIPTION
	 * @return DESCRIPTION
	 * @exception CLASSNAME DESCRIPTION
	 * @see CLASSNAME#METHOD
	 */
    public void valueChanged() {
        if ( valueObservers != null ) {
            for ( Enumeration e = valueObservers.elements(); e.hasMoreElements(); ) {
                ValueObserverAndArg oa = (ValueObserverAndArg)e.nextElement();
	            oa.observer.valueChanged( this, oa.arg );
            }
        }
    }

	/**
	 * Loops through the value observer's and asks them to try to save any
	 * current changes to the value, if possible.  For instance, if a TextField
	 * prompt has been changed for the value, but the most recent changes to
	 * the text in the field have not been stored in the value, this will attempt
	 * to get the ValueObserver associated with the text field to store them.
	 * 
	 * @param PARAMNAME DESCRIPTION
	 * @return DESCRIPTION
	 * @exception CLASSNAME DESCRIPTION
	 * @see CLASSNAME#METHOD
	 */
    public boolean acceptChange() {
        if ( valueObservers != null ) {
            for ( Enumeration e = valueObservers.elements(); e.hasMoreElements(); ) {
                ValueObserverAndArg oa = (ValueObserverAndArg)e.nextElement();
	            if ( ! oa.observer.acceptChange( this, oa.arg ) ) {
	                return false;
	            }
            }
        }
        return true;
    }

	/**
	 * Adds an object that implements ValueObserver to a list of observers to
	 * be notified when the Data object's value changes.
	 * 
	 * @param PARAMNAME DESCRIPTION
	 * @return DESCRIPTION
	 * @exception CLASSNAME DESCRIPTION
	 * @see CLASSNAME#METHOD
	 */
    public void addValueObserver( ValueObserver observer, Object arg ) {
        if ( valueObservers == null ) {
            valueObservers = new Vector();
        }
        valueObservers.addElement( new ValueObserverAndArg( observer, arg ) );
    }

	/**
	 * returns the maximum character width of the value in String form (for
	 * TextField prompts, for example)
	 * 
	 * @param PARAMNAME DESCRIPTION
	 * @return DESCRIPTION
	 * @exception CLASSNAME DESCRIPTION
	 * @see CLASSNAME#METHOD
	 */
    public int getMaxWidth() {
        if ( max_width > 0 ) {
            return max_width;
        }
        else {
            return 10;
        }
    }

	/**
	 * returns the maximum column width if the Data object were in a table
	 * 
	 * @param PARAMNAME DESCRIPTION
	 * @return DESCRIPTION
	 * @exception CLASSNAME DESCRIPTION
	 * @see CLASSNAME#METHOD
	 */
    public int getColumnWidth() {
        return getMaxWidth();
    }
}

class ValueObserverAndArg
{
    public ValueObserver observer;
    public Object arg;

    ValueObserverAndArg( ValueObserver observer, Object arg ) {
        this.observer = observer;
        this.arg = arg;
    }
}
