/**
 * Copyright(c) 1996 DTAI, Incorporated (http://www.dtai.com)
 *
 *                        All rights reserved
 *
 * Permission to use, copy, modify and distribute this material for
 * any purpose and without fee is hereby granted, provided that the
 * above copyright notice and this permission notice appear in all
 * copies, and that the name of DTAI, Incorporated not be used in
 * advertising or publicity pertaining to this material without the
 * specific, prior written permission of an authorized representative of
 * DTAI, Incorporated.
 *
 * DTAI, INCORPORATED MAKES NO REPRESENTATIONS AND EXTENDS NO WARRANTIES,
 * EXPRESS OR IMPLIED, WITH RESPECT TO THE SOFTWARE, INCLUDING, BUT
 * NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 * FITNESS FOR ANY PARTICULAR PURPOSE, AND THE WARRANTY AGAINST
 * INFRINGEMENT OF PATENTS OR OTHER INTELLECTUAL PROPERTY RIGHTS.  THE
 * SOFTWARE IS PROVIDED "AS IS", AND IN NO EVENT SHALL DTAI, INCORPORATED OR
 * ANY OF ITS AFFILIATES BE LIABLE FOR ANY DAMAGES, INCLUDING ANY
 * LOST PROFITS OR OTHER INCIDENTAL OR CONSEQUENTIAL DAMAGES RELATING
 * TO THE SOFTWARE.
 */

package COM.dtai.data;

import java.util.*;
import java.net.*;

/**
 * DataURLString - data management for a URL in string form
 *
 * @version	1.0b1
 * @author	DTAI, Incorporated
 *
 * $Id: DataURLString.java,v 1.3 1996/08/13 06:29:18 kadel Exp $
 *
 * $Source: /cvs/java/classes/COM/dtai/data/DataURLString.java,v $
 */

public class DataURLString extends DataSuper
{
    private String value;
    private String default_protocol;

	/**
	 * Constructs a DataURLString with the given maximum width
	 *
	 * @param PARAMNAME DESCRIPTION
	 * @return DESCRIPTION
	 * @exception CLASSNAME DESCRIPTION
	 * @see CLASSNAME#METHOD
	 */
    public DataURLString ( int max_width ) {
        setMaxWidth( max_width );
        value = "";
    }

	/**
	 * Constructs a DataURLString with the given default protocol (e.g., "http")
	 * and maximum width
	 *
	 * @param PARAMNAME DESCRIPTION
	 * @return DESCRIPTION
	 * @exception CLASSNAME DESCRIPTION
	 * @see CLASSNAME#METHOD
	 */
    public DataURLString ( String default_protocol, int max_width ) {
        setMaxWidth( max_width );
        value = "";

        try {
            URL testurl = new URL( default_protocol );
            value = default_protocol;
            this.default_protocol = testurl.getProtocol()+"://";
        }
        catch ( MalformedURLException mue ) {
            if ( default_protocol.equals( "mailto" ) ) {
                default_protocol = "mailto:";
            }
            if ( default_protocol.equals( "mailto:" ) ||
                 default_protocol.endsWith( "://" ) ) {
                this.default_protocol = default_protocol;
            }
            else if ( default_protocol.endsWith( ":" ) ) {
                this.default_protocol = default_protocol+"//";
            }
            else {
                this.default_protocol = default_protocol+"://";
            }
        }
    }

	/**
	 * Constructs a DataURLString with the given value and maximum width
	 *
	 * @param PARAMNAME DESCRIPTION
	 * @return DESCRIPTION
	 * @exception CLASSNAME DESCRIPTION
	 * @see CLASSNAME#METHOD
	 */
    public DataURLString ( URL value, int max_width ) {
        setMaxWidth( max_width );
        this.value = value.toString();
        default_protocol = value.getProtocol();
    }

	/**
	 * attempts to set the URL value by converting the given string
	 *
	 * @param PARAMNAME DESCRIPTION
	 * @return DESCRIPTION
	 * @exception Exception Check out the detail message, or test for instanceof
	 *              NumberFormatException if the Data type is a number, or
	 *              MalformedURLException if the Data type is a URL
	 * @see CLASSNAME#METHOD
	 */
    public void setFromString( String str )
        throws MalformedURLException {

        try {
            // test the URL...an exception may be thrown
            if ( ! ( str.startsWith( "mailto:" ) ||
                     default_protocol.equals( "mailto:" ) ) ) {
                if ( str.indexOf( "://" ) == -1 ) {
                    new URL( default_protocol+str );
                }
                else {
                    new URL( str );
                }
            }
            value = new String ( str );
            valueChanged();
        }
        catch ( MalformedURLException mue ) {
            throw new MalformedURLException ( "Invalid or improperly formatted URL" );
        }
    }

	/**
	 * returns the current URL value
	 *
	 * @param PARAMNAME DESCRIPTION
	 * @return DESCRIPTION
	 * @exception CLASSNAME DESCRIPTION
	 * @see CLASSNAME#METHOD
	 */
    public URL getURLValue() {
        URL url = null;
        try {
            if ( ! ( value.startsWith( "mailto:" ) ||
                     ( value.indexOf( "://" ) != -1 ) ) ) {
                url = new URL( default_protocol+value );
                String str = url.toString();
                if ( str.trim().equals( "" ) ||
                     str.endsWith( ":/" ) ||
					 str.endsWith( "://" ) ) {
					return null;
				}
            }
            else {
                url = new URL( value );
            }
        }
        catch ( MalformedURLException mue ) {
        }
        return url;
    }

	/**
	 * sets the URL value to the given URL
	 *
	 * @param PARAMNAME DESCRIPTION
	 * @return DESCRIPTION
	 * @exception CLASSNAME DESCRIPTION
	 * @see CLASSNAME#METHOD
	 */
    public void setURLValue( URL url ) {
        if ( url == null ) {
            value = "";
        }
        else {
            if ( url.getProtocol().equals( "mailto" ) ) {
                value = url.toString().substring( 7 );
            }
            else {
                value = url.toString();
                if ( value.equals( "http:/" ) ) {
                    value = "";
                }
            }
        }
        valueChanged();
    }

	/**
	 * returns the String value of the URL
	 *
	 * @param PARAMNAME DESCRIPTION
	 * @return DESCRIPTION
	 * @exception CLASSNAME DESCRIPTION
	 * @see CLASSNAME#METHOD
	 */
    public String toString() {
        return ( value );
    }

	/**
	 * compares this DataURLString to the given one for sort order
	 *
	 * @param PARAMNAME DESCRIPTION
	 * @return DESCRIPTION
	 * @exception CLASSNAME DESCRIPTION
	 * @see CLASSNAME#METHOD
	 */
    public int compareTo( Data rhs ) throws ClassCastException {
        return value.compareTo( ((DataURLString)rhs).value );
    }
}
