/**
 * Copyright(c) 1996 DTAI, Incorporated (http://www.dtai.com)
 *
 *                        All rights reserved
 *
 * Permission to use, copy, modify and distribute this material for
 * any purpose and without fee is hereby granted, provided that the
 * above copyright notice and this permission notice appear in all
 * copies, and that the name of DTAI, Incorporated not be used in
 * advertising or publicity pertaining to this material without the
 * specific, prior written permission of an authorized representative of
 * DTAI, Incorporated.
 *
 * DTAI, INCORPORATED MAKES NO REPRESENTATIONS AND EXTENDS NO WARRANTIES,
 * EXPRESS OR IMPLIED, WITH RESPECT TO THE SOFTWARE, INCLUDING, BUT
 * NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 * FITNESS FOR ANY PARTICULAR PURPOSE, AND THE WARRANTY AGAINST
 * INFRINGEMENT OF PATENTS OR OTHER INTELLECTUAL PROPERTY RIGHTS.  THE
 * SOFTWARE IS PROVIDED "AS IS", AND IN NO EVENT SHALL DTAI, INCORPORATED OR
 * ANY OF ITS AFFILIATES BE LIABLE FOR ANY DAMAGES, INCLUDING ANY
 * LOST PROFITS OR OTHER INCIDENTAL OR CONSEQUENTIAL DAMAGES RELATING
 * TO THE SOFTWARE.
 */

package COM.dtai.data;

import java.net.*;

/**
 * NamedData - adds a name to the data managed by the Data interface
 *
 * @version	1.0b1
 * @author	DTAI, Incorporated
 *
 * $Id: NamedData.java,v 1.2 1996/08/13 06:29:19 kadel Exp $
 *
 * $Source: /cvs/java/classes/COM/dtai/data/NamedData.java,v $
 */

public class NamedData implements Data
{
    String name;
    Data data;

	/**
	 * Constructs a NamedData from a String name and a Data object
	 * 
	 * @param PARAMNAME DESCRIPTION
	 * @return DESCRIPTION
	 * @exception CLASSNAME DESCRIPTION
	 * @see CLASSNAME#METHOD
	 */
    public NamedData ( String name, Data data ) {
        this.name = name;
        this.data = data;
    }

	/**
	 * returns the Data object associated with this NamedData object
	 * 
	 * @param PARAMNAME DESCRIPTION
	 * @return DESCRIPTION
	 * @exception CLASSNAME DESCRIPTION
	 * @see CLASSNAME#METHOD
	 */
    public Data getBasicDataObject() {
        return data;
    }

	/**
	 * returns the name of this object
	 * 
	 * @param PARAMNAME DESCRIPTION
	 * @return DESCRIPTION
	 * @exception CLASSNAME DESCRIPTION
	 * @see CLASSNAME#METHOD
	 */
    public String getName() {
        return name;
    }

	/**
	 * Converts the Data value to a String
	 * 
	 * @param PARAMNAME DESCRIPTION
	 * @return DESCRIPTION
	 * @exception CLASSNAME DESCRIPTION
	 * @see CLASSNAME#METHOD
	 */
    public String toString() {
        return data.toString();
    }

	/**
	 * Semantically "clears" the Data value
	 * 
	 * @param PARAMNAME DESCRIPTION
	 * @return DESCRIPTION
	 * @exception CLASSNAME DESCRIPTION
	 * @see CLASSNAME#METHOD
	 */
    public void clear() {
        data.clear();
    }

	/**
	 * Attempts to set the value of the Data object by converting the given String
	 * 
	 * @param PARAMNAME DESCRIPTION
	 * @return DESCRIPTION
	 * @exception CLASSNAME DESCRIPTION
	 * @see CLASSNAME#METHOD
	 */
    public void setFromString( String str )
        throws Exception {
        data.setFromString( str );
    }

	/**
	 * Calls the Data object's valueChange() function
	 * 
	 * @param PARAMNAME DESCRIPTION
	 * @return DESCRIPTION
	 * @exception CLASSNAME DESCRIPTION
	 * @see CLASSNAME#METHOD
	 */
    public void valueChanged() {
        data.valueChanged();
    }

	/**
	 * Calls the Data object's acceptChange() function
	 * 
	 * @param PARAMNAME DESCRIPTION
	 * @return DESCRIPTION
	 * @exception CLASSNAME DESCRIPTION
	 * @see CLASSNAME#METHOD
	 */
    public boolean acceptChange() {
        return data.acceptChange();
    }

	/**
	 * compares the Data object to the given Data object (Both should be of the
	 * same type).
	 * 
	 * @param PARAMNAME DESCRIPTION
	 * @return DESCRIPTION
	 * @exception CLASSNAME DESCRIPTION
	 * @see CLASSNAME#METHOD
	 */
    public int compareTo( Data rhs ) throws ClassCastException {
        return data.compareTo( rhs );
    }

	/**
	 * adds a ValueObserver to the Data object
	 * 
	 * @param PARAMNAME DESCRIPTION
	 * @return DESCRIPTION
	 * @exception CLASSNAME DESCRIPTION
	 * @see CLASSNAME#METHOD
	 */
    public void addValueObserver( ValueObserver observer, Object arg ) {
        data.addValueObserver( observer, arg );
    }

	/**
	 * Returns the maximum width of the data value in string form
	 * 
	 * @param PARAMNAME DESCRIPTION
	 * @return DESCRIPTION
	 * @exception CLASSNAME DESCRIPTION
	 * @see CLASSNAME#METHOD
	 */
    public int getMaxWidth() {
        return data.getMaxWidth();
    }

	/**
	 * Returns the maximum width of the Data in text form if a column in a table.
	 * In the case of this NamedData class, this is the larger of getMaxWidth()
	 * and the length of the name.
	 * 
	 * @param PARAMNAME DESCRIPTION
	 * @return DESCRIPTION
	 * @exception CLASSNAME DESCRIPTION
	 * @see CLASSNAME#METHOD
	 */
	public int getColumnWidth() {
        return ( Math.max( getMaxWidth(), name.length() ) );
    }
}
