/**
 * Copyright(c) 1996 DTAI, Incorporated (http://www.dtai.com)
 *
 *                        All rights reserved
 *
 * Permission to use, copy, modify and distribute this material for
 * any purpose and without fee is hereby granted, provided that the
 * above copyright notice and this permission notice appear in all
 * copies, and that the name of DTAI, Incorporated not be used in
 * advertising or publicity pertaining to this material without the
 * specific, prior written permission of an authorized representative of
 * DTAI, Incorporated.
 *
 * DTAI, INCORPORATED MAKES NO REPRESENTATIONS AND EXTENDS NO WARRANTIES,
 * EXPRESS OR IMPLIED, WITH RESPECT TO THE SOFTWARE, INCLUDING, BUT
 * NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 * FITNESS FOR ANY PARTICULAR PURPOSE, AND THE WARRANTY AGAINST
 * INFRINGEMENT OF PATENTS OR OTHER INTELLECTUAL PROPERTY RIGHTS.  THE
 * SOFTWARE IS PROVIDED "AS IS", AND IN NO EVENT SHALL DTAI, INCORPORATED OR
 * ANY OF ITS AFFILIATES BE LIABLE FOR ANY DAMAGES, INCLUDING ANY
 * LOST PROFITS OR OTHER INCIDENTAL OR CONSEQUENTIAL DAMAGES RELATING
 * TO THE SOFTWARE.
 */

package COM.dtai.data;

import java.awt.*;
import java.util.*;
import java.net.*;
import COM.dtai.util.*;

/**
 * TextPrompt - the typical prompt.  A label and a button.
 *
 * @see		COM.dtai.data.Data
 * @see		COM.dtai.data.NamedData
 * @see		COM.dtai.data.Prompt
 * @version	1.0b1
 * @author	DTAI, Incorporated
 *
 * $Id: TextPrompt.java,v 1.2 1996/08/13 06:29:19 kadel Exp $
 *
 * $Source: /cvs/java/classes/COM/dtai/data/TextPrompt.java,v $
 */

public class TextPrompt extends Observable implements Prompt {
    private NamedData data;
    private Component label;
    private Text text;

	/**
	 * Constructs a TextPrompt object from the name and Data value of the given
	 * NamedData object.
	 *
	 * @param PARAMNAME DESCRIPTION
	 * @return DESCRIPTION
	 * @exception CLASSNAME DESCRIPTION
	 * @see CLASSNAME#METHOD
	 */
    public TextPrompt( NamedData data ) {
        this.data = data;
    }

	/**
	 * Returns true if the text in the TextField has been modified, but not
	 * saved in the Data object yet.
	 *
	 * @param PARAMNAME DESCRIPTION
	 * @return DESCRIPTION
	 * @exception CLASSNAME DESCRIPTION
	 * @see CLASSNAME#METHOD
	 */
    public boolean isModified() {
        if ( text == null ) {
            return false;
        }
        else {
            return ( ! text.getText().equals( data.toString() ) );
        }
    }

	/**
	 * Adds an observer that wants to be notified if the label is clicked.  This
	 * makes the the label a Button, versus a Label object.
	 *
	 * @param PARAMNAME DESCRIPTION
	 * @return DESCRIPTION
	 * @exception CLASSNAME DESCRIPTION
	 * @see CLASSNAME#METHOD
	 */
    public void notifyOnButtonClick( Observer observer ) {
        addObserver( observer );
    }

	/**
	 * Returns the NamedData object associated with this TextPrompt
	 *
	 * @param PARAMNAME DESCRIPTION
	 * @return DESCRIPTION
	 * @exception CLASSNAME DESCRIPTION
	 * @see CLASSNAME#METHOD
	 */
    public NamedData getNamedData() {
        return data;
    }

	/**
	 * Returns the Data object associated with this TextPrompt
	 *
	 * @param PARAMNAME DESCRIPTION
	 * @return DESCRIPTION
	 * @exception CLASSNAME DESCRIPTION
	 * @see CLASSNAME#METHOD
	 */
    public Data getData() {
        return data.getBasicDataObject();
    }

	/**
	 * Returns the name label
	 *
	 * @param PARAMNAME DESCRIPTION
	 * @return DESCRIPTION
	 * @exception CLASSNAME DESCRIPTION
	 * @see CLASSNAME#METHOD
	 */
    public Component getLeftComponent() {
        if ( label == null ) {
            String name = data.getName();
            if ( countObservers() == 0 ) {
                if ( name.charAt(0) != ' ' ) {
    	            name = " " + name; // Correct for Netscape font problems
                }
                label = new Label( name, Label.RIGHT );
            }
            else {
                label = new TextPromptButton( name, this );
            }
        }
        return ( label );
    }

	/**
	 * Returns the TextField
	 *
	 * @param PARAMNAME DESCRIPTION
	 * @return DESCRIPTION
	 * @exception CLASSNAME DESCRIPTION
	 * @see CLASSNAME#METHOD
	 */
    public Component getRightComponent() {
        if ( text == null ) {
            text = new Text( this, data );
        }
        return ( text );
    }

    protected boolean buttonPressed( TextPromptButton button ) {

        if ( button == label ) {
            setChanged();
            notifyObservers( this );
            return true;
        }
        return false;
    }

	/**
	 * Attempts to set the value of the Data object to the given String value.
	 * Returns false if there was a problem.
	 *
	 * @param PARAMNAME DESCRIPTION
	 * @return DESCRIPTION
	 * @exception CLASSNAME DESCRIPTION
	 * @see CLASSNAME#METHOD
	 */
    public boolean acceptChanges() {
        try {
            data.setFromString( text.getText() );
            return true;
        }
        catch ( Exception exp ) {
            String message = exp.getMessage();
            if ( message.equals( "" ) ) {
                if ( exp instanceof NumberFormatException ) {
                    message = "Bad number: "+text.getText();
                }
                if ( exp instanceof IndexOutOfBoundsException ) {
                    message = "Value out of range.";
                }
                if ( exp instanceof MalformedURLException ) {
                    message = "Bad URL: "+text.getText();
                }
                else {
                    message = "Bad value.";
                }
            }
            ShowUser.error( "Input error for "+data.getName()+".\n"+message );
        }
        text.selectAll();
        return false;
    }
}

class TextPromptButton extends Button {

    TextPrompt prompt;

    protected TextPromptButton( String name, TextPrompt prompt ) {
        super( name );
        this.prompt = prompt;
    }

    public boolean action( Event evt, Object arg ) {
        return prompt.buttonPressed( this );
    }
}

class Text extends TextField implements ValueObserver
{
    TextPrompt prompt;
    Data data;

    public Text( TextPrompt prompt, Data data ) {
        super( data.getMaxWidth() );
        this.prompt = prompt;
        this.data = data;
        data.addValueObserver( this, null );
        setText( data.toString() );
    }

    public boolean action( Event evt, Object arg ) {
        prompt.acceptChanges();
        return true;
    }

    public boolean handleEvent( Event evt ) {
        // have to do this to avoid the beeping when the text field doesn't
        // accept the tab keys!
        if ( evt.key == '\t' ) {
            Container parent = getParent();
            while ( ( parent != null ) && ( ! parent.handleEvent( evt ) ) ) {
                parent = parent.getParent();
            }
            return true;
        }
        return super.handleEvent( evt );
    }

    public void valueChanged( Data data, Object arg ) {
        setText( data.toString() );
    }

    public boolean acceptChange( Data data, Object arg ) {
        return prompt.acceptChanges();
    }
}
