/**
 * Copyright(c) 1996 DTAI, Incorporated (http://www.dtai.com)
 *
 *                        All rights reserved
 *
 * Permission to use, copy, modify and distribute this material for
 * any purpose and without fee is hereby granted, provided that the
 * above copyright notice and this permission notice appear in all
 * copies, and that the name of DTAI, Incorporated not be used in
 * advertising or publicity pertaining to this material without the
 * specific, prior written permission of an authorized representative of
 * DTAI, Incorporated.
 *
 * DTAI, INCORPORATED MAKES NO REPRESENTATIONS AND EXTENDS NO WARRANTIES,
 * EXPRESS OR IMPLIED, WITH RESPECT TO THE SOFTWARE, INCLUDING, BUT
 * NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 * FITNESS FOR ANY PARTICULAR PURPOSE, AND THE WARRANTY AGAINST
 * INFRINGEMENT OF PATENTS OR OTHER INTELLECTUAL PROPERTY RIGHTS.  THE
 * SOFTWARE IS PROVIDED "AS IS", AND IN NO EVENT SHALL DTAI, INCORPORATED OR
 * ANY OF ITS AFFILIATES BE LIABLE FOR ANY DAMAGES, INCLUDING ANY
 * LOST PROFITS OR OTHER INCIDENTAL OR CONSEQUENTIAL DAMAGES RELATING
 * TO THE SOFTWARE.
 */

package COM.dtai.diagram;

import java.awt.*;

/**
 * NodeBackground - Superclass for a variety of background styles that can
 * be used in network and organizational chart diagrams
 *
 * @version	1.0b1
 * @author	DTAI, Incorporated
 *
 * $Id: NodeBackground.java,v 1.3 1996/08/14 05:48:55 kadel Exp $
 *
 * $Source: /cvs/java/classes/COM/dtai/diagram/NodeBackground.java,v $
 */

public abstract class NodeBackground {

	/**
	 * returns the left, right, top, and bottom insets of the background of the
	 * current style.
	 *
	 * @param PARAMNAME DESCRIPTION
	 * @return DESCRIPTION
	 * @exception CLASSNAME DESCRIPTION
	 * @see CLASSNAME#METHOD
	 */
    public abstract Insets insets();

	/**
	 * Draws the node background in the given style
	 *
	 * @param PARAMNAME DESCRIPTION
	 * @return DESCRIPTION
	 * @exception CLASSNAME DESCRIPTION
	 * @see CLASSNAME#METHOD
	 */
    public abstract void draw( Graphics g, Point location, Dimension size,
                                  boolean hilited, boolean disabled,
                                  boolean armed, boolean marked );

	/**
	 * Draws the node background in the given style
	 *
	 * @param PARAMNAME DESCRIPTION
	 * @return a graphics object with which parts of the arm color can be
	 * erased
	 * @exception CLASSNAME DESCRIPTION
	 * @see CLASSNAME#METHOD
	 */
    public abstract Graphics arm( Graphics g, Point location, Dimension size,
                                  boolean hilited );

	/**
	 * Draws the node background in the given style
	 *
	 * @param PARAMNAME DESCRIPTION
	 * @return a graphics object with which parts of the arm color can be
	 * erased
	 * @exception CLASSNAME DESCRIPTION
	 * @see CLASSNAME#METHOD
	 */
    public abstract Graphics disarm( Graphics g, Point location, Dimension size,
                                  boolean hilited );
}
