/**
 * Copyright(c) 1996 DTAI, Incorporated (http://www.dtai.com)
 *
 *                        All rights reserved
 *
 * Permission to use, copy, modify and distribute this material for
 * any purpose and without fee is hereby granted, provided that the
 * above copyright notice and this permission notice appear in all
 * copies, and that the name of DTAI, Incorporated not be used in
 * advertising or publicity pertaining to this material without the
 * specific, prior written permission of an authorized representative of
 * DTAI, Incorporated.
 *
 * DTAI, INCORPORATED MAKES NO REPRESENTATIONS AND EXTENDS NO WARRANTIES,
 * EXPRESS OR IMPLIED, WITH RESPECT TO THE SOFTWARE, INCLUDING, BUT
 * NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 * FITNESS FOR ANY PARTICULAR PURPOSE, AND THE WARRANTY AGAINST
 * INFRINGEMENT OF PATENTS OR OTHER INTELLECTUAL PROPERTY RIGHTS.  THE
 * SOFTWARE IS PROVIDED "AS IS", AND IN NO EVENT SHALL DTAI, INCORPORATED OR
 * ANY OF ITS AFFILIATES BE LIABLE FOR ANY DAMAGES, INCLUDING ANY
 * LOST PROFITS OR OTHER INCIDENTAL OR CONSEQUENTIAL DAMAGES RELATING
 * TO THE SOFTWARE.
 */

package COM.dtai.gui;

import java.awt.*;

/**
 * DrawLine - draws a line with varying thickness and dashes if selected.
 *
 * @version	1.0b1
 * @author	DTAI, Incorporated
 *
 * $Id: DrawLine.java,v 1.2 1996/08/13 06:29:21 kadel Exp $
 *
 * $Source: /cvs/java/classes/COM/dtai/gui/DrawLine.java,v $
 */

public class DrawLine {

    private Point from = new Point(0,0);
    private Point to = new Point(0,0);
    private Color color = Color.black;
    private int thickness = 1;
    private int dash_on_len = 1;
    private int dash_off_len = 0;

    static final int click_tolerance = 5;
    static final double radians_to_degrees = (double)180.0 / Math.PI;

	/**
	 * DESCRIPTION
	 *
	 * @param PARAMNAME DESCRIPTION
	 * @return DESCRIPTION
	 * @exception CLASSNAME DESCRIPTION
	 * @see CLASSNAME#METHOD
	 */
    public DrawLine() {
    }

	/**
	 * DESCRIPTION
	 *
	 * @param PARAMNAME DESCRIPTION
	 * @return DESCRIPTION
	 * @exception CLASSNAME DESCRIPTION
	 * @see CLASSNAME#METHOD
	 */
    public void setFrom( int x, int y ) {
        from.x = x;
        from.y = y;
    }

	/**
	 * DESCRIPTION
	 *
	 * @param PARAMNAME DESCRIPTION
	 * @return DESCRIPTION
	 * @exception CLASSNAME DESCRIPTION
	 * @see CLASSNAME#METHOD
	 */
    public void setTo( int x, int y ) {
        to.x = x;
        to.y = y;
    }

	/**
	 * DESCRIPTION
	 *
	 * @param PARAMNAME DESCRIPTION
	 * @return DESCRIPTION
	 * @exception CLASSNAME DESCRIPTION
	 * @see CLASSNAME#METHOD
	 */
    public void setCoords( int from_x, int from_y, int to_x, int to_y ){
        from.x = from_x;
        from.y = from_y;
        to.x = to_x;
        to.y = to_y;
    }

	/**
	 * DESCRIPTION
	 *
	 * @param PARAMNAME DESCRIPTION
	 * @return DESCRIPTION
	 * @exception CLASSNAME DESCRIPTION
	 * @see CLASSNAME#METHOD
	 */
    public void setColor( Color color ) {
        this.color = color;
    }

	/**
	 * DESCRIPTION
	 *
	 * @param PARAMNAME DESCRIPTION
	 * @return DESCRIPTION
	 * @exception CLASSNAME DESCRIPTION
	 * @see CLASSNAME#METHOD
	 */
    public void setThickness( int thickness ) {
        this.thickness = thickness;
    }

    /**
     * This is ignored if dash_off_len == 0.  Otherwise, it's the
     * approximate number of pixels in length to draw a dash in the line.
	 *
	 * @param PARAMNAME DESCRIPTION
	 * @return DESCRIPTION
	 * @exception CLASSNAME DESCRIPTION
	 * @see CLASSNAME#METHOD
	 */
    public void setDashOnLen( int len ) {
        dash_on_len = len;
    }

    /**
     * The line is solid if dash_off_len == 0.  Otherwise, this is the
     * approximate number of pixels in length to skip between dashes.
	 *
	 * @param PARAMNAME DESCRIPTION
	 * @return DESCRIPTION
	 * @exception CLASSNAME DESCRIPTION
	 * @see CLASSNAME#METHOD
	 */
    public void setDashOffLen( int len ) {
        dash_off_len = len;
    }

	/**
	 * DESCRIPTION
	 *
	 * @param PARAMNAME DESCRIPTION
	 * @return DESCRIPTION
	 * @exception CLASSNAME DESCRIPTION
	 * @see CLASSNAME#METHOD
	 */
    public void draw( Graphics g ) {

        if ( dash_on_len == 0 ) {
            return;
        }

        g.setColor( color );

        double roundoff = 0.0;

        double theta = Math.atan2( (double)( to.y - from.y ),
                                              (double)( to.x - from.x ) );

        int last_x_offset = 0;
        int last_y_offset = 0;
        int last2_x_offset = 0;
        int last2_y_offset = 0;

        drawLine( g, from.x, from.y,
                     to.x, to.y );

        for ( int idx = 1; idx < ( thickness + 1 ); idx++ ) {
            double delta = idx / 2;
            int dir = ( ( idx % 2 ) == 0 ) ? 1 : -1;

            double sintheta = Math.sin( theta );
            double costheta = Math.cos( theta );

            if ( sintheta > 0 ) {
                roundoff = 0.5;
            }
            else {
                roundoff = -0.5;
            }

            int x_offset = -(int)( roundoff +
                                  ( sintheta * delta * dir ) );

            if ( costheta > 0 ) {
                roundoff = 0.5;
            }
            else {
                roundoff = -0.5;
            }

            int y_offset = (int)( roundoff +
                                  ( costheta * delta * dir ) );

            if ( ( idx > 1 ) &&
                 ( x_offset != last2_x_offset ) &&
                 ( y_offset != last2_y_offset ) ) {
                drawLine( g, from.x + last2_x_offset, from.y + y_offset,
                             to.x + last2_x_offset, to.y + y_offset );
            }

            drawLine( g, from.x + x_offset, from.y + y_offset,
                         to.x + x_offset, to.y + y_offset );

            last2_x_offset = last_x_offset;
            last2_y_offset = last_y_offset;
            last_x_offset = x_offset;
            last_y_offset = y_offset;
        }
    }

	/**
	 * DESCRIPTION
	 *
	 * @param PARAMNAME DESCRIPTION
	 * @return DESCRIPTION
	 * @exception CLASSNAME DESCRIPTION
	 * @see CLASSNAME#METHOD
	 */
    public void drawLine( Graphics g, int from_x, int from_y, int to_x, int to_y ) {
        if ( dash_off_len <= 0 ) {
            g.drawLine( from_x, from_y, to_x, to_y );
        }
        else {
            // THIS IS ALGORITHM FOR DRAWING A DASHED LINE
            int xdist = to_x - from_x;
            int ydist = to_y - from_y;
            double len = Math.sqrt( (double)( ( xdist*xdist ) + ( ydist*ydist ) ) );
            double xnorm = xdist / len;
            double ynorm = ydist / len;
            int dashlen = dash_on_len + dash_off_len;
            int numdashes = (int)( len/dashlen );
            int prevx = from_x;
            int prevy = from_y;
            for ( int idx = 1; idx <= ( numdashes + 1 ); idx++ ) {
                int nextx = (int)( prevx + ( xnorm * dash_on_len ) );
                int nexty = (int)( prevy + ( ynorm * dash_on_len ) );
                if ( Math.abs( nextx - from_x ) > Math.abs( xdist ) ) {
                    nextx = to_x;
                }
                if ( Math.abs( nexty - from_y ) > Math.abs( ydist ) ) {
                    nexty = to_y;
                }
                g.drawLine( prevx, prevy, nextx, nexty );
                prevx = (int)( from_x + ( ( idx * dashlen ) * xnorm ) );
                prevy = (int)( from_y + ( ( idx * dashlen ) * ynorm ) );
            }
        }
    }

	/**
	 * DESCRIPTION
	 *
	 * @param PARAMNAME DESCRIPTION
	 * @return DESCRIPTION
	 * @exception CLASSNAME DESCRIPTION
	 * @see CLASSNAME#METHOD
	 */
    public boolean contains( int x, int y ) {

        int tolerance = Math.max( click_tolerance, thickness );

        //normalize to from
        x -= from.x;
        y -= from.y;
        int to_x = to.x - from.x;
        int to_y = to.y - from.y;

        if ( to_x < 0 ) {
            to_x *= -1;
            x *= -1;
        }

        if ( to_y < 0 ) {
            to_y *= -1;
            y *= -1;
        }

        if ( ( x < ( 0 - tolerance ) ) ||
             ( y < ( 0 - tolerance ) ) ||
             ( x > ( to_x + tolerance ) ) ||
             ( y > ( to_y + tolerance ) ) ) {
            return false;
        }

        if ( to_x == 0 ) {
            if ( ( x < ( - tolerance ) ) ||
                 ( x > tolerance ) ) {
                return false;
            }
            return true;
        }

        float slope = (float)to_y/(float)to_x;

        float targ_y = slope * x;

        if ( ( y >= ( targ_y - tolerance ) ) &&
             ( y <= ( targ_y + tolerance ) ) ) {
            return true;
        }

        float inv_slope = (float)to_x/(float)to_y;

        float targ_x = inv_slope * y;

        if ( ( x >= ( targ_x - tolerance ) ) &&
             ( x <= ( targ_x + tolerance ) ) ) {
            return true;
        }
        return false;
    }
}
