/**
 * Copyright(c) 1996 DTAI, Incorporated (http://www.dtai.com)
 *
 *                        All rights reserved
 *
 * Permission to use, copy, modify and distribute this material for
 * any purpose and without fee is hereby granted, provided that the
 * above copyright notice and this permission notice appear in all
 * copies, and that the name of DTAI, Incorporated not be used in
 * advertising or publicity pertaining to this material without the
 * specific, prior written permission of an authorized representative of
 * DTAI, Incorporated.
 *
 * DTAI, INCORPORATED MAKES NO REPRESENTATIONS AND EXTENDS NO WARRANTIES,
 * EXPRESS OR IMPLIED, WITH RESPECT TO THE SOFTWARE, INCLUDING, BUT
 * NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 * FITNESS FOR ANY PARTICULAR PURPOSE, AND THE WARRANTY AGAINST
 * INFRINGEMENT OF PATENTS OR OTHER INTELLECTUAL PROPERTY RIGHTS.  THE
 * SOFTWARE IS PROVIDED "AS IS", AND IN NO EVENT SHALL DTAI, INCORPORATED OR
 * ANY OF ITS AFFILIATES BE LIABLE FOR ANY DAMAGES, INCLUDING ANY
 * LOST PROFITS OR OTHER INCIDENTAL OR CONSEQUENTIAL DAMAGES RELATING
 * TO THE SOFTWARE.
 */

package COM.dtai.gui;

import java.awt.*;
import java.util.*;

/**
 * DrawText - draws text in some font and color
 *
 * @version	1.0b1
 * @author	DTAI, Incorporated
 *
 * $Id: DrawText.java,v 1.2 1996/08/13 06:29:21 kadel Exp $
 *
 * $Source: /cvs/java/classes/COM/dtai/gui/DrawText.java,v $
 */

public class DrawText {

    private Color color = Color.black;
    private Font font = new Font( "Helvetica", Font.PLAIN, 18 );
    private String text;
    private String wrapped_text;
    private int wrapat = 40;
    private int numlines = 0;

    public static final int LEFT = 0;
    public static final int CENTER = 1;
    public static final int RIGHT = 2;

    private int justification = DrawText.LEFT;

	/**
	 * DESCRIPTION
	 *
	 * @param PARAMNAME DESCRIPTION
	 * @return DESCRIPTION
	 * @exception CLASSNAME DESCRIPTION
	 * @see CLASSNAME#METHOD
	 */
    public DrawText() {
        text = "";
    }

	/**
	 * DESCRIPTION
	 *
	 * @param PARAMNAME DESCRIPTION
	 * @return DESCRIPTION
	 * @exception CLASSNAME DESCRIPTION
	 * @see CLASSNAME#METHOD
	 */
    public DrawText( String text ) {
        this.text = text;
        wrap();
    }

    private void wrap () {

        int col;

        StringBuffer buf = new StringBuffer();

        if ( text.length() == 0 ) {
            numlines = 0;
        }
        else {
            numlines = 1;
        }

        int oldidx = 0;

        int lastword = 0;
        boolean newline_inserted = true;
        boolean inspace = false;

        while ( oldidx < text.length() ) {
            oldidx = lastword;

            for ( col = 1;
                  ( ( oldidx ) < text.length() ) && ( col <= ( wrapat + 1 ) );
    	          col++ ) {

                if ( Character.isSpace( text.charAt( oldidx ) ) ) {
            	    if ( ! inspace ) {
                        if ( ! newline_inserted ) {
                            buf.append( '\n' );
                            numlines++;
                            newline_inserted = true;
                        }
                        while ( lastword < oldidx ) {
                            buf.append( text.charAt( lastword ) );
                            lastword++;
                        }
                        inspace = true;
                	}

                	if ( text.charAt( oldidx ) == '\n' ) {
                        buf.append( '\n' );
                        numlines++;
                        lastword++;
                        inspace = false;
                        col = 0;
                	}
                }
                else {
                    inspace = false;
                }

                oldidx++;
            }

            if ( oldidx < text.length() ) {
                while ( ( lastword < text.length() ) &&
                        ( Character.isSpace( text.charAt( lastword ) ) ) ) {
                	lastword++;
                }

                if ( ( oldidx - lastword ) >= wrapat ) {

                    // break in a long word
                    /////////////////////////

                    if ( ! newline_inserted ) {
                        buf.append( '\n' );
                        numlines++;
                        newline_inserted = true;
                    }

                    while ( lastword < oldidx ) {
                        buf.append( text.charAt( lastword ) );
                        lastword++;
                    }

                    while ( Character.isSpace( text.charAt( lastword ) ) ) {
                        lastword++;
                    }
                }

                inspace = false;
                newline_inserted = false;
            }
        }

        if ( lastword < oldidx ) {

            if ( ! newline_inserted ) {
                buf.append( '\n' );
                numlines++;
                newline_inserted = true;
            }

            while ( lastword < oldidx ) {
                buf.append( text.charAt( lastword ) );
                lastword++;
            }
        }

        wrapped_text = buf.toString();
    }

	/**
	 * DESCRIPTION
	 *
	 * @param PARAMNAME DESCRIPTION
	 * @return DESCRIPTION
	 * @exception CLASSNAME DESCRIPTION
	 * @see CLASSNAME#METHOD
	 */
    public void setJustification( int justification ) {
        this.justification = justification;
    }

	/**
	 * DESCRIPTION
	 *
	 * @param PARAMNAME DESCRIPTION
	 * @return DESCRIPTION
	 * @exception CLASSNAME DESCRIPTION
	 * @see CLASSNAME#METHOD
	 */
    public int getJustification() {
        return justification;
    }

	/**
	 * DESCRIPTION
	 *
	 * @param PARAMNAME DESCRIPTION
	 * @return DESCRIPTION
	 * @exception CLASSNAME DESCRIPTION
	 * @see CLASSNAME#METHOD
	 */
    public void setWrapAt( int wrapat ) {
        this.wrapat = wrapat;
        wrap();
    }

	/**
	 * DESCRIPTION
	 *
	 * @param PARAMNAME DESCRIPTION
	 * @return DESCRIPTION
	 * @exception CLASSNAME DESCRIPTION
	 * @see CLASSNAME#METHOD
	 */
    public int getWrapAt() {
        return wrapat;
    }

	/**
	 * DESCRIPTION
	 *
	 * @param PARAMNAME DESCRIPTION
	 * @return DESCRIPTION
	 * @exception CLASSNAME DESCRIPTION
	 * @see CLASSNAME#METHOD
	 */
    public Dimension size( Graphics g ) {

        g.setFont( font );
        FontMetrics metrics = g.getFontMetrics();

        int widest_line = 0;;
        StringTokenizer st = new StringTokenizer( wrapped_text, "\n" );
        while ( st.hasMoreTokens() ) {
            String line = st.nextToken();
            widest_line = Math.max( widest_line, metrics.stringWidth( line ) );
        }

        return new Dimension( widest_line, ( metrics.getHeight() * numlines ) );
    }

	/**
	 * DESCRIPTION
	 *
	 * @param PARAMNAME DESCRIPTION
	 * @return DESCRIPTION
	 * @exception CLASSNAME DESCRIPTION
	 * @see CLASSNAME#METHOD
	 */
    public void draw( Graphics g, Point upper_left ) {
        if ( color != null ) {
            g.setColor( color );
        }
        g.setFont( font );
        FontMetrics metrics = g.getFontMetrics();
        int x = upper_left.x;
        int y = upper_left.y + metrics.getAscent();
        Dimension size = size( g );
        StringTokenizer st = new StringTokenizer( wrapped_text, "\n" );
        while ( st.hasMoreTokens() ) {
            String line = st.nextToken();
            int width = metrics.stringWidth( line );
            if ( justification == RIGHT ) {
                x = upper_left.x + ( size.width - width );
            }
            else if ( justification == CENTER ) {
                x = upper_left.x + ( ( size.width / 2 ) - ( width / 2 ) );
            }
            g.drawString( line, x, y );
            y += metrics.getHeight();
        }
    }

	/**
	 * DESCRIPTION
	 *
	 * @param PARAMNAME DESCRIPTION
	 * @return DESCRIPTION
	 * @exception CLASSNAME DESCRIPTION
	 * @see CLASSNAME#METHOD
	 */
    public Font getFont() {
        return font;
    }

	/**
	 * DESCRIPTION
	 *
	 * @param PARAMNAME DESCRIPTION
	 * @return DESCRIPTION
	 * @exception CLASSNAME DESCRIPTION
	 * @see CLASSNAME#METHOD
	 */
    public void setFont( Font font ) {
        this.font = font;
    }

	/**
	 * DESCRIPTION
	 *
	 * @param PARAMNAME DESCRIPTION
	 * @return DESCRIPTION
	 * @exception CLASSNAME DESCRIPTION
	 * @see CLASSNAME#METHOD
	 */
    public Color getColor() {
        return color;
    }

	/**
	 * DESCRIPTION
	 *
	 * @param PARAMNAME DESCRIPTION
	 * @return DESCRIPTION
	 * @exception CLASSNAME DESCRIPTION
	 * @see CLASSNAME#METHOD
	 */
    public void setColor( Color color ) {
        this.color = color;
    }

	/**
	 * DESCRIPTION
	 *
	 * @param PARAMNAME DESCRIPTION
	 * @return DESCRIPTION
	 * @exception CLASSNAME DESCRIPTION
	 * @see CLASSNAME#METHOD
	 */
    public String getText() {
        return text;
    }

	/**
	 * DESCRIPTION
	 *
	 * @param PARAMNAME DESCRIPTION
	 * @return DESCRIPTION
	 * @exception CLASSNAME DESCRIPTION
	 * @see CLASSNAME#METHOD
	 */
    public void setText( String text ) {
        this.text = text;
        wrap();
    }
}
