/**
 * Copyright(c) 1996 DTAI, Incorporated (http://www.dtai.com)
 *
 *                        All rights reserved
 *
 * Permission to use, copy, modify and distribute this material for
 * any purpose and without fee is hereby granted, provided that the
 * above copyright notice and this permission notice appear in all
 * copies, and that the name of DTAI, Incorporated not be used in
 * advertising or publicity pertaining to this material without the
 * specific, prior written permission of an authorized representative of
 * DTAI, Incorporated.
 *
 * DTAI, INCORPORATED MAKES NO REPRESENTATIONS AND EXTENDS NO WARRANTIES,
 * EXPRESS OR IMPLIED, WITH RESPECT TO THE SOFTWARE, INCLUDING, BUT
 * NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 * FITNESS FOR ANY PARTICULAR PURPOSE, AND THE WARRANTY AGAINST
 * INFRINGEMENT OF PATENTS OR OTHER INTELLECTUAL PROPERTY RIGHTS.  THE
 * SOFTWARE IS PROVIDED "AS IS", AND IN NO EVENT SHALL DTAI, INCORPORATED OR
 * ANY OF ITS AFFILIATES BE LIABLE FOR ANY DAMAGES, INCLUDING ANY
 * LOST PROFITS OR OTHER INCIDENTAL OR CONSEQUENTIAL DAMAGES RELATING
 * TO THE SOFTWARE.
 */

package COM.dtai.gui;

import java.awt.*;
import java.applet.*;
import java.net.*;
import java.util.*;
import java.io.*;
import COM.dtai.util.*;

/**
 * DummyAppletContext - implements AppletContext and AppletStub to allow any
 * applet to easily run as an application.  The only thing it can't do is
 * access URL's.  Applet parameters are entered on the command line with
 * name as one word and value as the next.
 *
 * @version	1.0b1
 * @author	DTAI, Incorporated
 *
 * $Id: DummyAppletContext.java,v 1.3 1996/08/13 06:29:22 kadel Exp $
 *
 * $Source: /cvs/java/classes/COM/dtai/gui/DummyAppletContext.java,v $
 */

public class DummyAppletContext
    extends Frame
    implements AppletStub, AppletContext, URLStreamHandlerFactory {

    private TextField status;
    private String args[];

	/**
	 * DESCRIPTION
	 *
	 * @param PARAMNAME DESCRIPTION
	 * @return DESCRIPTION
	 * @exception CLASSNAME DESCRIPTION
	 * @see CLASSNAME#METHOD
	 */
    public DummyAppletContext( Applet applet, String args[],
                               int width, int height ) {

        super ( applet.getClass().getName() );

        URL.setURLStreamHandlerFactory( this );

        this.args = args;

        move( 5, 100 ); // this is only for debugging in Cafe

        status = new TextField();
        status.setEditable( false );

        add( "Center", applet );
        add( "South", status );

        applet.setStub(this);

        applet.init();

        appletResize( width, height );

        show();
        applet.start();
    }

	/**
	 * DESCRIPTION
	 *
	 * @param PARAMNAME DESCRIPTION
	 * @return DESCRIPTION
	 * @exception CLASSNAME DESCRIPTION
	 * @see CLASSNAME#METHOD
	 */
    public boolean handleEvent( Event evt ) {

        if ( evt.id == Event.WINDOW_DESTROY ) {
            System.exit(0);
        }

        return super.handleEvent(evt);
    }

    // AppletStub methods
	/**
	 * DESCRIPTION
	 *
	 * @param PARAMNAME DESCRIPTION
	 * @return DESCRIPTION
	 * @exception CLASSNAME DESCRIPTION
	 * @see CLASSNAME#METHOD
	 */
    public boolean isActive() { return true; }

	/**
	 * DESCRIPTION
	 *
	 * @param PARAMNAME DESCRIPTION
	 * @return DESCRIPTION
	 * @exception CLASSNAME DESCRIPTION
	 * @see CLASSNAME#METHOD
	 */
    public URL getDocumentBase() {
        URL url = null;
        try {
            File dummy = new File( "dummy.html" );
            String path = dummy.getAbsolutePath();
            if ( File.separator != "/" ) {
                StringBuffer buffer = new StringBuffer();
                StringTokenizer st = new StringTokenizer( path, File.separator );
                while ( st.hasMoreTokens() ) {
                    buffer.append( st.nextToken() + "/" );
                }
                path = buffer.toString();
                path = path.substring( 0, path.length()-1 );
            }
            url = new URL( "file", "", -1, path );
        }
        catch ( MalformedURLException mue ) {
            ShowUser.error( "Bad URL in DummyAppletContext, getDocumentBase()" );
        }
        return url;
    }

	/**
	 * DESCRIPTION
	 *
	 * @param PARAMNAME DESCRIPTION
	 * @return DESCRIPTION
	 * @exception CLASSNAME DESCRIPTION
	 * @see CLASSNAME#METHOD
	 */
    public final URL getCodeBase() { return getDocumentBase(); }

	/**
	 * DESCRIPTION
	 *
	 * @param PARAMNAME DESCRIPTION
	 * @return DESCRIPTION
	 * @exception CLASSNAME DESCRIPTION
	 * @see CLASSNAME#METHOD
	 */
    public final String getParameter( String name ) {
        for ( int idx = 0; idx < ( args.length - 1 ); idx+=2 ) {
            if ( args[idx].equals( name ) ) {
                return args[idx+1];
            }
        }
        return null;
    }

	/**
	 * DESCRIPTION
	 *
	 * @param PARAMNAME DESCRIPTION
	 * @return DESCRIPTION
	 * @exception CLASSNAME DESCRIPTION
	 * @see CLASSNAME#METHOD
	 */
    public final AppletContext getAppletContext() { return this; }

	/**
	 * DESCRIPTION
	 *
	 * @param PARAMNAME DESCRIPTION
	 * @return DESCRIPTION
	 * @exception CLASSNAME DESCRIPTION
	 * @see CLASSNAME#METHOD
	 */
    public void appletResize( int width, int height ) {

        Insets insets = insets();

        resize( ( width + insets.left + insets.right ),
                ( height + status.preferredSize().height +
                  insets.top + insets.bottom ) );
    }

    // AppletContext methods
	/**
	 * DESCRIPTION
	 *
	 * @param PARAMNAME DESCRIPTION
	 * @return DESCRIPTION
	 * @exception CLASSNAME DESCRIPTION
	 * @see CLASSNAME#METHOD
	 */
    public final AudioClip getAudioClip( URL url ) { return null; }

	/**
	 * DESCRIPTION
	 *
	 * @param PARAMNAME DESCRIPTION
	 * @return DESCRIPTION
	 * @exception CLASSNAME DESCRIPTION
	 * @see CLASSNAME#METHOD
	 */
    public final Image getImage( URL url ) {
        Image image = null;
        image = Toolkit.getDefaultToolkit().getImage( url.getFile() );

        return image;
    }

	/**
	 * DESCRIPTION
	 *
	 * @param PARAMNAME DESCRIPTION
	 * @return DESCRIPTION
	 * @exception CLASSNAME DESCRIPTION
	 * @see CLASSNAME#METHOD
	 */
    public final Applet getApplet( String name ) { return null; }

	/**
	 * DESCRIPTION
	 *
	 * @param PARAMNAME DESCRIPTION
	 * @return DESCRIPTION
	 * @exception CLASSNAME DESCRIPTION
	 * @see CLASSNAME#METHOD
	 */
    public final Enumeration getApplets() { return null; }

	/**
	 * DESCRIPTION
	 *
	 * @param PARAMNAME DESCRIPTION
	 * @return DESCRIPTION
	 * @exception CLASSNAME DESCRIPTION
	 * @see CLASSNAME#METHOD
	 */
    public void showDocument( URL url ) {
        status.setText( "AppletContext request to show URL " +
                        url.toString() );
    }

	/**
	 * DESCRIPTION
	 *
	 * @param PARAMNAME DESCRIPTION
	 * @return DESCRIPTION
	 * @exception CLASSNAME DESCRIPTION
	 * @see CLASSNAME#METHOD
	 */
    public void showDocument( URL url, String target ) {
        status.setText( "AppletContext request to show URL " +
                        url.toString() +
                        " in target: " + target );
    }

	/**
	 * DESCRIPTION
	 *
	 * @param PARAMNAME DESCRIPTION
	 * @return DESCRIPTION
	 * @exception CLASSNAME DESCRIPTION
	 * @see CLASSNAME#METHOD
	 */
    public void showStatus( String text ) { status.setText( text ); }

    public URLStreamHandler createURLStreamHandler( String protocol ) {
        return new MyHandler();
    }
}

class MyHandler extends URLStreamHandler {

    protected URLConnection openConnection( URL u ) throws IOException {
        return new MyConnection( u );
    }

}

class MyConnection extends URLConnection {

    boolean connected = false;
    InputStream instream;

    protected MyConnection( URL url ) {
        super( url );
    }

    public void connect() throws IOException {
        if ( ! connected ) {
            instream = new FileInputStream( url.getFile() );
        }
    }

    public InputStream getInputStream() throws IOException {
        if ( ! connected ) {
            connect();
        }
        if ( instream == null ) {
            throw new IOException();
        }
        return instream;
    }
}