/**
 * Copyright(c) 1996 DTAI, Incorporated (http://www.dtai.com)
 *
 *                        All rights reserved
 *
 * Permission to use, copy, modify and distribute this material for
 * any purpose and without fee is hereby granted, provided that the
 * above copyright notice and this permission notice appear in all
 * copies, and that the name of DTAI, Incorporated not be used in
 * advertising or publicity pertaining to this material without the
 * specific, prior written permission of an authorized representative of
 * DTAI, Incorporated.
 *
 * DTAI, INCORPORATED MAKES NO REPRESENTATIONS AND EXTENDS NO WARRANTIES,
 * EXPRESS OR IMPLIED, WITH RESPECT TO THE SOFTWARE, INCLUDING, BUT
 * NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 * FITNESS FOR ANY PARTICULAR PURPOSE, AND THE WARRANTY AGAINST
 * INFRINGEMENT OF PATENTS OR OTHER INTELLECTUAL PROPERTY RIGHTS.  THE
 * SOFTWARE IS PROVIDED "AS IS", AND IN NO EVENT SHALL DTAI, INCORPORATED OR
 * ANY OF ITS AFFILIATES BE LIABLE FOR ANY DAMAGES, INCLUDING ANY
 * LOST PROFITS OR OTHER INCIDENTAL OR CONSEQUENTIAL DAMAGES RELATING
 * TO THE SOFTWARE.
 */

package COM.dtai.gui;

import java.awt.*;
import java.awt.image.*;
import java.util.*;
import COM.dtai.util.*;

/**
 * GadgetManager - manages a bunch of GadgetLayers of Gadgets for drawing,
 * selecting, and interacting with the user.
 *
 * @see		COM.dtai.gui.GadgetLayer
 * @see		COM.dtai.gui.Gadget
 * @version	1.0b1
 * @author	DTAI, Incorporated
 *
 * $Id: GadgetManager.java,v 1.3 1996/08/13 06:29:23 kadel Exp $
 *
 * $Source: /cvs/java/classes/COM/dtai/gui/GadgetManager.java,v $
 */

public class GadgetManager extends Canvas {

    private Image image;
    private Vector layers = new Vector();
    private Vector observers = new Vector();
    private Gadget lastdragged;
    private Gadget lastover;
    private Gadget lastarmed;
    private Gadget lastdragover;
    private Point offset = new Point( 0, 0 );
    private Dimension image_size = new Dimension( 0, 0 );
    private Event last_mouse_up;
    private int frozen = 0;
    private boolean repaint_pending = false;
    private int lastx = 0;
    private int lasty = 0;
    private Image background_image;

	/**
	 * DESCRIPTION
	 *
	 * @param PARAMNAME DESCRIPTION
	 * @return DESCRIPTION
	 * @exception CLASSNAME DESCRIPTION
	 * @see CLASSNAME#METHOD
	 */
    public GadgetManager() {

      //  resize( 1, 1 );

        setBackground( Color.white );
    }

	/**
	 * DESCRIPTION
	 *
	 * @param PARAMNAME DESCRIPTION
	 * @return DESCRIPTION
	 * @exception CLASSNAME DESCRIPTION
	 * @see CLASSNAME#METHOD
	 */
    public void setBackgroundImage( Image image ) {
        background_image = image;
        if ( image != null ) {
            prepareImage( image, this );
        }
    }

	/**
	 * DESCRIPTION
	 *
	 * @param PARAMNAME DESCRIPTION
	 * @return DESCRIPTION
	 * @exception CLASSNAME DESCRIPTION
	 * @see CLASSNAME#METHOD
	 */
    public void freeze() {
        frozen++;
    }

	/**
	 * DESCRIPTION
	 *
	 * @param PARAMNAME DESCRIPTION
	 * @return DESCRIPTION
	 * @exception CLASSNAME DESCRIPTION
	 * @see CLASSNAME#METHOD
	 */
    public void thaw() {
        frozen--;
        if ( ( frozen == 0 ) &&
             repaint_pending ) {
            repaint();
        }
    }

	/**
	 * DESCRIPTION
	 *
	 * @param PARAMNAME DESCRIPTION
	 * @return DESCRIPTION
	 * @exception CLASSNAME DESCRIPTION
	 * @see CLASSNAME#METHOD
	 */
    public GadgetLayer getLayer( int layernum ) {
        int size = layers.size();

        for ( int idx = size; idx < ( layernum + 1 ); idx++ ) {
            layers.addElement( new GadgetLayer() );
        }

        return (GadgetLayer)layers.elementAt( layernum );
    }

	/**
	 * DESCRIPTION
	 *
	 * @param PARAMNAME DESCRIPTION
	 * @return DESCRIPTION
	 * @exception CLASSNAME DESCRIPTION
	 * @see CLASSNAME#METHOD
	 */
    public void addGadget( Gadget gadget ) {
        getLayer( 0 ).addGadget( gadget );
    }

	/**
	 * DESCRIPTION
	 *
	 * @param PARAMNAME DESCRIPTION
	 * @return DESCRIPTION
	 * @exception CLASSNAME DESCRIPTION
	 * @see CLASSNAME#METHOD
	 */
    public final Graphics getPermGraphics() {
        if ( image == null ) {
            return null;
        }
        else {
            return image.getGraphics();
        }
    }

    private protected void initImage() {
        if ( image == null ) {
            image = createImage( 5, 5 );
        }
    }

    private protected void paintImage() {

        image_size.width = Math.max( 1, image_size.width );
        image_size.height = Math.max( 1, image_size.height );
        image = createImage( image_size.width, image_size.height );

        Graphics g = image.getGraphics();

        g.setColor( getBackground() );

        if ( background_image == null ) {
            g.fillRect( 0, 0, image_size.width, image_size.height );
        }
        else {
            tileImage( g, image_size.width, image_size.height,
                       offset.x, offset.y, background_image );
        }

        for ( Enumeration e = layers.elements(); e.hasMoreElements(); ) {
            ((GadgetLayer)e.nextElement()).draw( g );
        }
    }

	/**
	 * DESCRIPTION
	 *
	 * @param PARAMNAME DESCRIPTION
	 * @return DESCRIPTION
	 * @exception CLASSNAME DESCRIPTION
	 * @see CLASSNAME#METHOD
	 */
    public void setImageSize( int width, int height ) {
        if ( ( width != image_size.width ) || ( height != image_size.height ) ) {
            image_size.width = width;
            image_size.height = height;

            for ( Enumeration e = observers.elements(); e.hasMoreElements(); ) {
                ((ImageObserver)e.nextElement()).imageUpdate(
                    image, ImageObserver.HEIGHT | ImageObserver.WIDTH, 0, 0,
                    image_size.width, image_size.height );
            }
        }
    }

	/**
	 * DESCRIPTION
	 *
	 * @param PARAMNAME DESCRIPTION
	 * @return DESCRIPTION
	 * @exception CLASSNAME DESCRIPTION
	 * @see CLASSNAME#METHOD
	 */
    public synchronized void reshape (int x, int y, int width, int height) {
        super.reshape (x, y, width, height);
        for ( Enumeration e = observers.elements(); e.hasMoreElements(); ) {
            ((ImageObserver)e.nextElement()).imageUpdate(
                image, ImageObserver.HEIGHT | ImageObserver.WIDTH, 0, 0,
                image_size.width, image_size.height );
        }
    }

	/**
	 * DESCRIPTION
	 *
	 * @param PARAMNAME DESCRIPTION
	 * @return DESCRIPTION
	 * @exception CLASSNAME DESCRIPTION
	 * @see CLASSNAME#METHOD
	 */
    public void setImageSize( Dimension size ) {
        setImageSize( size.width, size.height );
    }

	/**
	 * DESCRIPTION
	 *
	 * @param PARAMNAME DESCRIPTION
	 * @return DESCRIPTION
	 * @exception CLASSNAME DESCRIPTION
	 * @see CLASSNAME#METHOD
	 */
    public void addImageObserver( ImageObserver observer ) {
        observers.addElement( observer );
    }

	/**
	 * DESCRIPTION
	 *
	 * @param PARAMNAME DESCRIPTION
	 * @return DESCRIPTION
	 * @exception CLASSNAME DESCRIPTION
	 * @see CLASSNAME#METHOD
	 */
    public void removeImageObserver( ImageObserver observer ) {
        observers.removeElement( observer );
    }

	/**
	 * DESCRIPTION
	 *
	 * @param PARAMNAME DESCRIPTION
	 * @return DESCRIPTION
	 * @exception CLASSNAME DESCRIPTION
	 * @see CLASSNAME#METHOD
	 */
    public Dimension getImageSize() {
        return new Dimension( image_size.width, image_size.height );
    }

	/**
	 * DESCRIPTION
	 *
	 * @param PARAMNAME DESCRIPTION
	 * @return DESCRIPTION
	 * @exception CLASSNAME DESCRIPTION
	 * @see CLASSNAME#METHOD
	 */
    public void setOffsets( int x, int y ) {
        if ( ( x != offset.x ) ||
             ( y != offset.y ) ) {
            offset.x = x;
            offset.y = y;
            if ( isVisible() ) {
                repaint();
            }
        }
    }

	/**
	 * DESCRIPTION
	 *
	 * @param PARAMNAME DESCRIPTION
	 * @return DESCRIPTION
	 * @exception CLASSNAME DESCRIPTION
	 * @see CLASSNAME#METHOD
	 */
    public Point getOffsets() {
        return new Point( offset.x, offset.y );
    }

    private protected boolean repainting = false;

	/**
	 * DESCRIPTION
	 *
	 * @param PARAMNAME DESCRIPTION
	 * @return DESCRIPTION
	 * @exception CLASSNAME DESCRIPTION
	 * @see CLASSNAME#METHOD
	 */
    public void repaint() {
        if ( frozen > 0 ) {
            repaint_pending = true;
        }
        else {
            if ( ! repainting ) {
                repaint_pending = false;
                repainting = true;
                super.repaint();
            }
        }
    }

    private void tileImage( Graphics g, int width, int height,
                            int offset_x, int offset_y, Image image ) {

        int image_width = image.getWidth( this );
        int image_height = image.getHeight( this );
        for ( int i = -image_height; i < height + offset_y; i += image_height ) {
            for ( int j = -image_width; j < width + offset_x; j += image_width ) {
                g.drawImage( image, j+2 - ( offset_x ), i+2 - ( offset_y ), this );
            }
        }
    }

	/**
	 * DESCRIPTION
	 *
	 * @param PARAMNAME DESCRIPTION
	 * @return DESCRIPTION
	 * @exception CLASSNAME DESCRIPTION
	 * @see CLASSNAME#METHOD
	 */
    public void paint ( Graphics g ) {
        repainting = true;
        lastover = null;
        paintImage();
        Dimension size = size();
        g.setColor( getBackground() );
        if ( background_image == null ) {
            g.fillRect( 0, 0, size.width, size.height );
        }
        else {
            tileImage( g, size.width, size.height, 0, 0, background_image );
        }
        g.drawImage( image, offset.x, offset.y, this );
        mouseMove( null, lastx, lasty );
        repainting = false;
    }

	/**
	 * DESCRIPTION
	 *
	 * @param PARAMNAME DESCRIPTION
	 * @return DESCRIPTION
	 * @exception CLASSNAME DESCRIPTION
	 * @see CLASSNAME#METHOD
	 */
    public void update( Graphics g ) {
        paint( g );
    }

	/**
	 * DESCRIPTION
	 *
	 * @param PARAMNAME DESCRIPTION
	 * @return DESCRIPTION
	 * @exception CLASSNAME DESCRIPTION
	 * @see CLASSNAME#METHOD
	 */
    public void clear() {
        for ( Enumeration e = layers.elements(); e.hasMoreElements(); ) {
            ((GadgetLayer)e.nextElement()).removeAllGadgets();
        }
        repaint();
    }

	/**
	 * DESCRIPTION
	 *
	 * @param PARAMNAME DESCRIPTION
	 * @return DESCRIPTION
	 * @exception CLASSNAME DESCRIPTION
	 * @see CLASSNAME#METHOD
	 */
    public Gadget gadgetAt( int x, int y ) {
        int numlayers = layers.size();
        for ( int idx = numlayers - 1; idx >= 0; idx-- ) {
            GadgetLayer layer = (GadgetLayer)layers.elementAt( idx );
            if ( layer.clickable() ) {
                Gadget gadget = layer.gadgetAt( x, y );
                if ( gadget != null ) {
                    return gadget;
                }
            }
        }
        return null;
    }

	/**
	 * DESCRIPTION
	 *
	 * @param PARAMNAME DESCRIPTION
	 * @return DESCRIPTION
	 * @exception CLASSNAME DESCRIPTION
	 * @see CLASSNAME#METHOD
	 */
    public void disarmAll() {
        for ( Enumeration e = layers.elements(); e.hasMoreElements(); ) {
            ((GadgetLayer)e.nextElement()).disarmAll();
        }
    }

	/**
	 * DESCRIPTION
	 *
	 * @param PARAMNAME DESCRIPTION
	 * @return DESCRIPTION
	 * @exception CLASSNAME DESCRIPTION
	 * @see CLASSNAME#METHOD
	 */
    public void deselectAll() {
        for ( Enumeration e = layers.elements(); e.hasMoreElements(); ) {
            ((GadgetLayer)e.nextElement()).deselectAll();
        }
    }

	/**
	 * DESCRIPTION
	 *
	 * @param PARAMNAME DESCRIPTION
	 * @return DESCRIPTION
	 * @exception CLASSNAME DESCRIPTION
	 * @see CLASSNAME#METHOD
	 */
    public boolean gotFocus( Event evt, Object arg ) {
        return true;
    }

	/**
	 * DESCRIPTION
	 *
	 * @param PARAMNAME DESCRIPTION
	 * @return DESCRIPTION
	 * @exception CLASSNAME DESCRIPTION
	 * @see CLASSNAME#METHOD
	 */
    public boolean mouseDown( Event evt, int x, int y ) {

        x -= offset.x;
        y -= offset.y;
        Gadget armed = gadgetAt( x, y );
        if ( armed != null ) {
            if ( ! evt.controlDown() ) {
                armed.arm();
                lastarmed = armed;
            }
            if ( armed.getLayer().dragable() ) {
                armed.beginDrag( evt, x, y );
                lastdragged = armed;
                lastdragover = null;
            }
        }

        return true;
    }

	/**
	 * DESCRIPTION
	 *
	 * @param PARAMNAME DESCRIPTION
	 * @return DESCRIPTION
	 * @exception CLASSNAME DESCRIPTION
	 * @see CLASSNAME#METHOD
	 */
    public boolean mouseDrag( Event evt, int x, int y ) {

        x -= offset.x;
        y -= offset.y;

        if ( lastarmed != null ) {
            if ( lastarmed.contains( x, y ) ) {
                if ( ! lastarmed.isArmed() ) {
                    lastarmed.arm();
                }
            }
            else {
                if ( lastarmed.isArmed() ) {
                    lastarmed.disarm();
                }
            }
        }

        if ( lastdragged != null ) {
            lastdragged.drag( evt, x, y );
            Gadget dragover = gadgetAt( x, y );
            if ( dragover != lastdragover ) {
                if ( lastdragover != null ) {
                    lastdragover.notUnder( lastdragged );
                }
                lastdragover = dragover;
                if ( lastdragover != null ) {
                    lastdragover.isUnder( lastdragged );
                }
            }
        }
        return true;
    }

	/**
	 * DESCRIPTION
	 *
	 * @param PARAMNAME DESCRIPTION
	 * @return DESCRIPTION
	 * @exception CLASSNAME DESCRIPTION
	 * @see CLASSNAME#METHOD
	 */
    public boolean mouseUp( Event evt, int x, int y ) {

        x -= offset.x;
        y -= offset.y;

        boolean doubleclick = false;
        if ( ( last_mouse_up != null ) &&
             ( ( evt.when - last_mouse_up.when ) < 1000 ) && // 1 second
             ( Math.abs( evt.x - last_mouse_up.x ) < 4 ) &&
             ( Math.abs( evt.y - last_mouse_up.y ) < 4 ) ) {
             doubleclick = true;
        }
        last_mouse_up = evt;

        Gadget clicked = gadgetAt( x, y );

        if ( lastdragged != null ) {
            if ( lastdragover != null ) {
                lastdragover.notUnder( lastdragged );
            }
            lastdragover = gadgetAt( x, y );
            if ( ( lastdragover != lastdragged ) &&
                 ( lastdragover != null ) ) {
                lastdragged.clearDrag();
                if ( ! lastdragged.dropOn( lastdragover ) ) {
                    lastdragged.drop( evt, x, y );
                }
            }
            else {
                lastdragged.drop( evt, x, y );
            }
        }
        if ( clicked == null ) {
            if ( lastarmed == null ) {
                deselectAll();
                repaint();
            }
        }
        else {
            if ( clicked.isArmed() ) {
                if ( doubleclick ) {
                    clicked.doubleClick();
                }
                else {
                    clicked.select( evt, x, y );
                }
            }
        }

        disarmAll ();
        lastarmed = null;
        lastdragged = null;

        return true;
    }

  /**
   * DESCRIPTION
   *
   * @param PARAMNAME DESCRIPTION
   * @return DESCRIPTION
   * @exception CLASSNAME DESCRIPTION
   * @see CLASSNAME#METHOD
   */
//    public boolean mouseEnter( Event evt, int x, int y ) {
//        return false;
//    }

  /**
   * DESCRIPTION
   *
   * @param PARAMNAME DESCRIPTION
   * @return DESCRIPTION
   * @exception CLASSNAME DESCRIPTION
   * @see CLASSNAME#METHOD
   */
    public boolean mouseExit( Event evt, int x, int y ) {
        if ( lastover != null ) {
            lastover.mouseExit();
            lastover = null;
        }
        ShowUser.status( "" );
        return true;
    }

  /**
   * DESCRIPTION
   *
   * @param PARAMNAME DESCRIPTION
   * @return DESCRIPTION
   * @exception CLASSNAME DESCRIPTION
   * @see CLASSNAME#METHOD
   */
    public boolean mouseMove( Event evt, int x, int y ) {

        x -= offset.x;
        y -= offset.y;

        Gadget over = gadgetAt( x, y );
        if ( over != lastover ) {
            if ( lastover != null ) {
                lastover.mouseExit();
                lastover = null;
            }
            if ( over == null ) {
                ShowUser.status( "" );
            }
            else {
                over.mouseEnter();
            }
        }
        if ( over != null ) {
            over.mouseMove( evt, x, y );
        }

        lastover = over;

        lastx = x + offset.x;
        lasty = y + offset.y;

        return true;
    }

    public boolean imageUpdate( Image img, int infoflags, int x, int y,
                               int width, int height ) {
        if ( ( infoflags & ALLBITS ) != 0 ) {
            repaint();
        }
        return super.imageUpdate( img, infoflags, x, y, width, height );
    }
}