/**
 * Copyright(c) 1996 DTAI, Incorporated (http://www.dtai.com)
 *
 *                        All rights reserved
 *
 * Permission to use, copy, modify and distribute this material for
 * any purpose and without fee is hereby granted, provided that the
 * above copyright notice and this permission notice appear in all
 * copies, and that the name of DTAI, Incorporated not be used in
 * advertising or publicity pertaining to this material without the
 * specific, prior written permission of an authorized representative of
 * DTAI, Incorporated.
 *
 * DTAI, INCORPORATED MAKES NO REPRESENTATIONS AND EXTENDS NO WARRANTIES,
 * EXPRESS OR IMPLIED, WITH RESPECT TO THE SOFTWARE, INCLUDING, BUT
 * NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 * FITNESS FOR ANY PARTICULAR PURPOSE, AND THE WARRANTY AGAINST
 * INFRINGEMENT OF PATENTS OR OTHER INTELLECTUAL PROPERTY RIGHTS.  THE
 * SOFTWARE IS PROVIDED "AS IS", AND IN NO EVENT SHALL DTAI, INCORPORATED OR
 * ANY OF ITS AFFILIATES BE LIABLE FOR ANY DAMAGES, INCLUDING ANY
 * LOST PROFITS OR OTHER INCIDENTAL OR CONSEQUENTIAL DAMAGES RELATING
 * TO THE SOFTWARE.
 */

package COM.dtai.gui;

import java.awt.*;
import java.util.*;

/**
 * StdDialog - superclass for most DTAI dialogs
 *
 * @version	1.0b1
 * @author	DTAI, Incorporated
 *
 * $Id: StdDialog.java,v 1.3 1996/08/13 06:29:25 kadel Exp $
 *
 * $Source: /cvs/java/classes/COM/dtai/gui/StdDialog.java,v $
 */

public class StdDialog extends Dialog {

    private Panel buttons;
    private Panel mainpanel;
    private Frame parent_frame;
    private Vector observers;
    private boolean setup = false;

	/**
	 * DESCRIPTION
	 *
	 * @param PARAMNAME DESCRIPTION
	 * @return DESCRIPTION
	 * @exception CLASSNAME DESCRIPTION
	 * @see CLASSNAME#METHOD
	 */
    public StdDialog ( Frame frame, String name, boolean modal ) {

//        super( frame, name, modal );
        super( frame, name, false ); // Bug noticed in CAFE...modal dialogs
                                     // can lock up windows!

     //   setForeground( frame.getForeground() );
     //   setBackground( frame.getBackground() );
        setForeground( Color.black );
        setBackground( Color.lightGray );

        parent_frame = frame;

        setLayout( new BorderLayout() );

        add( "North", new Separator() );

        Panel buttons_and_sep = new Panel();
        add( "South", buttons_and_sep );

        buttons_and_sep.setLayout( new BorderLayout () );

        buttons_and_sep.add( "Center", new Separator() );

        Panel button_area = new Panel();
        buttons_and_sep.add( "South", button_area );

        button_area.setLayout ( new FlowLayout() );

        buttons = new Panel();
        button_area.add( buttons );

        buttons.setLayout ( new GridLayout( 1, 0, 3, 3 ) );
    }

	/**
	 * DESCRIPTION
	 *
	 * @param PARAMNAME DESCRIPTION
	 * @return DESCRIPTION
	 * @exception CLASSNAME DESCRIPTION
	 * @see CLASSNAME#METHOD
	 */
    public void addObserver( DialogObserver observer ) {
        if ( observers == null ) {
            observers = new Vector();
        }
        observers.addElement( observer );
    }

	/**
	 * DESCRIPTION
	 *
	 * @param PARAMNAME DESCRIPTION
	 * @return DESCRIPTION
	 * @exception CLASSNAME DESCRIPTION
	 * @see CLASSNAME#METHOD
	 */
    public void removeObserver( DialogObserver observer ) {
        if ( observers != null ) {
            observers.removeElement( observer );
        }
    }

	/**
	 * DESCRIPTION
	 *
	 * @param PARAMNAME DESCRIPTION
	 * @return DESCRIPTION
	 * @exception CLASSNAME DESCRIPTION
	 * @see CLASSNAME#METHOD
	 */
    public void removeAllObservers() {
        if ( observers != null ) {
            observers.removeAllElements();
        }
    }

	/**
	 * DESCRIPTION
	 *
	 * @param PARAMNAME DESCRIPTION
	 * @return DESCRIPTION
	 * @exception CLASSNAME DESCRIPTION
	 * @see CLASSNAME#METHOD
	 */
    public void notifyObservers() {
        if ( observers != null ) {
            for ( Enumeration e = observers.elements(); e.hasMoreElements(); ) {
                ((DialogObserver)e.nextElement()).dialogAction( this );
            }
        }
    }

	/**
	 * DESCRIPTION
	 *
	 * @param PARAMNAME DESCRIPTION
	 * @return DESCRIPTION
	 * @exception CLASSNAME DESCRIPTION
	 * @see CLASSNAME#METHOD
	 */
    public void hide() {
        super.hide();
        parent_frame.requestFocus();
    }

	/**
	 * DESCRIPTION
	 *
	 * @param PARAMNAME DESCRIPTION
	 * @return DESCRIPTION
	 * @exception CLASSNAME DESCRIPTION
	 * @see CLASSNAME#METHOD
	 */
    public void addButton( Button button ) {
        buttons.add( button );
    }

	/**
	 * DESCRIPTION
	 *
	 * @param PARAMNAME DESCRIPTION
	 * @return DESCRIPTION
	 * @exception CLASSNAME DESCRIPTION
	 * @see CLASSNAME#METHOD
	 */
    public Panel getMainPanel() {
        if ( mainpanel == null ) {
            mainpanel = new Panel();
            add( "Center", mainpanel );
        }
        return mainpanel;
    }

    private void positionDialog() {
        if ( parent_frame != null ) {
            Point      position   = parent_frame.location();
            Dimension  frame_size = parent_frame.size();
            Dimension  dlg_size   = size();

            position.x += ( frame_size.width - dlg_size.width ) / 2;
            position.y += ( frame_size.height - dlg_size.height ) / 2;
            move( Math.max( 0, position.x ), Math.max( 0, position.y ) );
        }
    }

	/**
	 * DESCRIPTION
	 *
	 * @param PARAMNAME DESCRIPTION
	 * @return DESCRIPTION
	 * @exception CLASSNAME DESCRIPTION
	 * @see CLASSNAME#METHOD
	 */
    public void show() {
        if ( ! setup ) {
            positionDialog();
            setup = true;
        }
        pack();
        super.show();
    }

	/**
	 * DESCRIPTION
	 *
	 * @param PARAMNAME DESCRIPTION
	 * @return DESCRIPTION
	 * @exception CLASSNAME DESCRIPTION
	 * @see CLASSNAME#METHOD
	 */
    public boolean handleEvent( Event evt ) {

        super.handleEvent ( evt );

        if ( evt.id == Event.WINDOW_DESTROY ) {
            hide ();
			notifyObservers();
            return ( true );
        }
        else {
            return ( false );
        }
    }
}
