/**
 * Copyright(c) 1996 DTAI, Incorporated (http://www.dtai.com)
 *
 *                        All rights reserved
 *
 * Permission to use, copy, modify and distribute this material for
 * any purpose and without fee is hereby granted, provided that the
 * above copyright notice and this permission notice appear in all
 * copies, and that the name of DTAI, Incorporated not be used in
 * advertising or publicity pertaining to this material without the
 * specific, prior written permission of an authorized representative of
 * DTAI, Incorporated.
 *
 * DTAI, INCORPORATED MAKES NO REPRESENTATIONS AND EXTENDS NO WARRANTIES,
 * EXPRESS OR IMPLIED, WITH RESPECT TO THE SOFTWARE, INCLUDING, BUT
 * NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 * FITNESS FOR ANY PARTICULAR PURPOSE, AND THE WARRANTY AGAINST
 * INFRINGEMENT OF PATENTS OR OTHER INTELLECTUAL PROPERTY RIGHTS.  THE
 * SOFTWARE IS PROVIDED "AS IS", AND IN NO EVENT SHALL DTAI, INCORPORATED OR
 * ANY OF ITS AFFILIATES BE LIABLE FOR ANY DAMAGES, INCLUDING ANY
 * LOST PROFITS OR OTHER INCIDENTAL OR CONSEQUENTIAL DAMAGES RELATING
 * TO THE SOFTWARE.
 */

package COM.dtai.net;

import java.net.*;
import java.io.*;

/**
 * NetPortToClient - in client/server, this is a class in the server that
 * provides a simple API for a socket interface to one of the clients.
 *
 * @see		COM.dtai.net.NetServer
 * @version	1.0b1
 * @author	DTAI, Incorporated
 *
 * $Id: NetPortToClient.java,v 1.3 1996/07/22 23:52:29 kadel Exp $
 *
 * $Source: /cvs/java/classes/COM/dtai/net/NetPortToClient.java,v $
 */

public class NetPortToClient extends NetPort {

    private NetServer server;

	/**
	 * Constructs a NetPortToClient object for the given server over the Socket
	 * already opened to the client
	 * 
	 * @param server the NetServer that the given client connected to
	 * @param socket the socket opened to the client
	 * @exception IOException if some IO error occurred
	 */
    public NetPortToClient( NetServer server, Socket socket )
		throws IOException {
        super ( socket );
        this.server = server;
    }

	/**
	 * If an error occurs, the server will be told to issue a warning, and this
	 * NetPortToClient thread will be stopped, if started.
	 * 
	 * @param message an error message
	 */
    public synchronized void error( String message ) {
        server.warning ( "Client port error: " + message );
        stop();
    }
}
