/**
 * Copyright(c) 1996 DTAI, Incorporated (http://www.dtai.com)
 *
 *                        All rights reserved
 *
 * Permission to use, copy, modify and distribute this material for
 * any purpose and without fee is hereby granted, provided that the
 * above copyright notice and this permission notice appear in all
 * copies, and that the name of DTAI, Incorporated not be used in
 * advertising or publicity pertaining to this material without the
 * specific, prior written permission of an authorized representative of
 * DTAI, Incorporated.
 *
 * DTAI, INCORPORATED MAKES NO REPRESENTATIONS AND EXTENDS NO WARRANTIES,
 * EXPRESS OR IMPLIED, WITH RESPECT TO THE SOFTWARE, INCLUDING, BUT
 * NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 * FITNESS FOR ANY PARTICULAR PURPOSE, AND THE WARRANTY AGAINST
 * INFRINGEMENT OF PATENTS OR OTHER INTELLECTUAL PROPERTY RIGHTS.  THE
 * SOFTWARE IS PROVIDED "AS IS", AND IN NO EVENT SHALL DTAI, INCORPORATED OR
 * ANY OF ITS AFFILIATES BE LIABLE FOR ANY DAMAGES, INCLUDING ANY
 * LOST PROFITS OR OTHER INCIDENTAL OR CONSEQUENTIAL DAMAGES RELATING
 * TO THE SOFTWARE.
 */

package COM.dtai.net;

import java.net.*;
import java.io.*;
import java.util.*;

/**
 * NetServer - a superclass for a server in a TCP/IP-based client/server
 * architecture.
 *
 * @see		COM.dtai.net.NetPortToClient
 * @version	1.0b1
 * @author	DTAI, Incorporated
 *
 * $Id: NetServer.java,v 1.3 1996/07/22 23:52:30 kadel Exp $
 *
 * $Source: /cvs/java/classes/COM/dtai/net/NetServer.java,v $
 */

public abstract class NetServer extends Thread
{
    private int portnum;

	/**
     * Opens the given TCP/IP port on the current machine to accept
     * new clients.  Starts a separate thread to execute the server function
	 * of waiting for and accepting the clients.
	 *
     * @param portnum	The specific port number
	 */
    public NetServer ( int portnum ) {
        this.portnum = portnum;
        start ();
    }

	/**
	 * Executes the main body of the thread to loop and wait for and accept
	 * client connections
	 */
    public void run () {

        ServerSocket server_socket = null;

        try {
            server_socket = new ServerSocket( portnum );

            while ( true ) {
                createClientPort ( server_socket.accept () );
            }
        }
        catch ( IOException ioerror ) {
            error ( "IO Error opening server socket" );
        }
        finally {
            if ( server_socket != null ) {
                try {
                    server_socket.close();
                }
                catch ( IOException ioe ) {
                }
            }
        }
    }

    /**
     * subclass and extend this class to create your own servers
	 * @param socket the socket connection from the client from which
	 *				 to create the NetPortToClient.
	 * @see NetPortToClient
     */

    protected abstract void createClientPort( Socket socket );

    /**
     * Outputs the given error message and stops the thread.
     *
     * @param message	The error message
     */
    public synchronized void error ( String message ) {
        System.err.println ( "Error: " + message );
        stop();
    }

    /**
     * Outputs the given warning message.
     *
     * @param message	The warning message
     */
    public synchronized void warning ( String message ) {
        System.err.println ( "Warning: " + message );
    }
}
