/**
 * Copyright(c) 1996 DTAI, Incorporated (http://www.dtai.com)
 *
 *                        All rights reserved
 *
 * Permission to use, copy, modify and distribute this material for
 * any purpose and without fee is hereby granted, provided that the
 * above copyright notice and this permission notice appear in all
 * copies, and that the name of DTAI, Incorporated not be used in
 * advertising or publicity pertaining to this material without the
 * specific, prior written permission of an authorized representative of
 * DTAI, Incorporated.
 *
 * DTAI, INCORPORATED MAKES NO REPRESENTATIONS AND EXTENDS NO WARRANTIES,
 * EXPRESS OR IMPLIED, WITH RESPECT TO THE SOFTWARE, INCLUDING, BUT
 * NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 * FITNESS FOR ANY PARTICULAR PURPOSE, AND THE WARRANTY AGAINST
 * INFRINGEMENT OF PATENTS OR OTHER INTELLECTUAL PROPERTY RIGHTS.  THE
 * SOFTWARE IS PROVIDED "AS IS", AND IN NO EVENT SHALL DTAI, INCORPORATED OR
 * ANY OF ITS AFFILIATES BE LIABLE FOR ANY DAMAGES, INCLUDING ANY
 * LOST PROFITS OR OTHER INCIDENTAL OR CONSEQUENTIAL DAMAGES RELATING
 * TO THE SOFTWARE.
 */

package COM.dtai.serial;

import java.net.*;
import java.io.*;
import COM.dtai.net.*;

/**
 * DbClient - a client-side API to the DbServer database server
 *
 * @version	1.0b1
 * @author	DTAI, Incorporated
 *
 * $Id: DbClient.java,v 1.2 1996/08/15 08:16:54 kadel Exp $
 *
 * $Source: /cvs/java/classes/COM/dtai/serial/DbClient.java,v $
 */

public class DbClient {

    protected NetPortToServer port;

	protected boolean open = false;

	/**
	 * DESCRIPTION
	 *
	 * @param PARAMNAME DESCRIPTION
	 * @return DESCRIPTION
	 * @exception CLASSNAME DESCRIPTION
	 * @see CLASSNAME#METHOD
	 */
    public DbClient( String host, int portnum ) throws IOException {
        port = new NetPortToServer( host, portnum );
    }

    /**
     * dummy db if can't connect
     */

	/**
	 * DESCRIPTION
	 *
	 * @param PARAMNAME DESCRIPTION
	 * @return DESCRIPTION
	 * @exception CLASSNAME DESCRIPTION
	 * @see CLASSNAME#METHOD
	 */
    public DbClient() {
        port = null;
    }

	/**
	 * DESCRIPTION
	 *
	 * @param PARAMNAME DESCRIPTION
	 * @return DESCRIPTION
	 * @exception CLASSNAME DESCRIPTION
	 * @see CLASSNAME#METHOD
	 */
    public synchronized void openForReading( String database )
		throws IOException, IllegalAccessException {

        if ( port == null ) return;
        port.writeInt( DbCommands.OpenForReading );
        port.writeUTF( database );

        if ( port.readInt() == DbCommands.Failure ) {
            throw new IllegalAccessException(
                "Unable to open database for reading!" );
        }
		open = true;
    }

	/**
	 * DESCRIPTION
	 *
	 * @param PARAMNAME DESCRIPTION
	 * @return DESCRIPTION
	 * @exception CLASSNAME DESCRIPTION
	 * @see CLASSNAME#METHOD
	 */
    public synchronized void openForWriting( String database )
		throws IOException, IllegalAccessException {

        if ( port == null ) return;
        port.writeInt( DbCommands.OpenForWriting );
        port.writeUTF( database );

        if ( port.readInt() == DbCommands.Failure ) {
            throw new IllegalAccessException(
                "Unable to open database for writing (probably in use)" );
        }
		open = true;
    }

	/**
	 * DESCRIPTION
	 *
	 * @param PARAMNAME DESCRIPTION
	 * @return DESCRIPTION
	 * @exception CLASSNAME DESCRIPTION
	 * @see CLASSNAME#METHOD
	 */
    public synchronized void forceOpenForWriting( String database )
		throws IOException, IllegalAccessException {

        if ( port == null ) return;
        port.writeInt( DbCommands.ForceOpenForWriting );
        port.writeUTF( database );

        if ( port.readInt() == DbCommands.Failure ) {
            throw new IllegalAccessException(
                "Unable to force open database for writing!" );
        }
		open = true;
    }

	/**
	 * DESCRIPTION
	 *
	 * @param PARAMNAME DESCRIPTION
	 * @return DESCRIPTION
	 * @exception CLASSNAME DESCRIPTION
	 * @see CLASSNAME#METHOD
	 */
    public synchronized boolean isOpen() {
		return open;
	}

	/**
	 * DESCRIPTION
	 *
	 * @param PARAMNAME DESCRIPTION
	 * @return DESCRIPTION
	 * @exception CLASSNAME DESCRIPTION
	 * @see CLASSNAME#METHOD
	 */
    public synchronized void undoAll()
		throws IOException, IllegalAccessException {

        if ( port == null ) return;
        port.writeInt( DbCommands.UndoAll );
        if ( port.readInt() == DbCommands.Failure ) {
            throw new IllegalAccessException(
                "Unable to undo database");
        }
    }

	/**
	 * DESCRIPTION
	 *
	 * @param PARAMNAME DESCRIPTION
	 * @return DESCRIPTION
	 * @exception CLASSNAME DESCRIPTION
	 * @see CLASSNAME#METHOD
	 */
    public synchronized void saveAll()
		throws IOException, IllegalAccessException {

        if ( port == null ) return;
        port.writeInt( DbCommands.Save );

        if ( port.readInt() == DbCommands.Failure ) {
            throw new IllegalAccessException(
                "Unable to save database");
        }
    }

	/**
	 * DESCRIPTION
	 *
	 * @param PARAMNAME DESCRIPTION
	 * @return DESCRIPTION
	 * @exception CLASSNAME DESCRIPTION
	 * @see CLASSNAME#METHOD
	 */
    public synchronized void close()
		throws IOException, IllegalAccessException {

        if ( port == null ) return;
        port.writeInt( DbCommands.Close );

        if ( port.readInt() == DbCommands.Failure ) {
            throw new IllegalAccessException(
                "Unable to close database");
        }
        port.close();
        port = null;
        open = false;
    }
}
