/**
 * Copyright(c) 1996 DTAI, Incorporated (http://www.dtai.com)
 *
 *                        All rights reserved
 *
 * Permission to use, copy, modify and distribute this material for
 * any purpose and without fee is hereby granted, provided that the
 * above copyright notice and this permission notice appear in all
 * copies, and that the name of DTAI, Incorporated not be used in
 * advertising or publicity pertaining to this material without the
 * specific, prior written permission of an authorized representative of
 * DTAI, Incorporated.
 *
 * DTAI, INCORPORATED MAKES NO REPRESENTATIONS AND EXTENDS NO WARRANTIES,
 * EXPRESS OR IMPLIED, WITH RESPECT TO THE SOFTWARE, INCLUDING, BUT
 * NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 * FITNESS FOR ANY PARTICULAR PURPOSE, AND THE WARRANTY AGAINST
 * INFRINGEMENT OF PATENTS OR OTHER INTELLECTUAL PROPERTY RIGHTS.  THE
 * SOFTWARE IS PROVIDED "AS IS", AND IN NO EVENT SHALL DTAI, INCORPORATED OR
 * ANY OF ITS AFFILIATES BE LIABLE FOR ANY DAMAGES, INCLUDING ANY
 * LOST PROFITS OR OTHER INCIDENTAL OR CONSEQUENTIAL DAMAGES RELATING
 * TO THE SOFTWARE.
 */

package COM.dtai.serial;

/**
 * DbCommands - holds the command IDs for communication between the
 * DbServer and a client (requests and status code responses).
 *
 * @version	1.0b1
 * @author	DTAI, Incorporated
 *
 * $Id: DbCommands.java,v 1.1 1996/08/09 01:21:18 kadel Exp $
 *
 * $Source: /cvs/java/classes/COM/dtai/serial/DbCommands.java,v $
 */

public abstract class DbCommands {
    /**
     * returned from server to indicate that the command was processed
     */
    public static final int Success	= 0;

    /**
     * returned from the server to indicate that the command could not
	 * be processed
     */
    public static final int Failure	= 1;

    /**
     * Returned from the server after a query if there is a record to return.
	 * This command status is followed by the record, and that is followed
	 * either by another SendingObject status, or a NoMoreRecords status.
     */
    public static final int SendingObject	= 2;

    /**
     * Returned from the server after a query if there are no more records to
	 * return.
     */
    public static final int NoMoreObjects	= 3;

    /**
     * This will open the database for reading
     */
    public static final int OpenForReading	= 4;

    /**
     * If not already open for writing, this will open the database for writing
	 * by this client only.
     */
    public static final int OpenForWriting	= 5;

    /**
     * Even if another client has the database open for writing, this will
	 * close that client's connection (saving changes first), open it for
	 * reading for that client, and open it for writing for this one.
     */
    public static final int ForceOpenForWriting = 6;

    /**
     * Saves current changes
     */
    public static final int Save = 7;

    /**
     * Saves current changes and closes the database (this does not
	 * close the connection to the server
     */
    public static final int Close = 8;

    /**
     * Throws away all changes made since the last save and reinitiallized
	 * with the current saved version
     */
    public static final int UndoAll	= 9;

    /**
     * command to server to enable routing Syslog messages to the default
	 * file, or to a port if specified with RerouteSyslog
     */
    public static final int EnableSyslog	= 10;

    /**
     * command to server to disable routing the Syslog to a file or port
     */
    public static final int DisableSyslog	= 11;

    /**
     * Command to server to re-route the Syslog to this client's port
     */
    public static final int RerouteSyslog	= 12;

}
