/**
 * Copyright(c) 1996 DTAI, Incorporated (http://www.dtai.com)
 *
 *                        All rights reserved
 *
 * Permission to use, copy, modify and distribute this material for
 * any purpose and without fee is hereby granted, provided that the
 * above copyright notice and this permission notice appear in all
 * copies, and that the name of DTAI, Incorporated not be used in
 * advertising or publicity pertaining to this material without the
 * specific, prior written permission of an authorized representative of
 * DTAI, Incorporated.
 *
 * DTAI, INCORPORATED MAKES NO REPRESENTATIONS AND EXTENDS NO WARRANTIES,
 * EXPRESS OR IMPLIED, WITH RESPECT TO THE SOFTWARE, INCLUDING, BUT
 * NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 * FITNESS FOR ANY PARTICULAR PURPOSE, AND THE WARRANTY AGAINST
 * INFRINGEMENT OF PATENTS OR OTHER INTELLECTUAL PROPERTY RIGHTS.  THE
 * SOFTWARE IS PROVIDED "AS IS", AND IN NO EVENT SHALL DTAI, INCORPORATED OR
 * ANY OF ITS AFFILIATES BE LIABLE FOR ANY DAMAGES, INCLUDING ANY
 * LOST PROFITS OR OTHER INCIDENTAL OR CONSEQUENTIAL DAMAGES RELATING
 * TO THE SOFTWARE.
 */

package COM.dtai.serial;

import COM.dtai.net.*;
import COM.dtai.util.*;
import java.net.*;
import java.io.*;
import java.util.*;

/**
 * DbData - holds database information
 *
 * @version	1.0b1
 * @author	DTAI, Incorporated
 *
 * $Id: DbServer.java,v 1.1 1996/08/09 01:21:18 kadel Exp $
 *
 * $Source: /cvs/java/classes/COM/dtai/serial/DbServer.java,v $
 */

public abstract class DbData
{

	protected String name;

    protected Vector read_managers = new Vector();
    protected Vector edit_managers = new Vector();

    protected Vector ports = new Vector();
    protected DbPortToClient current_editor = null;

	/**
	 * DESCRIPTION
	 *
	 * @param PARAMNAME DESCRIPTION
	 * @return DESCRIPTION
	 * @exception CLASSNAME DESCRIPTION
	 * @see CLASSNAME#METHOD
	 */
	public DbData( String name ) {
		this.name = name;
	}

	/**
	 * DESCRIPTION
	 *
	 * @param PARAMNAME DESCRIPTION
	 * @return DESCRIPTION
	 * @exception CLASSNAME DESCRIPTION
	 * @see CLASSNAME#METHOD
	 */
    protected void addPort( DbPortToClient port )
	{
		ports.addElement( port );
	}

	/**
	 * DESCRIPTION
	 * 
	 * @param PARAMNAME DESCRIPTION
	 * @return DESCRIPTION
	 * @exception CLASSNAME DESCRIPTION
	 * @see CLASSNAME#METHOD
	 */
	protected void addReadManager( SerialDataManager reader ) {
		read_managers.addElement( reader );
	}

	/**
	 * DESCRIPTION
	 * 
	 * @param PARAMNAME DESCRIPTION
	 * @return DESCRIPTION
	 * @exception CLASSNAME DESCRIPTION
	 * @see CLASSNAME#METHOD
	 */
	protected void addEditManager( SerialDataManager editor ) {
		edit_managers.addElement( editor );
	}

	/**
	 * DESCRIPTION
	 *
	 * @param PARAMNAME DESCRIPTION
	 * @return DESCRIPTION
	 * @exception CLASSNAME DESCRIPTION
	 * @see CLASSNAME#METHOD
	 */
    public synchronized void openForReading( DbPortToClient port )
		throws IOException, SerialDataException {

		for ( Enumeration e = read_managers.elements(); e.hasMoreElements(); ) {
			SerialDataManager mgr = (SerialDataManager)e.nextElement();
			mgr.open();
		}
	}

	/**
	 * DESCRIPTION
	 *
	 * @param PARAMNAME DESCRIPTION
	 * @return DESCRIPTION
	 * @exception CLASSNAME DESCRIPTION
	 * @see CLASSNAME#METHOD
	 */
    public synchronized void openForWriting( DbPortToClient port )
		throws IOException, SerialDataException, IllegalAccessException
	{
        if (port.isEditor()) return;

		if (current_editor != null) throw new IllegalAccessException();

        current_editor = port;

		for ( Enumeration e = edit_managers.elements(); e.hasMoreElements(); ) {
			SerialDataManager mgr = (SerialDataManager)e.nextElement();
			mgr.open();
		}
    }

	/**
	 * DESCRIPTION
	 *
	 * @param PARAMNAME DESCRIPTION
	 * @return DESCRIPTION
	 * @exception CLASSNAME DESCRIPTION
	 * @see CLASSNAME#METHOD
	 */
    public synchronized void forceOpenForWriting( DbPortToClient port )
	throws IOException, SerialDataException
	{
        if (port.isEditor()) return;

		for ( Enumeration e = edit_managers.elements(); e.hasMoreElements(); ) {
			SerialDataManager mgr = (SerialDataManager)e.nextElement();
			mgr.save();
		}

        current_editor = port;
    }

	/**
	 * DESCRIPTION
	 *
	 * @param PARAMNAME DESCRIPTION
	 * @return DESCRIPTION
	 * @exception CLASSNAME DESCRIPTION
	 * @see CLASSNAME#METHOD
	 */
    public synchronized void save( DbPortToClient port )
	throws IOException, SerialDataException, IllegalAccessException
	{
		if (! port.isEditor()) throw new IllegalAccessException();

		for ( Enumeration e = read_managers.elements(); e.hasMoreElements(); ) {
			SerialDataManager mgr = (SerialDataManager)e.nextElement();
			mgr.close();
		}

		for ( Enumeration e = edit_managers.elements(); e.hasMoreElements(); ) {
			SerialDataManager mgr = (SerialDataManager)e.nextElement();
			mgr.save();
		}

		for ( Enumeration e = read_managers.elements(); e.hasMoreElements(); ) {
			SerialDataManager mgr = (SerialDataManager)e.nextElement();
			mgr.open();
		}
    }

	/**
	 * DESCRIPTION
	 *
	 * @param PARAMNAME DESCRIPTION
	 * @return DESCRIPTION
	 * @exception CLASSNAME DESCRIPTION
	 * @see CLASSNAME#METHOD
	 */
    public synchronized void close( DbPortToClient port )
	throws IOException, SerialDataException
	{
		ports.removeElement(port);

		Syslog.warning("Port count = " + ports.size());

        if ( ( port.isEditor() ) ||
		     ( ports.size() == 0 ) ) {

			for ( Enumeration e = read_managers.elements();
				  e.hasMoreElements(); ) {
				SerialDataManager mgr = (SerialDataManager)e.nextElement();
				mgr.close();
			}

			for ( Enumeration e = edit_managers.elements();
				  e.hasMoreElements(); ) {
				SerialDataManager mgr = (SerialDataManager)e.nextElement();
				mgr.close();
			}
		}

        if ( port.isEditor() ) {

			if ( ports.size() > 0 ) {
				for ( Enumeration e = read_managers.elements();
					  e.hasMoreElements(); ) {
					SerialDataManager mgr = (SerialDataManager)e.nextElement();
					mgr.open();
				}
			}

			current_editor = null;
		}

		port.setDbData( null );
    }

	/**
	 * DESCRIPTION
	 *
	 * @param PARAMNAME DESCRIPTION
	 * @return DESCRIPTION
	 * @exception CLASSNAME DESCRIPTION
	 * @see CLASSNAME#METHOD
	 */
    public synchronized void undoAll( DbPortToClient port )
	throws IOException, SerialDataException, IllegalAccessException
	{
		if (! port.isEditor()) throw new IllegalAccessException();

		for ( Enumeration e = edit_managers.elements(); e.hasMoreElements(); ) {
			SerialDataManager mgr = (SerialDataManager)e.nextElement();
			mgr.undo();
		}
    }
}
