/**
 * Copyright(c) 1996 DTAI, Incorporated (http://www.dtai.com)
 *
 *                        All rights reserved
 *
 * Permission to use, copy, modify and distribute this material for
 * any purpose and without fee is hereby granted, provided that the
 * above copyright notice and this permission notice appear in all
 * copies, and that the name of DTAI, Incorporated not be used in
 * advertising or publicity pertaining to this material without the
 * specific, prior written permission of an authorized representative of
 * DTAI, Incorporated.
 *
 * DTAI, INCORPORATED MAKES NO REPRESENTATIONS AND EXTENDS NO WARRANTIES,
 * EXPRESS OR IMPLIED, WITH RESPECT TO THE SOFTWARE, INCLUDING, BUT
 * NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 * FITNESS FOR ANY PARTICULAR PURPOSE, AND THE WARRANTY AGAINST
 * INFRINGEMENT OF PATENTS OR OTHER INTELLECTUAL PROPERTY RIGHTS.  THE
 * SOFTWARE IS PROVIDED "AS IS", AND IN NO EVENT SHALL DTAI, INCORPORATED OR
 * ANY OF ITS AFFILIATES BE LIABLE FOR ANY DAMAGES, INCLUDING ANY
 * LOST PROFITS OR OTHER INCIDENTAL OR CONSEQUENTIAL DAMAGES RELATING
 * TO THE SOFTWARE.
 */

package COM.dtai.serial;

import COM.dtai.net.*;
import COM.dtai.util.*;
import java.net.*;
import java.io.*;
import java.util.*;

/**
 * DbPortToClient - manages the communication from the DbServer to one client
 *
 * @version	1.0b1
 * @author	DTAI, Incorporated
 *
 * $Id: DbPortToClient.java,v 1.2 1996/08/15 08:16:55 kadel Exp $
 *
 * $Source: /cvs/java/classes/COM/dtai/serial/DbPortToClient.java,v $
 */

public class DbPortToClient extends NetPortToClient
{
    private DbServer server;
    private DbData dbdata;

	/**
	 * DESCRIPTION
	 * 
	 * @param NAME DESCRIPTION
	 * @return DESCRIPTION
	 * @exception DESCRIPTION
	 */
    public DbPortToClient( DbServer server, Socket socket )
        throws IOException {

        super( server, socket );
        this.server = server;
        start(); // as a thread
    }

	/**
	 * DESCRIPTION
	 * 
	 * @param NAME DESCRIPTION
	 * @return DESCRIPTION
	 * @exception DESCRIPTION
	 */
    protected void setDbData( DbData dbdata ) {
		this.dbdata = dbdata;
	}

	/**
	 * DESCRIPTION
	 * 
	 * @param NAME DESCRIPTION
	 * @return DESCRIPTION
	 * @exception DESCRIPTION
	 */
    public boolean isEditor() {
		return ( dbdata.current_editor == this );
	}

    private void closeThread(Exception exc) {
		Syslog.error(exc);

		try {
			if ( dbdata != null ) {
				dbdata.close(this);
			}
			stop();
		}
		catch (Exception exp) {
			removeThread(exp);
		}
    }

    private void removeThread(Exception exc) {
		Syslog.error(exc);
		if ( dbdata != null ) {
			dbdata.ports.removeElement(this);
			if (isEditor()) {
				dbdata.current_editor = null;
			}
		}
		stop();
    }

	private void sendFailure(Exception exc) {
		Syslog.error(exc);
		try {
			writeInt( DbCommands.Failure );
		}
		catch (IOException ioe) {
			closeThread(ioe);
		}
	}

    protected void readInput() {
		int command = -1;

		try {
			command = readInt();
		}
		catch (IOException ioe) {
			closeThread(ioe);
		}

        try {
            process( command );
            flush();
        }
        catch (IllegalAccessException iae) {
            sendFailure(iae);
        }
        catch (FileNotFoundException fnf) {
			sendFailure(fnf);
        }
        catch (Exception exp) {
			removeThread(exp);
        }
    }

	/**
	 * DESCRIPTION
	 * 
	 * @param command The integer command from DbCommands or subclass
	 * @return true if the command was processed
	 * @exception IOException
	 * @exception SerialDataException Problem with the database
	 * @exception IllegalAccessException Client had no permission to perform operation
	 */
    protected synchronized boolean process( int command )
    throws IOException, SerialDataException, IllegalAccessException
	{
        if ( command == DbCommands.OpenForReading ) {
            String name = readUTF();
            Syslog.warning( "OpenForReading "+name );
			setDbData( server.getDbData( name ) );
			dbdata.openForReading( this );
			writeInt( DbCommands.Success );
        }
        else if ( command == DbCommands.OpenForWriting ) {
            String name = readUTF();
			Syslog.warning( "OpenForWriting "+name );
			setDbData( server.getDbData( name ) );
			dbdata.openForWriting( this );
			writeInt( DbCommands.Success );
        }
        else if ( command == DbCommands.ForceOpenForWriting ) {
            String name = readUTF();
            Syslog.warning( "ForceOpenForWriting "+name );
			setDbData( server.getDbData( name ) );
			dbdata.forceOpenForWriting( this );
			writeInt( DbCommands.Success );
        }
        else {

			if ( dbdata == null ) {
				throw new IllegalAccessException(
					"Attempt to use database without opening it." );
			}
		
			if ( command == DbCommands.Save ) {
				Syslog.warning( "Save" );
				dbdata.save( this );
				writeInt( DbCommands.Success );
			}
			else if ( command == DbCommands.Close ) {
				Syslog.warning( "Close" );
				dbdata.close( this );
				writeInt( DbCommands.Success );
			}
			else if ( command == DbCommands.UndoAll ) {
				Syslog.warning( "UndoAll" );
				dbdata.undoAll( this );
				writeInt( DbCommands.Success );
			}
			else if ( command == DbCommands.EnableSyslog ) {
				Syslog.warning( "EnableSyslog" );
				Syslog.enableLogging(true);
				writeInt( DbCommands.Success );
			}
			else if ( command == DbCommands.DisableSyslog ) {
				Syslog.warning( "DisableSyslog" );
				Syslog.enableLogging(false);
				writeInt( DbCommands.Success );
			}
			else if ( command == DbCommands.RerouteSyslog ) {
				Syslog.warning( "RerouteSyslog" );
				Syslog.setStream(getOutputStream());
				writeInt( DbCommands.Success );
			}
			else {
				return false;
	//            Syslog.error("Unknown Server command");	// commented out because
															// subclass may process
			}
        }
		return true;
    }
}
