/**
 * Copyright(c) 1996 DTAI, Incorporated (http://www.dtai.com)
 *
 *                        All rights reserved
 *
 * Permission to use, copy, modify and distribute this material for
 * any purpose and without fee is hereby granted, provided that the
 * above copyright notice and this permission notice appear in all
 * copies, and that the name of DTAI, Incorporated not be used in
 * advertising or publicity pertaining to this material without the
 * specific, prior written permission of an authorized representative of
 * DTAI, Incorporated.
 *
 * DTAI, INCORPORATED MAKES NO REPRESENTATIONS AND EXTENDS NO WARRANTIES,
 * EXPRESS OR IMPLIED, WITH RESPECT TO THE SOFTWARE, INCLUDING, BUT
 * NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 * FITNESS FOR ANY PARTICULAR PURPOSE, AND THE WARRANTY AGAINST
 * INFRINGEMENT OF PATENTS OR OTHER INTELLECTUAL PROPERTY RIGHTS.  THE
 * SOFTWARE IS PROVIDED "AS IS", AND IN NO EVENT SHALL DTAI, INCORPORATED OR
 * ANY OF ITS AFFILIATES BE LIABLE FOR ANY DAMAGES, INCLUDING ANY
 * LOST PROFITS OR OTHER INCIDENTAL OR CONSEQUENTIAL DAMAGES RELATING
 * TO THE SOFTWARE.
 */

package COM.dtai.serial;

import COM.dtai.net.*;
import COM.dtai.util.*;
import java.net.*;
import java.io.*;
import java.util.*;

/**
 * DbServer - listens for and accepts clients, then spawns a thread
 * to handle the client requests.
 *
 * @version	1.0b1
 * @author	DTAI, Incorporated
 *
 * $Id: DbServer.java,v 1.2 1996/08/15 08:16:55 kadel Exp $
 *
 * $Source: /cvs/java/classes/COM/dtai/serial/DbServer.java,v $
 */

public abstract class DbServer extends NetServer
{
    protected Hashtable dbdata_by_name = new Hashtable();

	/**
	 * DESCRIPTION
	 *
	 * @param PARAMNAME DESCRIPTION
	 * @return DESCRIPTION
	 * @exception CLASSNAME DESCRIPTION
	 * @see CLASSNAME#METHOD
	 */
    public DbServer( int portnum ) {
        super ( portnum );
    }

	/**
	 * DESCRIPTION
	 *
	 * @param PARAMNAME DESCRIPTION
	 * @return DESCRIPTION
	 * @exception CLASSNAME DESCRIPTION
	 * @see CLASSNAME#METHOD
	 */
    public static int getPortnum ( String argv[], int default_portnum ) {

        int portnum = -1;

        for ( int idx = 0; idx < argv.length; idx++ ) {
            if ( argv[idx].equals( "-port" ) ) {
                idx++;
                if ( idx < argv.length ) {
                    portnum = Integer.parseInt( argv[idx] );
                }
            }
            else {
                usage();
            }
        }

        if ( portnum < 0 ) {
            portnum = default_portnum;
            Syslog.warning ( "Using port " + portnum + " by default." );
        }

		Syslog.enableLogging ( false );

		return portnum;
	}

    public static void main ( String argv[] ) {

		Syslog.error( "Attempt to start abstract version of DbServer." );

// You should reimplement main, probably with something like the following
//        new YourDbServer( getPortnum( argv, 9090 ) );
    }

	/**
	 * DESCRIPTION
	 *
	 * @param PARAMNAME DESCRIPTION
	 * @return DESCRIPTION
	 * @exception CLASSNAME DESCRIPTION
	 * @see CLASSNAME#METHOD
	 */
    public abstract void createClientPort( Socket socket );
//	{
//		try {
//			new YourDbPortToClient ( this, socket );
//		}
//		catch( IOException ioe ) {
//			Syslog.error( "IO Exception creating client port handler object" );
//			Syslog.error( ioe );
//		}
//	}

	/**
	 * DESCRIPTION
	 *
	 * @param PARAMNAME DESCRIPTION
	 * @return DESCRIPTION
	 * @exception CLASSNAME DESCRIPTION
	 * @see CLASSNAME#METHOD
	 */
    public synchronized void error ( String message ) {
		Syslog.error(message);
    }

	/**
	 * DESCRIPTION
	 *
	 * @param PARAMNAME DESCRIPTION
	 * @return DESCRIPTION
	 * @exception CLASSNAME DESCRIPTION
	 * @see CLASSNAME#METHOD
	 */
    public synchronized void warning ( String message ) {
		Syslog.warning(message);
    }

    static protected void usage() {
        System.err.println ( "Usage: java DbServer -port portnum" );
        System.exit( -1 );
    }

	/**
	 * DESCRIPTION
	 *
	 * @param PARAMNAME DESCRIPTION
	 * @return DESCRIPTION
	 * @exception CLASSNAME DESCRIPTION
	 * @see CLASSNAME#METHOD
	 */
    public abstract DbData createDbData( String name );

	/**
	 * DESCRIPTION
	 * 
	 * @param PARAMNAME DESCRIPTION
	 * @return DESCRIPTION
	 * @exception CLASSNAME DESCRIPTION
	 * @see CLASSNAME#METHOD
	 */
	public DbData getDbData( String name ) {
		DbData dbdata = (DbData)dbdata_by_name.get( name );
		if ( dbdata == null ) {
			dbdata = createDbData( name );
			dbdata_by_name.put( name, dbdata );
		}
		return dbdata;
	}
}
