/**
 * Copyright(c) 1996 DTAI, Incorporated (http://www.dtai.com)
 *
 *                        All rights reserved
 *
 * Permission to use, copy, modify and distribute this material for
 * any purpose and without fee is hereby granted, provided that the
 * above copyright notice and this permission notice appear in all
 * copies, and that the name of DTAI, Incorporated not be used in
 * advertising or publicity pertaining to this material without the
 * specific, prior written permission of an authorized representative of
 * DTAI, Incorporated.
 *
 * DTAI, INCORPORATED MAKES NO REPRESENTATIONS AND EXTENDS NO WARRANTIES,
 * EXPRESS OR IMPLIED, WITH RESPECT TO THE SOFTWARE, INCLUDING, BUT
 * NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 * FITNESS FOR ANY PARTICULAR PURPOSE, AND THE WARRANTY AGAINST
 * INFRINGEMENT OF PATENTS OR OTHER INTELLECTUAL PROPERTY RIGHTS.  THE
 * SOFTWARE IS PROVIDED "AS IS", AND IN NO EVENT SHALL DTAI, INCORPORATED OR
 * ANY OF ITS AFFILIATES BE LIABLE FOR ANY DAMAGES, INCLUDING ANY
 * LOST PROFITS OR OTHER INCIDENTAL OR CONSEQUENTIAL DAMAGES RELATING
 * TO THE SOFTWARE.
 */

package COM.dtai.serial;

import java.io.*;
import java.util.*;

/**
 * SerialDataFile - manages interface between data file and indexes
 *
 * @version	1.0b1
 * @author	DTAI, Incorporated
 *
 * $Id: SerialDataFile.java,v 1.8 1996/08/09 01:21:19 kadel Exp $
 *
 * $Source: /cvs/java/classes/COM/dtai/serial/SerialDataFile.java,v $
 */

public class SerialDataFile extends RandomAccessFile
{
	private Vector indexes = new Vector();
	private boolean editable;
	private SerialData sd;

	/**
	 * DESCRIPTION
	 * 
	 * @param PARAMNAME DESCRIPTION
	 * @return DESCRIPTION
	 * @exception CLASSNAME DESCRIPTION
	 * @see CLASSNAME#METHOD
	 */
    public SerialDataFile(String name, String mode, SerialData sd)
	throws IOException
    {
		super(name, mode);
		this.editable = (mode.compareTo("rw") == 0);
		this.sd = sd;
    }

	/**
	 * DESCRIPTION
	 * 
	 * @param PARAMNAME DESCRIPTION
	 * @return DESCRIPTION
	 * @exception CLASSNAME DESCRIPTION
	 * @see CLASSNAME#METHOD
	 */
	public boolean isEditable()
	{
		return editable;
	}

	/**
	 * DESCRIPTION
	 * 
	 * @param PARAMNAME DESCRIPTION
	 * @return DESCRIPTION
	 * @exception CLASSNAME DESCRIPTION
	 * @see CLASSNAME#METHOD
	 */
	public void validate()
	throws SerialDataException
	{
		if (indexes.isEmpty())
			throw new SerialDataException("Empty index list");
	}

	/**
	 * DESCRIPTION
	 * 
	 * @param PARAMNAME DESCRIPTION
	 * @return DESCRIPTION
	 * @exception CLASSNAME DESCRIPTION
	 * @see CLASSNAME#METHOD
	 */
	public void validate(SerialDataIndex sdi)
	throws SerialDataException
	{
		if (!indexes.contains(sdi))
			throw new SerialDataException("Invalid index");
	}

	/**
	 * DESCRIPTION
	 * 
	 * @param PARAMNAME DESCRIPTION
	 * @return DESCRIPTION
	 * @exception CLASSNAME DESCRIPTION
	 * @see CLASSNAME#METHOD
	 */
	public void addIndex(SerialDataIndex sdi)
	{
		indexes.addElement(sdi);
	}

	/**
	 * DESCRIPTION
	 * 
	 * @param PARAMNAME DESCRIPTION
	 * @return DESCRIPTION
	 * @exception CLASSNAME DESCRIPTION
	 * @see CLASSNAME#METHOD
	 */
	public synchronized Enumeration elements(SerialDataIndex sdi, SerialData sd)
	throws IOException, SerialDataException
	{
		validate(sdi);

		SerialDataValue sdv = sdi.getValue(sd);

		if (sdv == null) return null;

		return sdv.elements();
	}

	/**
	 * DESCRIPTION
	 * 
	 * @param PARAMNAME DESCRIPTION
	 * @return DESCRIPTION
	 * @exception CLASSNAME DESCRIPTION
	 * @see CLASSNAME#METHOD
	 */
	public synchronized boolean loadNextElement(Enumeration fps, SerialData sd)
	throws IOException
	{
		if ( fps == null ) return false;
		if (!fps.hasMoreElements()) return false;

		seek(((Long)fps.nextElement()).longValue());
		sd.read(this);

		return true;
	}

	/**
	 * DESCRIPTION
	 * 
	 * @param PARAMNAME DESCRIPTION
	 * @return DESCRIPTION
	 * @exception CLASSNAME DESCRIPTION
	 * @see CLASSNAME#METHOD
	 */
	public void clear()
	{
    	for (Enumeration e = indexes.elements(); e.hasMoreElements();) {
    	    SerialDataIndex index = (SerialDataIndex)e.nextElement();
			index.clear();
    	}
	}


	/**
	 * DESCRIPTION
	 * 
	 * @param PARAMNAME DESCRIPTION
	 * @return DESCRIPTION
	 * @exception CLASSNAME DESCRIPTION
	 * @see CLASSNAME#METHOD
	 */
	public SerialDataIndex findUniqueIndex()
	throws SerialDataException
	{
		validate();

		for (Enumeration e = indexes.elements(); e.hasMoreElements();) {
			SerialDataIndex index = (SerialDataIndex)e.nextElement();
			if (index.isUnique()) return index;
		}

		throw new SerialDataException("Unique index not found");
	}

	/**
	 * DESCRIPTION
	 * 
	 * @param PARAMNAME DESCRIPTION
	 * @return DESCRIPTION
	 * @exception CLASSNAME DESCRIPTION
	 * @see CLASSNAME#METHOD
	 */
	public synchronized void initialize()
	throws IOException, SerialDataException
	{
		validate();

		clear();
		seek(0);

    	try {
    	    while (true) {
				long filepos = getFilePointer();
				sd.read(this);
				for (Enumeration e = indexes.elements(); e.hasMoreElements();) {
					SerialDataIndex index = (SerialDataIndex)e.nextElement();
					index.insert(sd, filepos);
				}
    	    }
    	}
    	catch (EOFException eof) {
    	}
	}

	/**
	 * DESCRIPTION
	 * 
	 * @param PARAMNAME DESCRIPTION
	 * @return DESCRIPTION
	 * @exception CLASSNAME DESCRIPTION
	 * @see CLASSNAME#METHOD
	 */
	public synchronized void update(SerialDataIndex sdi, SerialData sd)
	throws IOException, SerialDataException, IllegalAccessException
	{
		validate(sdi);

		if (!sdi.isUnique())
			throw new SerialDataException("Cannot update non-unique index");

		if (!editable)
			throw new IllegalAccessException("Data is read only");

		SerialDataValue sdv = sdi.getValue(sd);
		long newpos = length();
		seek(newpos);
		sd.write(this);

		if (sdv == null) {
			for (Enumeration e = indexes.elements(); e.hasMoreElements();) {
				SerialDataIndex index = (SerialDataIndex)e.nextElement();
				index.insert(sd, newpos);
			}
		}
		else {
			for (Enumeration fps = sdv.elements(); fps.hasMoreElements();) {
				long oldpos = ((Long)fps.nextElement()).longValue();
				seek(oldpos);
				sd.read(this);

				for (Enumeration e = indexes.elements(); e.hasMoreElements();) {
					SerialDataIndex index = (SerialDataIndex)e.nextElement();
					index.update(sd, oldpos, newpos);
				}
			}
    	}
	}

	/**
	 * DESCRIPTION
	 * 
	 * @param PARAMNAME DESCRIPTION
	 * @return DESCRIPTION
	 * @exception CLASSNAME DESCRIPTION
	 * @see CLASSNAME#METHOD
	 */
	public void update(SerialData sd)
	throws IOException, SerialDataException, IllegalAccessException
	{
		update(findUniqueIndex(), sd);
	}

	/**
	 * DESCRIPTION
	 * 
	 * @param PARAMNAME DESCRIPTION
	 * @return DESCRIPTION
	 * @exception CLASSNAME DESCRIPTION
	 * @see CLASSNAME#METHOD
	 */
	public synchronized void delete(SerialDataIndex sdi, SerialData sd)
	throws IOException, SerialDataException, IllegalAccessException
	{
		validate(sdi);

		if (!editable)
			throw new IllegalAccessException("Data is read only");

		SerialDataValue sdv = sdi.getValue(sd);

		if (sdv == null) return;

		for (Enumeration fps = sdv.elements(); fps.hasMoreElements();) {
			long filepos = ((Long)fps.nextElement()).longValue();
			seek(filepos);
			sd.read(this);

			for (Enumeration e = indexes.elements(); e.hasMoreElements();) {
				SerialDataIndex index = (SerialDataIndex)e.nextElement();
				index.delete(sd, filepos);
			}
    	}
	}

	/**
	 * DESCRIPTION
	 * 
	 * @param PARAMNAME DESCRIPTION
	 * @return DESCRIPTION
	 * @exception CLASSNAME DESCRIPTION
	 * @see CLASSNAME#METHOD
	 */
	public void delete(SerialData sd)
	throws IOException, SerialDataException, IllegalAccessException
	{
		delete(findUniqueIndex(), sd);
	}

	/**
	 * DESCRIPTION
	 * 
	 * @param PARAMNAME DESCRIPTION
	 * @return DESCRIPTION
	 * @exception CLASSNAME DESCRIPTION
	 * @see CLASSNAME#METHOD
	 */
    public synchronized void transfer(RandomAccessFile raf)
	throws IOException, SerialDataException
	{
		validate();

		SerialDataIndex sdi = findUniqueIndex();

    	for (Enumeration e = sdi.elements(); e.hasMoreElements();) {
    	    SerialDataValue sdv = (SerialDataValue)e.nextElement();

			for (Enumeration fps = sdv.elements(); fps.hasMoreElements();) {
				seek(((Long)fps.nextElement()).longValue());
				sd.read(this);
				sd.write(raf);
			}
    	}
	}

	/**
	 * DESCRIPTION
	 * 
	 * @param PARAMNAME DESCRIPTION
	 * @return DESCRIPTION
	 * @exception CLASSNAME DESCRIPTION
	 * @see CLASSNAME#METHOD
	 */
    public synchronized void print(SerialDataIndex sdi, PrintStream ps)
	throws IOException, SerialDataException
	{
		validate(sdi);

    	for (Enumeration e = sdi.elements(); e.hasMoreElements();) {
    	    SerialDataValue sdv = (SerialDataValue)e.nextElement();

			for (Enumeration fps = sdv.elements(); fps.hasMoreElements();) {
				seek(((Long)fps.nextElement()).longValue());
				sd.read(this);
				sd.print(ps);
			}
    	}
	}

	/**
	 * DESCRIPTION
	 * 
	 * @param PARAMNAME DESCRIPTION
	 * @return DESCRIPTION
	 * @exception CLASSNAME DESCRIPTION
	 * @see CLASSNAME#METHOD
	 */
	public void print(SerialDataIndex sdi)
	throws IOException, SerialDataException
	{
		print(sdi, System.out);
	}

	/**
	 * DESCRIPTION
	 * 
	 * @param PARAMNAME DESCRIPTION
	 * @return DESCRIPTION
	 * @exception CLASSNAME DESCRIPTION
	 * @see CLASSNAME#METHOD
	 */
	public void print()
	throws IOException, SerialDataException
	{
		print(findUniqueIndex(), System.out);
	}
}
