/**
 * Copyright(c) 1996 DTAI, Incorporated (http://www.dtai.com)
 *
 *                        All rights reserved
 *
 * Permission to use, copy, modify and distribute this material for
 * any purpose and without fee is hereby granted, provided that the
 * above copyright notice and this permission notice appear in all
 * copies, and that the name of DTAI, Incorporated not be used in
 * advertising or publicity pertaining to this material without the
 * specific, prior written permission of an authorized representative of
 * DTAI, Incorporated.
 *
 * DTAI, INCORPORATED MAKES NO REPRESENTATIONS AND EXTENDS NO WARRANTIES,
 * EXPRESS OR IMPLIED, WITH RESPECT TO THE SOFTWARE, INCLUDING, BUT
 * NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 * FITNESS FOR ANY PARTICULAR PURPOSE, AND THE WARRANTY AGAINST
 * INFRINGEMENT OF PATENTS OR OTHER INTELLECTUAL PROPERTY RIGHTS.  THE
 * SOFTWARE IS PROVIDED "AS IS", AND IN NO EVENT SHALL DTAI, INCORPORATED OR
 * ANY OF ITS AFFILIATES BE LIABLE FOR ANY DAMAGES, INCLUDING ANY
 * LOST PROFITS OR OTHER INCIDENTAL OR CONSEQUENTIAL DAMAGES RELATING
 * TO THE SOFTWARE.
 */

package COM.dtai.serial;

import java.util.*;

/**
 * SerialDataIndex - hash table of SerialDataValue objects
 *
 * @version	1.0b1
 * @author	DTAI, Incorporated
 *
 * $Id: SerialDataIndex.java,v 1.4 1996/08/09 01:21:20 kadel Exp $
 *
 * $Source: /cvs/java/classes/COM/dtai/serial/SerialDataIndex.java,v $
 */

public abstract class SerialDataIndex extends Hashtable
{
	private boolean unique;

	/**
	 * DESCRIPTION
	 * 
	 * @param PARAMNAME DESCRIPTION
	 * @return DESCRIPTION
	 * @exception CLASSNAME DESCRIPTION
	 * @see CLASSNAME#METHOD
	 */
    public SerialDataIndex(boolean unique)
    {
		super();
		this.unique = unique;
    }

	/**
	 * DESCRIPTION
	 * 
	 * @param PARAMNAME DESCRIPTION
	 * @return DESCRIPTION
	 * @exception CLASSNAME DESCRIPTION
	 * @see CLASSNAME#METHOD
	 */
    protected abstract Object key(SerialData sd);

	/**
	 * DESCRIPTION
	 * 
	 * @param PARAMNAME DESCRIPTION
	 * @return DESCRIPTION
	 * @exception CLASSNAME DESCRIPTION
	 * @see CLASSNAME#METHOD
	 */
    protected SerialDataValue value() {
		if ( unique ) {
			return new SerialSingleValue();
		}
		else {
			return new SerialVectorValue();
		}
	}

	/**
	 * DESCRIPTION
	 * 
	 * @param PARAMNAME DESCRIPTION
	 * @return DESCRIPTION
	 * @exception CLASSNAME DESCRIPTION
	 * @see CLASSNAME#METHOD
	 */
	public boolean isUnique()
	{
		return unique;
	}

	/**
	 * DESCRIPTION
	 * 
	 * @param PARAMNAME DESCRIPTION
	 * @return DESCRIPTION
	 * @exception CLASSNAME DESCRIPTION
	 * @see CLASSNAME#METHOD
	 */
    public SerialDataValue getValue(SerialData sd)
    {
		return (SerialDataValue)get(key(sd));
    }

	/**
	 * DESCRIPTION
	 * 
	 * @param PARAMNAME DESCRIPTION
	 * @return DESCRIPTION
	 * @exception CLASSNAME DESCRIPTION
	 * @see CLASSNAME#METHOD
	 */
    public void insert(SerialData sd, long filepos)
    {
		SerialDataValue sdv = getValue(sd);

		if (sdv == null) {
			sdv = value();
			put(key(sd), sdv);
		}

		sdv.addPos(sd, filepos);
    }

	/**
	 * DESCRIPTION
	 * 
	 * @param PARAMNAME DESCRIPTION
	 * @return DESCRIPTION
	 * @exception CLASSNAME DESCRIPTION
	 * @see CLASSNAME#METHOD
	 */
    public void update(SerialData sd, long oldpos, long newpos)
    {
		SerialDataValue sdv = getValue(sd);

		if (sdv == null) {
			sdv = value();
			put(key(sd), sdv);
		}

		sdv.delPos(sd, oldpos);
		sdv.addPos(sd, newpos);
    }

	/**
	 * DESCRIPTION
	 * 
	 * @param PARAMNAME DESCRIPTION
	 * @return DESCRIPTION
	 * @exception CLASSNAME DESCRIPTION
	 * @see CLASSNAME#METHOD
	 */
    public void delete(SerialData sd, long filepos)
    {
		SerialDataValue sdv = getValue(sd);

		if (sdv != null) {
			sdv.delPos(sd, filepos);
			if (sdv.getCount() == 0) remove(key(sd));
		}
    }
}
