/**
 * Copyright(c) 1996 DTAI, Incorporated (http://www.dtai.com)
 *
 *                        All rights reserved
 *
 * Permission to use, copy, modify and distribute this material for
 * any purpose and without fee is hereby granted, provided that the
 * above copyright notice and this permission notice appear in all
 * copies, and that the name of DTAI, Incorporated not be used in
 * advertising or publicity pertaining to this material without the
 * specific, prior written permission of an authorized representative of
 * DTAI, Incorporated.
 *
 * DTAI, INCORPORATED MAKES NO REPRESENTATIONS AND EXTENDS NO WARRANTIES,
 * EXPRESS OR IMPLIED, WITH RESPECT TO THE SOFTWARE, INCLUDING, BUT
 * NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 * FITNESS FOR ANY PARTICULAR PURPOSE, AND THE WARRANTY AGAINST
 * INFRINGEMENT OF PATENTS OR OTHER INTELLECTUAL PROPERTY RIGHTS.  THE
 * SOFTWARE IS PROVIDED "AS IS", AND IN NO EVENT SHALL DTAI, INCORPORATED OR
 * ANY OF ITS AFFILIATES BE LIABLE FOR ANY DAMAGES, INCLUDING ANY
 * LOST PROFITS OR OTHER INCIDENTAL OR CONSEQUENTIAL DAMAGES RELATING
 * TO THE SOFTWARE.
 */

package COM.dtai.serial;

import java.io.*;
import java.util.*;

/**
 * SerialDataManager - file manager for SerialDataFile class
 *
 * @version	1.0b1
 * @author	DTAI, Incorporated
 *
 * $Id: SerialDataManager.java,v 1.11 1996/08/09 01:21:20 kadel Exp $
 *
 * $Source: /cvs/java/classes/COM/dtai/serial/SerialDataManager.java,v $
 */

public abstract class SerialDataManager
{
    private String dir = "";
    private String name = "default";
    private boolean editable = false;
	private protected SerialDataFile sdf = null;

	/**
	 * DESCRIPTION
	 * 
	 * @param PARAMNAME DESCRIPTION
	 * @return DESCRIPTION
	 * @exception CLASSNAME DESCRIPTION
	 * @see CLASSNAME#METHOD
	 */
    public SerialDataManager(String dir, String name, boolean editable)
    {
		if (dir != null) this.dir = dir;
		if (name != null) this.name = name;
		this.editable = editable;
    }

    private boolean fileExists(String name)
    {
    	File file = new File(name);
    	return file.exists();
    }

    private String readFileName()
    {
    	return dir + name + ".dat";
    }

    private String editFileName()
    {
    	return dir + name + ".edt";
    }

    private String tempFileName()
    {
    	return dir + name + ".tmp";
    }

	private void copyFile(String name1, String name2)
	throws IOException
	{
    	RandomAccessFile raf1 = new RandomAccessFile(name1, "r");
    	RandomAccessFile raf2 = new RandomAccessFile(name2, "rw");

    	int num_read;
    	byte[] tmpbuf = new byte[1024];

    	while ((num_read = raf1.read(tmpbuf)) > 0)
    	    raf2.write(tmpbuf, 0, num_read);

    	raf1.close();
    	raf2.close();
	}

    private void renameTempFiles()
	throws IOException
    {
		String rname = readFileName();
		String ename = editFileName();
		String tname = tempFileName();

		File rfile = new File(rname);
		File efile = new File(ename);
		File tfile = new File(tname);

		rfile.delete();
		copyFile(tname, rname);
		if (!efile.delete()) throw new IOException("Delete failed: " + ename);
		if (!tfile.delete()) throw new IOException("Delete failed: " + tname);
    }

    private void removeTempFiles()
    {
    	File rfile = new File(readFileName());
    	File efile = new File(editFileName());
    	File tfile = new File(tempFileName());

    	if (tfile.exists()) tfile.delete();
    	if (efile.exists()) efile.delete();
    }

	/**
	 * DESCRIPTION
	 * 
	 * @param PARAMNAME DESCRIPTION
	 * @return DESCRIPTION
	 * @exception CLASSNAME DESCRIPTION
	 * @see CLASSNAME#METHOD
	 */
	protected abstract SerialData newData();

	/**
	 * DESCRIPTION
	 * 
	 * @param PARAMNAME DESCRIPTION
	 * @return DESCRIPTION
	 * @exception CLASSNAME DESCRIPTION
	 * @see CLASSNAME#METHOD
	 */
	protected abstract void addIndexes();

	/**
	 * DESCRIPTION
	 * 
	 * @param PARAMNAME DESCRIPTION
	 * @return DESCRIPTION
	 * @exception CLASSNAME DESCRIPTION
	 * @see CLASSNAME#METHOD
	 */
	public boolean isEditable()
	{
		return editable;
	}

	/**
	 * DESCRIPTION
	 * 
	 * @param PARAMNAME DESCRIPTION
	 * @return DESCRIPTION
	 * @exception CLASSNAME DESCRIPTION
	 * @see CLASSNAME#METHOD
	 */
	public boolean isOpen()
	{
		return (sdf != null);
	}

	/**
	 * DESCRIPTION
	 * 
	 * @param PARAMNAME DESCRIPTION
	 * @return DESCRIPTION
	 * @exception CLASSNAME DESCRIPTION
	 * @see CLASSNAME#METHOD
	 */
	public void validate()
	throws SerialDataException
	{
		if (sdf == null) throw new SerialDataException();
		sdf.validate();
	}

	/**
	 * DESCRIPTION
	 * 
	 * @param PARAMNAME DESCRIPTION
	 * @return DESCRIPTION
	 * @exception CLASSNAME DESCRIPTION
	 * @see CLASSNAME#METHOD
	 */
	public void validate(SerialDataIndex sdi)
	throws SerialDataException
	{
		if (sdf == null) throw new SerialDataException();
		sdf.validate(sdi);
	}

	/**
	 * DESCRIPTION
	 * 
	 * @param PARAMNAME DESCRIPTION
	 * @return DESCRIPTION
	 * @exception CLASSNAME DESCRIPTION
	 * @see CLASSNAME#METHOD
	 */
	public synchronized Enumeration elements(SerialDataIndex sdi, SerialData sd)
	throws IOException, SerialDataException
	{
		if (sdf == null) throw new SerialDataException();
		return sdf.elements( sdi, sd );
	}

	/**
	 * DESCRIPTION
	 * 
	 * @param PARAMNAME DESCRIPTION
	 * @return DESCRIPTION
	 * @exception CLASSNAME DESCRIPTION
	 * @see CLASSNAME#METHOD
	 */
	public synchronized boolean loadNextElement(Enumeration fps, SerialData sd)
	throws IOException
	{
		return sdf.loadNextElement( fps, sd );
	}

	/**
	 * DESCRIPTION
	 * 
	 * @param PARAMNAME DESCRIPTION
	 * @return DESCRIPTION
	 * @exception CLASSNAME DESCRIPTION
	 * @see CLASSNAME#METHOD
	 */
	public synchronized void delete(SerialDataIndex sdi, SerialData sd)
	throws IOException, SerialDataException, IllegalAccessException
	{
		if (sdf == null) throw new SerialDataException();
		sdf.delete( sdi, sd );
	}

	/**
	 * DESCRIPTION
	 * 
	 * @param PARAMNAME DESCRIPTION
	 * @return DESCRIPTION
	 * @exception CLASSNAME DESCRIPTION
	 * @see CLASSNAME#METHOD
	 */
	public void delete(SerialData sd)
	throws IOException, SerialDataException, IllegalAccessException
	{
		if (sdf == null) throw new SerialDataException();
		sdf.delete( sd );
	}

	/**
	 * DESCRIPTION
	 * 
	 * @param PARAMNAME DESCRIPTION
	 * @return DESCRIPTION
	 * @exception CLASSNAME DESCRIPTION
	 * @see CLASSNAME#METHOD
	 */
	public synchronized void update(SerialDataIndex sdi, SerialData sd)
	throws IOException, SerialDataException, IllegalAccessException
	{
		if (sdf == null) throw new SerialDataException();
		sdf.update( sdi, sd );
	}

	/**
	 * DESCRIPTION
	 * 
	 * @param PARAMNAME DESCRIPTION
	 * @return DESCRIPTION
	 * @exception CLASSNAME DESCRIPTION
	 * @see CLASSNAME#METHOD
	 */
	public void update(SerialData sd)
	throws IOException, SerialDataException, IllegalAccessException
	{
		if (sdf == null) throw new SerialDataException();
		sdf.update( sd );
	}

	/**
	 * DESCRIPTION
	 * 
	 * @param PARAMNAME DESCRIPTION
	 * @return DESCRIPTION
	 * @exception CLASSNAME DESCRIPTION
	 * @see CLASSNAME#METHOD
	 */
	public SerialDataFile getFile()
	{
		return sdf;
	}

	/**
	 * DESCRIPTION
	 * 
	 * @param PARAMNAME DESCRIPTION
	 * @return DESCRIPTION
	 * @exception CLASSNAME DESCRIPTION
	 * @see CLASSNAME#METHOD
	 */
    public synchronized void open(boolean editable)
	throws IOException, SerialDataException
    {
		if (sdf != null) return;

		this.editable = editable;

    	if (editable) {
			removeTempFiles();

			String rname = readFileName();
			String ename = editFileName();

    	    if (fileExists(rname)) copyFile(rname, ename);
    	    sdf = new SerialDataFile(editFileName(), "rw", newData());
    	}
    	else {
			String rname = readFileName();

			if (!fileExists(rname))
				throw new FileNotFoundException("Open failed: " + rname);

			sdf = new SerialDataFile(rname, "r", newData());
		}

		addIndexes();

		sdf.initialize();
    }

	/**
	 * DESCRIPTION
	 * 
	 * @param PARAMNAME DESCRIPTION
	 * @return DESCRIPTION
	 * @exception CLASSNAME DESCRIPTION
	 * @see CLASSNAME#METHOD
	 */
	public synchronized void open()
	throws IOException, SerialDataException
	{
		open(editable);
	}

	/**
	 * DESCRIPTION
	 * 
	 * @param PARAMNAME DESCRIPTION
	 * @return DESCRIPTION
	 * @exception CLASSNAME DESCRIPTION
	 * @see CLASSNAME#METHOD
	 */
    public synchronized void close()
    throws IOException, SerialDataException
    {
		if (sdf == null) return;

    	if (editable) {
			RandomAccessFile raf = new RandomAccessFile(tempFileName(), "rw");

			sdf.transfer(raf);
			raf.close();
			sdf.close();

			renameTempFiles();
    	}
		else sdf.close();

		sdf = null;
    }

	/**
	 * DESCRIPTION
	 * 
	 * @param PARAMNAME DESCRIPTION
	 * @return DESCRIPTION
	 * @exception CLASSNAME DESCRIPTION
	 * @see CLASSNAME#METHOD
	 */
    public void discard()
    throws IOException
    {
		if (sdf == null) return;
    	sdf.close();
		sdf = null;
		removeTempFiles();
    }

	/**
	 * DESCRIPTION
	 * 
	 * @param PARAMNAME DESCRIPTION
	 * @return DESCRIPTION
	 * @exception CLASSNAME DESCRIPTION
	 * @see CLASSNAME#METHOD
	 */
    public synchronized void save()
	throws IOException, SerialDataException
    {
		close();
		open();
    }

	/**
	 * DESCRIPTION
	 * 
	 * @param PARAMNAME DESCRIPTION
	 * @return DESCRIPTION
	 * @exception CLASSNAME DESCRIPTION
	 * @see CLASSNAME#METHOD
	 */
    public synchronized void undo()
	throws IOException, SerialDataException
    {
		discard();
		open();
    }
}
