/**
 * Copyright(c) 1996 DTAI, Incorporated (http://www.dtai.com)
 *
 *                        All rights reserved
 *
 * Permission to use, copy, modify and distribute this material for
 * any purpose and without fee is hereby granted, provided that the
 * above copyright notice and this permission notice appear in all
 * copies, and that the name of DTAI, Incorporated not be used in
 * advertising or publicity pertaining to this material without the
 * specific, prior written permission of an authorized representative of
 * DTAI, Incorporated.
 *
 * DTAI, INCORPORATED MAKES NO REPRESENTATIONS AND EXTENDS NO WARRANTIES,
 * EXPRESS OR IMPLIED, WITH RESPECT TO THE SOFTWARE, INCLUDING, BUT
 * NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 * FITNESS FOR ANY PARTICULAR PURPOSE, AND THE WARRANTY AGAINST
 * INFRINGEMENT OF PATENTS OR OTHER INTELLECTUAL PROPERTY RIGHTS.  THE
 * SOFTWARE IS PROVIDED "AS IS", AND IN NO EVENT SHALL DTAI, INCORPORATED OR
 * ANY OF ITS AFFILIATES BE LIABLE FOR ANY DAMAGES, INCLUDING ANY
 * LOST PROFITS OR OTHER INCIDENTAL OR CONSEQUENTIAL DAMAGES RELATING
 * TO THE SOFTWARE.
 */

package COM.dtai.util;

import java.awt.*;
import java.applet.*;
import COM.dtai.gui.*;

/**
 * ShowUser - this shows the user things like error and warning messages, etc.
 *
 * @version	1.0b1
 * @author	DTAI, Incorporated
 *
 * $Id: ShowUser.java,v 1.1 1996/07/29 21:12:49 kadel Exp $
 *
 * $Source: /cvs/java/classes/COM/dtai/util/ShowUser.java,v $
 */

public class ShowUser {

    static private Frame baseframe;
    static private Frame outerframe;
    static private Applet applet;
    static private ErrorDialog error_dialog;
    static private WarningDialog warning_dialog;
    static private InfoDialog info_dialog;
    static private QuestionDialog question_dialog;
    static private AboutDialog about_dialog;
    static private String default_status;
    static private String last_status;
    static private int cursor_type = Frame.DEFAULT_CURSOR;

	/**
	 * DESCRIPTION
	 *
	 * @param PARAMNAME DESCRIPTION
	 * @return DESCRIPTION
	 * @exception CLASSNAME DESCRIPTION
	 * @see CLASSNAME#METHOD
	 */
    static public void setDefaultStatus( String message ) {
        default_status = message;
        status( message );
    }

	/**
	 * DESCRIPTION
	 *
	 * @param PARAMNAME DESCRIPTION
	 * @return DESCRIPTION
	 * @exception CLASSNAME DESCRIPTION
	 * @see CLASSNAME#METHOD
	 */
    static public ErrorDialog getErrorDialog() {
        return error_dialog;
    }

	/**
	 * DESCRIPTION
	 *
	 * @param PARAMNAME DESCRIPTION
	 * @return DESCRIPTION
	 * @exception CLASSNAME DESCRIPTION
	 * @see CLASSNAME#METHOD
	 */
    static public WarningDialog getWarningDialog() {
        return  warning_dialog;
    }

	/**
	 * DESCRIPTION
	 *
	 * @param PARAMNAME DESCRIPTION
	 * @return DESCRIPTION
	 * @exception CLASSNAME DESCRIPTION
	 * @see CLASSNAME#METHOD
	 */
    static public InfoDialog getInfoDialog() {
        return  info_dialog;
    }

	/**
	 * DESCRIPTION
	 *
	 * @param PARAMNAME DESCRIPTION
	 * @return DESCRIPTION
	 * @exception CLASSNAME DESCRIPTION
	 * @see CLASSNAME#METHOD
	 */
    static public QuestionDialog getQuestionDialog() {
        return  question_dialog;
    }

	/**
	 * DESCRIPTION
	 *
	 * @param PARAMNAME DESCRIPTION
	 * @return DESCRIPTION
	 * @exception CLASSNAME DESCRIPTION
	 * @see CLASSNAME#METHOD
	 */
    static public AboutDialog getAboutDialog() {
        return  about_dialog;
    }

	/**
	 * DESCRIPTION
	 *
	 * @param PARAMNAME DESCRIPTION
	 * @return DESCRIPTION
	 * @exception CLASSNAME DESCRIPTION
	 * @see CLASSNAME#METHOD
	 */
    static public synchronized void error( String message ) {

        if ( baseframe == null ) {
            System.err.println( "Error: "+message );
        }
        else {
            if ( error_dialog == null ) {
                error_dialog = new ErrorDialog( outerframe );
            }
            error_dialog.setMessage( message );
            error_dialog.pack();
            error_dialog.show();
            error_dialog.repaint( 100 );        // work around bug
        }
    }

	/**
	 * DESCRIPTION
	 *
	 * @param PARAMNAME DESCRIPTION
	 * @return DESCRIPTION
	 * @exception CLASSNAME DESCRIPTION
	 * @see CLASSNAME#METHOD
	 */
    static public synchronized void warning( String message ) {

        if ( baseframe == null ) {
            System.err.println( "Warning: "+message );
        }
        else {
            if ( warning_dialog == null ) {
                warning_dialog = new WarningDialog( outerframe );
            }
            warning_dialog.setMessage( message );
            warning_dialog.pack();
            warning_dialog.show();
            warning_dialog.repaint( 100 );        // work around bug
        }
    }

	/**
	 * DESCRIPTION
	 *
	 * @param PARAMNAME DESCRIPTION
	 * @return DESCRIPTION
	 * @exception CLASSNAME DESCRIPTION
	 * @see CLASSNAME#METHOD
	 */
    static public synchronized void info( String message ) {

        if ( baseframe == null ) {
            System.err.println( "Info: "+message );
        }
        else {
            if ( info_dialog == null ) {
                info_dialog = new InfoDialog( outerframe );
            }
            info_dialog.setMessage( message );
            info_dialog.pack();
            info_dialog.show();
            info_dialog.repaint( 100 );        // work around bug
        }
    }

	/**
	 * DESCRIPTION
	 *
	 * @param PARAMNAME DESCRIPTION
	 * @return DESCRIPTION
	 * @exception CLASSNAME DESCRIPTION
	 * @see CLASSNAME#METHOD
	 */
    static public synchronized void question( DialogObserver observer,
                                              String message ) {

        if ( baseframe == null ) {
            System.err.println( "Question: "+message );
        }
        else {
            if ( question_dialog == null ) {
                question_dialog = new QuestionDialog( outerframe );
            }
            question_dialog.removeAllObservers();
            question_dialog.addObserver( observer );
            question_dialog.setMessage( message );
            question_dialog.pack();
            question_dialog.show();
            question_dialog.repaint( 100 );        // work around bug
        }
    }

	/**
	 * DESCRIPTION
	 *
	 * @param PARAMNAME DESCRIPTION
	 * @return DESCRIPTION
	 * @exception CLASSNAME DESCRIPTION
	 * @see CLASSNAME#METHOD
	 */
    static public synchronized void about( String what, String message ) {

        if ( baseframe == null ) {
            System.err.println( "About "+what+": "+message );
        }
        else {
            if ( about_dialog == null ) {
                about_dialog = new AboutDialog( outerframe, what );
            }
            else {
                about_dialog.setTitle( what );
            }
            about_dialog.setMessage( message );
            about_dialog.pack();
            about_dialog.show();
            about_dialog.repaint( 100 );        // work around bug
        }
    }

	/**
	 * DESCRIPTION
	 *
	 * @param PARAMNAME DESCRIPTION
	 * @return DESCRIPTION
	 * @exception CLASSNAME DESCRIPTION
	 * @see CLASSNAME#METHOD
	 */
    static public synchronized void status( String message ) {
        if ( message.equals( "" ) ) {
            message = default_status;
        }
        if ( message == null ) {
            message = "";
        }
        if ( last_status == null ) {
            last_status = "";
        }
        if ( ! message.equals( last_status ) ) {
            last_status = message;
            if ( applet != null ) {
                applet.showStatus( message );
            }
            else {
                System.out.println( message );
            }
        }
    }

	/**
	 * DESCRIPTION
	 *
	 * @param PARAMNAME DESCRIPTION
	 * @return DESCRIPTION
	 * @exception CLASSNAME DESCRIPTION
	 * @see CLASSNAME#METHOD
	 */
    static private Frame findBaseFrame( Component component ) {
        Container parent = component.getParent();
        while ( ! ( parent instanceof Frame ) ) {
            parent = parent.getParent();
        }
        return ((Frame)parent);
    }

	/**
	 * DESCRIPTION
	 *
	 * @param PARAMNAME DESCRIPTION
	 * @return DESCRIPTION
	 * @exception CLASSNAME DESCRIPTION
	 * @see CLASSNAME#METHOD
	 */
    static private Frame findOuterFrame( Component component ) {
        Container last_parent = null;
        Container parent = component.getParent();
        while ( parent != null ) {
            last_parent = parent;
            parent = parent.getParent();
        }
        return ((Frame)last_parent);
    }

	/**
	 * DESCRIPTION
	 *
	 * @param PARAMNAME DESCRIPTION
	 * @return DESCRIPTION
	 * @exception CLASSNAME DESCRIPTION
	 * @see CLASSNAME#METHOD
	 */
    static public void setApplet( Applet applet ) {
        ShowUser.applet = applet;
        baseframe = findBaseFrame( applet );
        outerframe = findOuterFrame( applet );

        if ( cursor_type != Frame.DEFAULT_CURSOR ) {
            baseframe.setCursor( cursor_type );
        }
    }

	/**
	 * DESCRIPTION
	 *
	 * @param PARAMNAME DESCRIPTION
	 * @return DESCRIPTION
	 * @exception CLASSNAME DESCRIPTION
	 * @see CLASSNAME#METHOD
	 */
    static public final Applet getDefaultApplet() {
        return applet;
    }

	/**
	 * DESCRIPTION
	 *
	 * @param PARAMNAME DESCRIPTION
	 * @return DESCRIPTION
	 * @exception CLASSNAME DESCRIPTION
	 * @see CLASSNAME#METHOD
	 */
    static public final Frame getBaseFrame() {
        return baseframe;
    }

	/**
	 * DESCRIPTION
	 *
	 * @param PARAMNAME DESCRIPTION
	 * @return DESCRIPTION
	 * @exception CLASSNAME DESCRIPTION
	 * @see CLASSNAME#METHOD
	 */
    static public void cursor( int cursor_type ) {
        if ( cursor_type != ShowUser.cursor_type ) {
            ShowUser.cursor_type = cursor_type;
            if ( baseframe != null ) {
                baseframe.setCursor( cursor_type );
            }
        }
    }
}
