/**
 * Copyright(c) 1996 DTAI, Incorporated (http://www.dtai.com)
 *
 *                        All rights reserved
 *
 * Permission to use, copy, modify and distribute this material for
 * any purpose and without fee is hereby granted, provided that the
 * above copyright notice and this permission notice appear in all
 * copies, and that the name of DTAI, Incorporated not be used in
 * advertising or publicity pertaining to this material without the
 * specific, prior written permission of an authorized representative of
 * DTAI, Incorporated.
 *
 * DTAI, INCORPORATED MAKES NO REPRESENTATIONS AND EXTENDS NO WARRANTIES,
 * EXPRESS OR IMPLIED, WITH RESPECT TO THE SOFTWARE, INCLUDING, BUT
 * NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 * FITNESS FOR ANY PARTICULAR PURPOSE, AND THE WARRANTY AGAINST
 * INFRINGEMENT OF PATENTS OR OTHER INTELLECTUAL PROPERTY RIGHTS.  THE
 * SOFTWARE IS PROVIDED "AS IS", AND IN NO EVENT SHALL DTAI, INCORPORATED OR
 * ANY OF ITS AFFILIATES BE LIABLE FOR ANY DAMAGES, INCLUDING ANY
 * LOST PROFITS OR OTHER INCIDENTAL OR CONSEQUENTIAL DAMAGES RELATING
 * TO THE SOFTWARE.
 */

import java.awt.*;
import java.net.*;
import java.util.*;
import java.io.*;
import COM.dtai.gui.*;
import COM.dtai.data.*;
import COM.dtai.util.*;

/**
 * NodeDialog - a dialog for viewing and changing OrgChart node and
 * employee information
 *
 * @version	1.0b1
 * @author	DTAI, Incorporated
 *
 * $Id: NodeDialog.java,v 1.16 1996/08/13 06:29:15 kadel Exp $
 *
 * $Source: /cvs/java/OrgChart/NodeDialog.java,v $
 */

public class NodeDialog extends StdDialog implements Observer, DialogObserver {

    private OrgNode node;
    private OrgNode boss;
    private OrgChartGraph graph;
    private OrgChartPrefs prefs;

    private QuestionDialog question_dialog;
    private OrgNode new_node;

    private OrgChart chart;
    private OrgChartDb db;
    boolean editable;

    private PromptBlock prompts;
    private TextPrompt email_prompt;

    private Checkbox subordinate_cb;
    private Checkbox assistant_cb;
    private CheckboxGroup radio;

    private Button ok_b;
    private Button apply_b;
    private Button cancel_b;
    private Button help_b;

    private DataString first_name;
    private DataString last_name;
    private DataString number;
    private DataString phone;
    private DataString role;
    private DataString section_title;
    private DataURLString section_url;
    private DataURLString email;
    private DataURLString url;

    private TabGroup tab_group = new TabGroup();

    private GridBagConstraints subconstraints;

    public NodeDialog ( Frame frame, OrgChart chart ) {
        super ( frame, "Node Dialog", false );
        this.chart = chart;
        db = chart.getDatabase();
        prefs = chart.getPreferences();

        editable = chart.isEditable();

        Panel mainpanel = getMainPanel();
        mainpanel.setLayout( new BorderLayout() );

        prompts = new PromptBlock ();
        mainpanel.add ( "North", prompts );

        mainpanel.add ( "South",
                        new Label( "Press \"E-mail\" to send a message, or a \"Web Page\" label to view the page.",
                                   Label.CENTER ) );

        TextPrompt tp;
        Component temp;

        prompts.addPrompt(
            tp = new TextPrompt(
                new NamedData( "First Name:",
                    first_name = new DataString( 15 ) ) ) );
        temp = tp.getRightComponent();
	    tab_group.add( temp );
	    if ( ! editable ) {
	        if ( temp instanceof TextComponent ) {
	            ((TextComponent)temp).setEditable( false );
	        }
	        else {
	            temp.disable();
	        }
	    }

        prompts.addPrompt(
            tp = new TextPrompt(
                new NamedData( "Last Name:",
                    last_name = new DataString( 15 ) ) ) );
        temp = tp.getRightComponent();
	    tab_group.add( temp );
	    if ( ! editable ) {
	        if ( temp instanceof TextComponent ) {
	            ((TextComponent)temp).setEditable( false );
	        }
	        else {
	            temp.disable();
	        }
	    }

        prompts.addPrompt(
            tp = new TextPrompt(
                new NamedData( "Job Title:",
                    role = new DataString( 25 ) ) ) );
        temp = tp.getRightComponent();
	    tab_group.add( temp );
	    if ( ! editable ) {
	        if ( temp instanceof TextComponent ) {
	            ((TextComponent)temp).setEditable( false );
	        }
	        else {
	            temp.disable();
	        }
	    }

        prompts.addPrompt(
            tp = new TextPrompt(
                new NamedData( "Employee ID:",
                    number = new DataString( 8 ) ) ) );
        temp = tp.getRightComponent();
	    tab_group.add( temp );
	    if ( ! editable ) {
	        if ( temp instanceof TextComponent ) {
	            ((TextComponent)temp).setEditable( false );
	        }
	        else {
	            temp.disable();
	        }
	    }

        prompts.addPrompt(
            tp = new TextPrompt(
                new NamedData( "Phone/Ext.:",
                    phone = new DataString( 20 ) ) ) );
        temp = tp.getRightComponent();
	    tab_group.add( temp );
	    if ( ! editable ) {
	        if ( temp instanceof TextComponent ) {
	            ((TextComponent)temp).setEditable( false );
	        }
	        else {
	            temp.disable();
	        }
	    }

        tp = new TextPrompt(
            new NamedData( "E-mail:",
                email = new DataURLString( "mailto", 25 ) ) );
        email_prompt = tp;
	    tp.notifyOnButtonClick( this );
        prompts.addPrompt( tp );
        temp = tp.getRightComponent();
	    tab_group.add( temp );
	    if ( ! editable ) {
	        if ( temp instanceof TextComponent ) {
	            ((TextComponent)temp).setEditable( false );
	        }
	        else {
	            temp.disable();
	        }
	    }

        tp = new TextPrompt(
            new NamedData( "Personal Web Page:",
                url = new DataURLString( "http", 40 ) ) );
	    tp.notifyOnButtonClick( this );
        prompts.addPrompt( tp );
        temp = tp.getRightComponent();
	    tab_group.add( temp );
	    if ( ! editable ) {
	        if ( temp instanceof TextComponent ) {
	            ((TextComponent)temp).setEditable( false );
	        }
	        else {
	            temp.disable();
	        }
	    }

        tp = new TextPrompt(
            new NamedData( "Section Web Page:",
                section_url = new DataURLString( "http", 40 ) ) );
	    tp.notifyOnButtonClick( this );
        prompts.addPrompt( tp );
        temp = tp.getRightComponent();
	    tab_group.add( temp );
	    if ( ! editable ) {
	        if ( temp instanceof TextComponent ) {
	            ((TextComponent)temp).setEditable( false );
	        }
	        else {
	            temp.disable();
	        }
	    }

        prompts.addPrompt(
            tp = new TextPrompt(
                new NamedData( "Section Title:",
                    section_title = new DataString( 40 ) ) ) );
        temp = tp.getRightComponent();
	    tab_group.add( temp );
	    if ( ! editable ) {
	        if ( temp instanceof TextComponent ) {
	            ((TextComponent)temp).setEditable( false );
	        }
	        else {
	            temp.disable();
	        }
	    }

        Panel toggles = new Panel ();
        mainpanel.add ( "Center", toggles );

        toggles.setLayout ( new FlowLayout ( FlowLayout.CENTER, 10, 10 ) );
        radio = new CheckboxGroup();

        toggles.add ( subordinate_cb =
			new Checkbox( "Subordinate", radio, false ) );
// NEED SPECIAL CASES FOR RADIOS, because focus SELECTS the item in Windows
// Need to implement arrow keys to go between radios
//      tab_group.add( subordinate_cb );
        toggles.add ( assistant_cb =
			new Checkbox( "Assistant", radio, true ) );
//	    tab_group.add( assistant_cb );

        if ( ! editable ) {
            subordinate_cb.disable();
            assistant_cb.disable();
        }

        addButton ( ok_b = new Button ( "OK" ) );
	    tab_group.add( ok_b );

	    if ( editable ) {
            addButton ( apply_b = new Button ( "Apply" ) );
    	    tab_group.add( apply_b );
            addButton ( cancel_b = new Button ( "Cancel" ) );
    	    tab_group.add( cancel_b );
	    }

        addButton ( help_b = new Button ( "Help" ) );
	    tab_group.add( help_b );

        pack ();
    }

	/**
	 * DESCRIPTION
	 *
	 * @param PARAMNAME DESCRIPTION
	 * @return DESCRIPTION
	 * @exception CLASSNAME DESCRIPTION
	 * @see CLASSNAME#METHOD
	 */
    public void update( Observable o, Object arg ) {
        // a prompt button was clicked...
        TextPrompt prompt = (TextPrompt)arg;
        if ( prompt.acceptChanges() ) {
            if ( prompt.getData() instanceof DataURLString ) {
                DataURLString data = (DataURLString)prompt.getData();
				URL url = data.getURLValue();
				if ( url != null ) {
					if ( arg == email_prompt ) {
						chart.getAppletContext().showDocument( url, "_self" );
					}
					else {
						chart.getAppletContext().showDocument( url, "_blank" );
					}
				}
            }
        }
    }

	/**
	 * DESCRIPTION
	 *
	 * @param PARAMNAME DESCRIPTION
	 * @return DESCRIPTION
	 * @exception CLASSNAME DESCRIPTION
	 * @see CLASSNAME#METHOD
	 */
    public void clear() {
        first_name.clear();
        last_name.clear();
        number.clear();
        phone.clear();
        role.clear();
        section_title.clear();
        section_url.clear();
        email.clear();
        url.clear();
        subordinate_cb.setState( true );
        tab_group.firstFocus();
    }

	/**
	 * DESCRIPTION
	 *
	 * @param PARAMNAME DESCRIPTION
	 * @return DESCRIPTION
	 * @exception CLASSNAME DESCRIPTION
	 * @see CLASSNAME#METHOD
	 */
    public void refreshDialog( OrgNode node ) {
        this.node = node;
        boss = null;
        graph = null;
        clear();
        Employee employee = node.getEmployee();
        if ( employee != null ) {
            first_name.setFromString( employee.getFirstName() );
            last_name.setFromString( employee.getLastName() );
            number.setFromString( employee.getNumber() );
            phone.setFromString( employee.getPhone() );
            try {
                email.setFromString( employee.getEmail() );
            }
            catch ( MalformedURLException mue ) {
            }
            if ( employee.getWebPage() != null ) {
                url.setURLValue( employee.getWebPage() );
            }
        }
        role.setFromString( node.getRole() );
        section_title.setFromString( node.getSectionTitle() );

        if ( node.getSectionURL() != null ) {
            section_url.setURLValue( node.getSectionURL() );
        }

        if ( node.getType() == OrgNode.SUBORDINATE ) {
            subordinate_cb.setState( true );
        }
        else {
            assistant_cb.setState( true );
        }
        show();
    }

	/**
	 * DESCRIPTION
	 *
	 * @param PARAMNAME DESCRIPTION
	 * @return DESCRIPTION
	 * @exception CLASSNAME DESCRIPTION
	 * @see CLASSNAME#METHOD
	 */
    public void editNode( OrgNode node ) {
        if ( isVisible() &&
             ( node != this.node ) ) {
            changeNode( node );
        }
        else {
            refreshDialog( node );
        }
    }

	/**
	 * DESCRIPTION
	 *
	 * @param PARAMNAME DESCRIPTION
	 * @return DESCRIPTION
	 * @exception CLASSNAME DESCRIPTION
	 * @see CLASSNAME#METHOD
	 */
    public void addNode( OrgNode boss ) {
        if ( ( ! isVisible() ) ||
             ( ! isModified() ) ) {
            node = null;
            this.boss = boss;
            graph = null;
            clear();
            show();
        }
        else {
            ShowUser.warning( "You've made some changes.  Please \"Apply\" or "+
                              "\"Cancel\" them before adding a new node." );
        }
    }

	/**
	 * DESCRIPTION
	 *
	 * @param PARAMNAME DESCRIPTION
	 * @return DESCRIPTION
	 * @exception CLASSNAME DESCRIPTION
	 * @see CLASSNAME#METHOD
	 */
    public void addTopNode( OrgChartGraph graph ) {
        node = null;
        boss = null;
        this.graph = graph;
        clear();
        show();
    }

	/**
	 * DESCRIPTION
	 *
	 * @param PARAMNAME DESCRIPTION
	 * @return DESCRIPTION
	 * @exception CLASSNAME DESCRIPTION
	 * @see CLASSNAME#METHOD
	 */
    public boolean isModified() {
        if ( ( ( node == null ) &&
               ( radio.getCurrent() != subordinate_cb ) ) ||
             ( ( node != null ) &&
               ( ( ( radio.getCurrent() == assistant_cb ) &&
                   ( node.getType() != OrgNode.ASSISTANT ) ) ||
                 ( ( radio.getCurrent() == subordinate_cb ) &&
                   ( node.getType() != OrgNode.SUBORDINATE ) ) ) ) ||
             prompts.isModified() ) {
            return true;
        }
        else {
            return false;
        }
    }

	/**
	 * DESCRIPTION
	 *
	 * @param PARAMNAME DESCRIPTION
	 * @return DESCRIPTION
	 * @exception CLASSNAME DESCRIPTION
	 * @see CLASSNAME#METHOD
	 */
    public void removeNode( OrgNode node ) {
        if ( node == this.node ) {
            hide();
        }
    }

	/**
	 * DESCRIPTION
	 *
	 * @param PARAMNAME DESCRIPTION
	 * @return DESCRIPTION
	 * @exception CLASSNAME DESCRIPTION
	 * @see CLASSNAME#METHOD
	 */
    public void changeNode( OrgNode new_node ) {
        if ( isVisible() &&
             ( new_node != node ) ) {
            if ( ( ! editable ) ||
                 ( ! isModified() ) ) {
                editNode( node = new_node );
            }
            else {
                this.new_node = new_node;
                ShowUser.question( this,
                                   "You've made some changes.  Are you sure you "+
                                   "want to abandon the changes and "+
                                   "edit the selected node?" );
                question_dialog = ShowUser.getQuestionDialog();
            }
        }
    }

	/**
	 * DESCRIPTION
	 *
	 * @param PARAMNAME DESCRIPTION
	 * @return DESCRIPTION
	 * @exception CLASSNAME DESCRIPTION
	 * @see CLASSNAME#METHOD
	 */
    public void dialogAction( StdDialog dialog ) {
        if ( dialog == question_dialog ) {
            if ( question_dialog.okSelected() ) {
                editNode( node = new_node );
            }
        }
    }

	/**
	 * DESCRIPTION
	 *
	 * @param PARAMNAME DESCRIPTION
	 * @return DESCRIPTION
	 * @exception CLASSNAME DESCRIPTION
	 * @see CLASSNAME#METHOD
	 */
    public boolean isValid() {
        if ( ( ! isVisible() ) ||
             ( ! editable ) ) {
            return true;
        }
        if ( first_name.acceptChange() &&
             last_name.acceptChange() &&
             number.acceptChange() &&
             phone.acceptChange() &&
             role.acceptChange() &&
             section_title.acceptChange() &&
             section_url.acceptChange() &&
             email.acceptChange() &&
             url.acceptChange() ) {
            return true;
        }
        else {
            return false;
        }
    }

	/**
	 * DESCRIPTION
	 *
	 * @param PARAMNAME DESCRIPTION
	 * @return DESCRIPTION
	 * @exception CLASSNAME DESCRIPTION
	 * @see CLASSNAME#METHOD
	 */
    public void applyChanges( OrgNode node ) {

        Employee employee = node.getEmployee();
        if ( employee != null ) {
            employee.setFirstName( first_name.toString().trim() );
            employee.setLastName( last_name.toString().trim() );
            employee.setNumber( number.toString().trim() );
            employee.setPhone( phone.toString().trim() );
            employee.setEmail( email.toString() );
        }
        node.setRole( role.toString().trim() );
        node.setSectionTitle( section_title.toString().trim() );
        node.setSectionURL( section_url.getURLValue() );
        employee.setWebPage( url.getURLValue() );

        if ( radio.getCurrent() == assistant_cb ) {
            node.setType( OrgNode.ASSISTANT );
        }
        else {
            node.setType( OrgNode.SUBORDINATE );
        }
    }

	/**
	 * DESCRIPTION
	 *
	 * @param PARAMNAME DESCRIPTION
	 * @return DESCRIPTION
	 * @exception CLASSNAME DESCRIPTION
	 * @see CLASSNAME#METHOD
	 */
    public boolean apply() {
        if ( ( ! isVisible() ) ||
             ( ! editable ) ) {
            return true;
        }
        if ( ! isValid() ) {
            return false;
        }
        else {
            if ( node != null ) {
                // change it and send to db
                applyChanges( node );
                try {
                    node.save();
                    node.getEmployee().save();
                }
                catch ( IllegalAccessException iae ) {
                    chart.lostWritability();
                }
                catch ( IOException exp ) {
                    ShowUser.error( "Database error trying to save the information." );
                }
                node.refresh();
            }
            else {
                if ( graph == null ) {
                    graph = boss.getGraph();
                }
                Employee employee = new Employee( db );
                OrgNode new_node = new OrgNode( chart, employee );
                if ( radio.getCurrent() == assistant_cb ) {
                    new_node.setType( OrgNode.ASSISTANT );
                }
                applyChanges( new_node );
                if ( boss == null ) {
                    graph.setTopNode( new_node );
                    new_node.select();
                }
                else {
                    boss.addNode( new_node );

                    if ( boss.getCurrentLevel() == prefs.max_levels ) {
                        OrgNode new_top = boss;
                        for ( int idx = prefs.max_levels; idx > 2; idx-- ) {
                            new_top = new_top.getSupervisor();
                        }
                        new_top.becomeGraphTop();
                    }
                }

                try {
                    new_node.save();
                    new_node.getEmployee().save();
                    db.newNode( new_node );
                    db.newEmployee( new_node.getEmployee() );
                }
                catch ( IllegalAccessException iae ) {
                    chart.lostWritability();
                }
                catch ( IOException exp ) {
                    ShowUser.error( "Database error trying to save the new location." );
                }
                new_node.refresh();

                refreshDialog( new_node ); // in case dialog stays up (Apply, not OK)
            }
            return true;
        }
    }

	/**
	 * DESCRIPTION
	 *
	 * @param PARAMNAME DESCRIPTION
	 * @return DESCRIPTION
	 * @exception CLASSNAME DESCRIPTION
	 * @see CLASSNAME#METHOD
	 */
    public boolean action ( Event evt, Object arg ) {

        boolean retval = true;

        if ( evt.target instanceof Checkbox ) {
            retval = false;
        }
        else if ( evt.target instanceof Button ) {

            retval = false;
            boolean valid = true;

            if ( ( evt.target == ok_b ) || ( evt.target == apply_b ) ) {

	            if ( editable ) {
	                valid = apply();
	            }
	            retval = true;
            }

            if ( ( evt.target == ok_b ) || ( evt.target == cancel_b ) ) {
                if ( valid ) {
	                hide ();
	            }
	            retval = true;
            }

            if ( evt.target == help_b ) {
                try {
                    URL url = new URL( "javascript:NetHelp('view')" );
                    chart.getAppletContext().showDocument( url );
                }
                catch ( MalformedURLException mue ) {
                    ShowUser.warning( "JavaScript error. Could not open the help window." );
                }
            }
        }
        else {
            retval = false;
        }

        return retval;
    }

	/**
	 * DESCRIPTION
	 *
	 * @param PARAMNAME DESCRIPTION
	 * @return DESCRIPTION
	 * @exception CLASSNAME DESCRIPTION
	 * @see CLASSNAME#METHOD
	 */
    public boolean keyDown( Event evt, int key ) {
        if ( key == '\t' ) {
            if ( evt.shiftDown() ) {
                tab_group.prevFocus( (Component)evt.target );
            }
            else {
                tab_group.nextFocus( (Component)evt.target );
            }
            return true;
        }

        if ( key == '\n' ) {
            if ( editable ) {
                if ( tab_group.getFocus() instanceof Button ) {
                    evt.target = tab_group.getFocus();
                    return action( evt, null );
                }
                else {
                    if ( apply() ) {
                        hide();
                    }
                }
            }
            return true;
        }
        return false;
    }
}
