/**
 * Copyright(c) 1996 DTAI, Incorporated (http://www.dtai.com)
 *
 *                        All rights reserved
 *
 * Permission to use, copy, modify and distribute this material for
 * any purpose and without fee is hereby granted, provided that the
 * above copyright notice and this permission notice appear in all
 * copies, and that the name of DTAI, Incorporated not be used in
 * advertising or publicity pertaining to this material without the
 * specific, prior written permission of an authorized representative of
 * DTAI, Incorporated.
 *
 * DTAI, INCORPORATED MAKES NO REPRESENTATIONS AND EXTENDS NO WARRANTIES,
 * EXPRESS OR IMPLIED, WITH RESPECT TO THE SOFTWARE, INCLUDING, BUT
 * NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 * FITNESS FOR ANY PARTICULAR PURPOSE, AND THE WARRANTY AGAINST
 * INFRINGEMENT OF PATENTS OR OTHER INTELLECTUAL PROPERTY RIGHTS.  THE
 * SOFTWARE IS PROVIDED "AS IS", AND IN NO EVENT SHALL DTAI, INCORPORATED OR
 * ANY OF ITS AFFILIATES BE LIABLE FOR ANY DAMAGES, INCLUDING ANY
 * LOST PROFITS OR OTHER INCIDENTAL OR CONSEQUENTIAL DAMAGES RELATING
 * TO THE SOFTWARE.
 */

import java.awt.*;
import java.applet.*;
import java.io.*;
import java.net.*;
import java.util.*;
import COM.dtai.gui.*;
import COM.dtai.util.*;

/**
 * OrgChart - The Appalanche Org Chart Applet
 *
 * @version	1.0b1
 * @author	DTAI, Incorporated
 *
 * $Id: OrgChart.java,v 1.24 1996/08/15 08:16:53 kadel Exp $
 *
 * $Source: /cvs/java/OrgChart/OrgChart.java,v $
 */

public class OrgChart extends Applet implements DialogObserver {

    private OrgChartGraph graph;
    private OrgChartDb db;
    private OrgChartPrefs prefs;

    private CommandButtons edit_buttons;
    private CommandButtons graph_buttons;
    private MyButton edit;
    private MyButton add;
    private MyButton remove;
    private MyButton shrink;
    private MyButton grow;
    private MyButton more;
    private MyButton less;
    private MyButton revert;
    private MyButton save;
    private MyButton help;

    private String host;
    private int port;
    private String editflag;
    private String dbname;
    private boolean editable = false;

    private TabGroup tab_group;

    private NodeDialog node_dialog;
    private OverrideDialog override_dialog;
    private QuestionDialog question_dialog;
    private RemoveDialog remove_dialog;
    private OrgNode remove_node;
    private int save_top_node_parentid = -1;

	/**
	 * DESCRIPTION
	 *
	 * @param PARAMNAME DESCRIPTION
	 * @return DESCRIPTION
	 * @exception CLASSNAME DESCRIPTION
	 * @see CLASSNAME#METHOD
	 */
    public static void main ( String args[] ) {
        new DummyAppletContext ( new OrgChart(), args, 500, 500 );
    }

	/**
	 * DESCRIPTION
	 *
	 * @param PARAMNAME DESCRIPTION
	 * @return DESCRIPTION
	 * @exception CLASSNAME DESCRIPTION
	 * @see CLASSNAME#METHOD
	 */
    public String getAppletInfo() {
        return "OrgChart Applet.\nFor Netscape's Project Appalanche.\nDeveloped by DTAI, Incorporated\nhttp://www.dtai.com\nCopyright 1996, DTAI, Incorporated";
    }

	/**
	 * DESCRIPTION
	 *
	 * @param PARAMNAME DESCRIPTION
	 * @return DESCRIPTION
	 * @exception CLASSNAME DESCRIPTION
	 * @see CLASSNAME#METHOD
	 */
    public String[][] getParameterInfo() {
        String pinfo[][] = {
            { "Host", "String",
              "Database host machine. Default: host of this applet" },
            { "Port", "Integer",
              "Database server TCP/IP port number.  Default: 9006" },
            { "OrgChartName", "String",
              "The name for the Org Chart, used to query the database for a particular hierarchy. "+
              "(You are encouraged to limit yourself to letters and numbers.) Default: \"default\"" },
            { "Edit", "Boolean",
              "If \"Yes\", allow the database to be modified.  Default: No" },
            { "StartScriptURL", "String",
              "Absolute URL path to OrgChart server CGI startup script.  "+
                "Default: /OrgChart_DTAI/user/start_server.cgi, or if not "+
				"there, then /OrgChart_DTAI/user/start_server.bat" },
            { "InitialSection", "String",
              "If set, and the section exists, this section will be displayed initially. "+
              "Default: Display the top-most node or section." },
            { "ChartBackgroundColor", "String",
              "Name of color for the chart background.  Default: White" },
            { "ChartBackgroundImage", "URL",
              "Absolute or relative (to document base) URL for an image to tile in the chart background.  Default: None" },
            { "BackgroundColor", "String",
              "Name of color for the background of most of the rest of the applet.  Default: Black" },
            { "ForegroundColor", "String",
              "Name of color for the foreground of most of the rest of the applet.  Default: LightGray" },
            { "VertSpacing", "Integer",
              "Default vertical spacing in pixels between nodes, and below title.  Default: 20" },
            { "HorizSpacing", "Integer",
              "Default horizontal spacing in pixels between nodes.  Default: 10" },
            { "MaxLevels", "Integer",
              "Maximum number of levels to show from top node to leaf nodes.  Default: 2" },
            { "NodeFontFace", "String",
              "Name of font to display text in the nodes.  Default: Helvetica" },
            { "NodeFontSize", "Integer",
              "Point size of font to display text in the nodes.  Default: 18" },
            { "NodeFontStyle", "String",
              "Style (Bold, Italic, or BoldItalic) of font to display text in the nodes.  Default: PLAIN" },
            { "TitleFontFace", "String",
              "Name of font for the chart title.  Default: TimesRoman" },
            { "TitleFontSize", "Integer",
              "Point size of font for the chart title.  Default: 36" },
            { "TitleFontStyle", "String",
              "Style (Bold, Italic, or BoldItalic) of font for the chart title.  Default: BOLD" },
            { "NodeTextColor", "String",
              "Name of color of text in the nodes.  Default: Black" },
            { "TitleTextColor", "String",
              "Name of color for chart title.  Default: Black" },
            { "LinkColor", "String",
              "Name of color for links between nodes.  Default: Black" },
            { "NodeBorderColor", "String",
              "Name of color for the node border.  Default: Black" },
            { "NodeBackgroundColor", "String",
              "Name of color for the node backgrounds.  Default: White" },
            { "NodeArmColor", "String",
              "Name of color for the node backgrounds when mouse button is held down.  Default: Gray" },
            { "NodeHiliteColor", "String",
              "Name of color for the node backgrounds when hilited.  Default: Yellow" },
            { "NodeTextHiliteColor", "String",
              "Name of color for the node text when hilited.  Default: Black" },
            { "NodeShadowColor", "String",
              "Name of color for the node shadow.  Default: Gray" },
            { "NodeShadowOffset", "Integer",
              "Pixels down and to the right of node to place shadow.  Default: 5" },
            { "NodeBorderWidth", "Integer",
              "Pixel width of the node border.  Default: 1" },
            { "LinkThickness", "Integer",
              "Pixel width of the links between nodes.  Default: 1" },
            { "RoundNodeCorners", "Boolean",
              "If \"Yes\", round the corners on the node rectangles.  Default: Yes" },
            { "ThreeDBorders", "Boolean",
              "If \"Yes\", draw the node borders with a 3-D effect (implies NO ROUND CORNERS).  Default: No" },
            { "SuppressNodeBackground", "Boolean",
              "If \"Yes\", draw the node with no border or background.  Default: No" },
            { "TransparentNodeBackground", "Boolean",
              "If \"Yes\", draw the node with a border but no background (implies NO SHADOW).  Default: No" }
        };
        return pinfo;
    }

	/**
	 * DESCRIPTION
	 *
	 * @param PARAMNAME DESCRIPTION
	 * @return DESCRIPTION
	 * @exception CLASSNAME DESCRIPTION
	 * @see CLASSNAME#METHOD
	 */
    public void init () {

        ShowUser.setApplet( this );

        ShowUser.cursor( Frame.WAIT_CURSOR );

        prefs = new OrgChartPrefs( this );

        getParent().setBackground( prefs.background_color );
        getParent().setForeground( prefs.foreground_color );

        makeGraph();

        makeButtons();

        getDbParams();
    }

	/**
	 * DESCRIPTION
	 *
	 * @param PARAMNAME DESCRIPTION
	 * @return DESCRIPTION
	 * @exception CLASSNAME DESCRIPTION
	 * @see CLASSNAME#METHOD
	 */
    public void repaint() {
        ShowUser.cursor( Frame.WAIT_CURSOR );
        super.repaint();
    }

	/**
	 * DESCRIPTION
	 *
	 * @param PARAMNAME DESCRIPTION
	 * @return DESCRIPTION
	 * @exception CLASSNAME DESCRIPTION
	 * @see CLASSNAME#METHOD
	 */
    public void stop() {
        saveTopNodeParent();
        graph.clear( false );
        try {
            db.close();
        }
        catch ( IllegalAccessException iae ) {
            lostWritability();
        }
        catch ( IOException ioe ) {
            ShowUser.error( "Database I/O error trying to save and close the Org Chart." );
        }
        db = null;

        if ( editable ) {
            editflag = "yes";
        }
        else {
            editflag = "no";
        }
    }

	/**
	 * DESCRIPTION
	 *
	 * @param PARAMNAME DESCRIPTION
	 * @return DESCRIPTION
	 * @exception CLASSNAME DESCRIPTION
	 * @see CLASSNAME#METHOD
	 */
    public void start() {

        ShowUser.cursor( Frame.WAIT_CURSOR );

        openDatabase();

        updateButtons();

        updateSelected( null );

        if ( editable ) {
            ShowUser.setDefaultStatus( "Ctrl-drag to move nodes.  Shift to constrain." );
        }

        ShowUser.cursor( Frame.DEFAULT_CURSOR );
    }

    private void makeGraph() {
        setLayout( new BorderLayout() );

        ScrolledGadgetManager scrolled = new ScrolledGadgetManager();
        scrolled.centerHorizontally( true );
        scrolled.centerVertically( true );

        super.add( "Center", scrolled );

        scrolled.addChild( graph = new OrgChartGraph( this ) );
    }

	/**
	 * DESCRIPTION
	 *
	 * @param PARAMNAME DESCRIPTION
	 * @return DESCRIPTION
	 * @exception CLASSNAME DESCRIPTION
	 * @see CLASSNAME#METHOD
	 */
    public void loadInitialNodes() {

        graph.freeze();

        try {
            db.clear();

            int parentid = 0;

            if ( save_top_node_parentid > -1 ) {
                parentid = save_top_node_parentid;
            }
            else {
                String section = getParameter( "InitialSection" );
                if ( ( section != null ) &&
                     ( ! section.equals( "" ) ) ) {
                    db.getOrgChartBranchBySection( section, this, prefs.max_levels + 1 );
                }
            }

            if ( graph.getTopNode() == null ) {
				db.getOrgChartBranchByParent( 0, this, prefs.max_levels + 1 );
            }
        }
        catch ( Exception exp ) {
            ShowUser.error( "Database Error trying to read the Org Chart." );
        }
        finally {
            graph.repaint();

            graph.thaw();
        }
    }

    private void getDbParams() {

        host = getParameter( "Host" );
        if ( host == null ) {
            host = "";
        }
        if ( host.equals( "" ) ) {
            host = getCodeBase().getHost();
        }
        if ( host.equals( "" ) ) {
            host = "localhost";
        }

        try {
            port = Integer.parseInt( getParameter( "Port" ) );
        }
        catch ( NumberFormatException nfe ) {
            port = 9006; // default
        }

        dbname = getParameter( "OrgChartName" );
        if ( ( dbname == null ) ||
             dbname.equals( "" ) ) {
            dbname = "default";
        }

        editflag = getParameter( "Edit" );
        if ( editflag == null ) {
            editflag = "";
        }
    }

    private synchronized void openDatabase() {

        db = null;
        try {
            db = new OrgChartDb( host, port );
        }
        catch ( IOException ioe ) {
			ShowUser.info( "Starting up the database.  Please wait..." );

            try {
                URL url = getCodeBase();
                URL prefix = new URL( url.getProtocol(), url.getHost(),
                                      url.getPort(), "" );

                String start_script_url = getParameter( "StartScriptURL" );
                if ( ( start_script_url == null ) ||
                     start_script_url.equals( "" ) ) {
                    start_script_url = "/OrgChart_DTAI/user/start_server.cgi";
                    url = new URL( prefix, start_script_url );
                    // test for existing URL file
                    try {
                        InputStream is = url.openStream();
                        byte b[] = new byte[64];
                        is.read( b );
                        String begin = new String( b, 0 );
                        if ( begin.indexOf( "Not Found" ) > -1 ) {
							start_script_url = "/OrgChart_DTAI/user/start_server.bat";
							url = new URL( prefix, start_script_url );
                        }
						else {
							url = null; // done executing CGI
						}
                    }
                    catch ( IOException ioe2 ) {
                        if ( ioe2.getMessage() != null && ioe2.getMessage().equals( "stream closed" ) ) {
                            url = null; // probably finished reading the input already
                        }
                        else {
    						start_script_url = "/OrgChart_DTAI/user/start_server.bat";
    						url = new URL( prefix, start_script_url );
    					}
                    }
                }
                else {
                    url = new URL( prefix, start_script_url );
                }

                int max_tries = 5;

				if ( url != null ) {
                    try {
                        InputStream is = url.openStream();
                        is.read(); // read one byte to get the CGI started
                    }
                    catch ( IOException ioe3 ) {
                        if ( ioe.getMessage() != null && ioe3.getMessage().equals( "stream closed" ) ) {
                            // probably finished reading the input already
                        }
                        else {
                            ShowUser.error( "IO Error executing start script." );
                            max_tries = 0;
                        }
                    }
				}

                for ( int idx = 0; idx < max_tries; idx++ ) {
                    try {
                        wait( 5000 );
                        db = new OrgChartDb( host, port );
                        break;
                    }
                    catch ( InterruptedException ie ) {
                    }
                    catch ( IOException ioe2 ) {
                    }
                    ShowUser.info( "Attempt #"+(idx+2)+" to connect.  Please wait..." );
                }

                ShowUser.getInfoDialog().hide();
            }
            catch ( MalformedURLException mue ) {
                ShowUser.error (
                    "URL error attempting to start the server (URL: "+
                    getCodeBase().toString()+"/start_server.cgi)" );
            }
        }

        if ( db == null ) {
            ShowUser.error (
                "Network error trying to connect to database."+
                " Changes will not be saved." );
            db = new OrgChartDb(); // dummy db
            editable = true; // fake editing
        }
        else {
            if ( editflag.equalsIgnoreCase( "yes" ) ||
                 editflag.equalsIgnoreCase( "true" ) ) {
                try {
                    db.openForWriting( dbname );
                    editable = true;
                }
                catch ( IOException ioe ) {
                    ShowUser.error( "Error trying to open the OrgChart for writing." );
                }
                catch ( IllegalAccessException iae ) {
                    override_dialog = new OverrideDialog( ShowUser.getBaseFrame() );
                    override_dialog.addObserver( this );
                    override_dialog.show();
                }
            }
            else {
                try {
                    db.openForReading( dbname );
                }
                catch ( Exception except ) {
					ShowUser.error( "The OrgChart database was not found or is empty." );
                }
            }

			if ( db.isOpen() ) {
				loadInitialNodes();
			}
        }
    }

    private void makeButtons() {

        tab_group = new TabGroup();

        edit_buttons = new CommandButtons();
        super.add( "North", edit_buttons );

        edit_buttons.add( edit = new MyButton( this, "Details..." ) );
        tab_group.add( edit );

        edit_buttons.add( add = new MyButton( this, "Add..." ) );
        tab_group.add( add );

        edit_buttons.add( remove = new MyButton( this, "Remove..." ) );
        tab_group.add( remove );

        edit_buttons.add( revert = new MyButton( this, "Undo All" ) );
        tab_group.add( revert );

        edit_buttons.add( save = new MyButton( this, "Save" ) );
        tab_group.add( save );

        edit_buttons.add( help = new MyButton( this, "Help..." ) );
        tab_group.add( help );

        graph_buttons = new CommandButtons();
        super.add( "South", graph_buttons );

        graph_buttons.add( grow = new MyButton( this, "Zoom In" ) );
        tab_group.add( grow );

        graph_buttons.add( shrink = new MyButton( this, "Zoom Out" ) );
        tab_group.add( shrink );

        graph_buttons.add( more = new MyButton( this, "More" ) );
        tab_group.add( more );

        graph_buttons.add( less = new MyButton( this, "Less" ) );
        tab_group.add( less );
    }

    private void updateButtons() {
        if ( editable ) {
            add.show();
            remove.show();
            revert.show();
            save.show();
        }
        else {
            add.hide();
            remove.hide();
            revert.hide();
            save.hide();
        }
        edit_buttons.layout();
    }

    private void removeNode( OrgNode node ) {
        if ( node_dialog != null ) {
            node_dialog.removeNode( node );
        }
        try {
            if ( graph.getTopNode() == node ) {
                if ( node.getSupervisor() == null ) {
                    graph.setTopNode( null );
                }
                else {
                    graph.setTopNode( node.getSupervisor() );
                }
            }
            node.removeSelf();
            repaint();
        }
        catch ( IllegalAccessException iae ) {
            lostWritability();
        }
        catch ( IOException ioe ) {
            ShowUser.error( "Database error trying to remove the node." );
        }
    }

	/**
	 * DESCRIPTION
	 *
	 * @param PARAMNAME DESCRIPTION
	 * @return DESCRIPTION
	 * @exception CLASSNAME DESCRIPTION
	 * @see CLASSNAME#METHOD
	 */
    public void dialogAction( StdDialog dialog ) {
        if ( dialog == question_dialog ) {
            if ( question_dialog.okSelected() ) { // remove the selected node
                removeNode( remove_node );
            }
        }
        else if ( dialog == remove_dialog ) {
            if ( remove_dialog.removeAllSelected() ) {
                removeNode( remove_node );
            }
            else if ( remove_dialog.removeOneSelected() ) {
                remove_node.giveChildrenToParent();
                removeNode( remove_node );
            }
        }
        else if ( dialog == override_dialog ) {
            if ( override_dialog.overrideSelected() ) {
                try {
                    ShowUser.cursor( Frame.WAIT_CURSOR );
                    db.forceOpenForWriting( dbname );
                    editable = true;
                    updateButtons();
                    ShowUser.setDefaultStatus( "Ctrl-drag to move nodes.  Shift to constrain." );
                    if ( node_dialog != null ) {
                        node_dialog.hide();
                        node_dialog = null;
                    }
                    graph.clear( false ); // NOT permanently from database
                    loadInitialNodes();
                }
                catch ( Exception exp ) {
                    ShowUser.error( "Database error trying to force chart open for writing." );
                }
            }
			else {
				try {
					db.openForReading( dbname );
				}
				catch ( Exception except ) {
					ShowUser.error( "The OrgChart database was not found or is empty." );
				}
				if ( db.isOpen() ) {
					loadInitialNodes();
				}
			}
        }
    }

	/**
	 * DESCRIPTION
	 *
	 * @param PARAMNAME DESCRIPTION
	 * @return DESCRIPTION
	 * @exception CLASSNAME DESCRIPTION
	 * @see CLASSNAME#METHOD
	 */
    public void updateSelected( OrgNode selected_node ) {
        if ( editable ) {
            if ( selected_node == null ) {
                edit.disable();
                if ( graph.getTopNode() == null ) {
                    add.enable();
                }
                else {
                    add.disable();
                }
                remove.disable();
            }
            else {
                edit.enable();
                add.enable();
                remove.enable();
            }
        }
        if ( node_dialog != null ) {
            if ( selected_node != null ) {
                node_dialog.changeNode( selected_node );
            }
        }
    }

	/**
	 * DESCRIPTION
	 *
	 * @param PARAMNAME DESCRIPTION
	 * @return DESCRIPTION
	 * @exception CLASSNAME DESCRIPTION
	 * @see CLASSNAME#METHOD
	 */
    public final OrgChartGraph getGraph() {
        return graph;
    }

	/**
	 * DESCRIPTION
	 *
	 * @param PARAMNAME DESCRIPTION
	 * @return DESCRIPTION
	 * @exception CLASSNAME DESCRIPTION
	 * @see CLASSNAME#METHOD
	 */
    public final OrgChartPrefs getPreferences() {
        return prefs;
    }

	/**
	 * DESCRIPTION
	 *
	 * @param PARAMNAME DESCRIPTION
	 * @return DESCRIPTION
	 * @exception CLASSNAME DESCRIPTION
	 * @see CLASSNAME#METHOD
	 */
    public final OrgChartDb getDatabase() {
        return db;
    }

	/**
	 * DESCRIPTION
	 *
	 * @param PARAMNAME DESCRIPTION
	 * @return DESCRIPTION
	 * @exception CLASSNAME DESCRIPTION
	 * @see CLASSNAME#METHOD
	 */
    public final boolean isEditable() {
        return editable;
    }

	/**
	 * DESCRIPTION
	 *
	 * @param PARAMNAME DESCRIPTION
	 * @return DESCRIPTION
	 * @exception CLASSNAME DESCRIPTION
	 * @see CLASSNAME#METHOD
	 */
    public OrgNode getSelectedNode() {
        return graph.getSelectedNode();
    }

	/**
	 * DESCRIPTION
	 *
	 * @param PARAMNAME DESCRIPTION
	 * @return DESCRIPTION
	 * @exception CLASSNAME DESCRIPTION
	 * @see CLASSNAME#METHOD
	 */
    public NodeDialog getNodeDialog() {
        if ( node_dialog == null ) {
            node_dialog = new NodeDialog( ShowUser.getBaseFrame(), this );
        }
        return node_dialog;
    }

    protected boolean buttonPressed( MyButton button ) {

        OrgNode selected = graph.getSelectedNode();

        if ( button == edit ) {
            if ( selected == null ) {
                ShowUser.info( "You must select a node first." );
            }
            else {
                NodeDialog dialog = getNodeDialog();
                dialog.editNode( selected );
            }
        }
        if ( button == add ) {
            if ( graph.getTopNode() == null ) {
                NodeDialog dialog = getNodeDialog();
                dialog.addTopNode( graph );
            }
            else if ( selected == null ) {
                ShowUser.info( "You must select a node first." );
            }
            else {
                NodeDialog dialog = getNodeDialog();
                dialog.addNode( selected );
            }
        }
        else if ( button == remove ) {
            if ( selected == null ) {
                ShowUser.info( "You must select a node first." );
            }
            else {
                remove_node = selected;
                if ( selected.getSupervisor() == null ) {
                    ShowUser.question( this,
                                       "Are you sure you want to remove all of "+
                                       "the nodes in the chart?  If not, you might "+
                                       "want to edit the selected node and just "+
                                       "change the information, instead of trying "+
                                       "to remove it and all of its subnodes.  Press "+
                                       "OK to remove everything, or CANCEL to abort." );
                    question_dialog = ShowUser.getQuestionDialog();
                }
                else if ( ! selected.hasChildren() ) {
                    ShowUser.question( this,
                                       "Are you sure you want to remove this node?" );
                    question_dialog = ShowUser.getQuestionDialog();
                }
                else {
                    remove_dialog = new RemoveDialog( ShowUser.getBaseFrame() );
                    remove_dialog.addObserver( this );
                    remove_dialog.show();
                }
            }
        }
        else if ( button == grow ) {
            prefs.grow();
            graph.repaint();
        }
        else if ( button == shrink ) {
            prefs.shrink();
            graph.repaint();
        }
        else if ( button == more ) {
            int save_num_vis = db.getNumVisibleNodes();
            prefs.more();
            OrgNode top_node = graph.getTopNode();
            if ( top_node != null ) {
                top_node.hide();
                top_node.show( prefs.max_levels );
            }
            if ( save_num_vis == db.getNumVisibleNodes() ) {
                if ( top_node != null ) {
                    OrgNode top_node_parent = top_node.getSupervisor();
                    if ( top_node_parent != null ) {
                        graph.setTopNode( top_node_parent );
                    }
                }
            }
            graph.repaint();
        }
        else if ( button == less ) {
            prefs.less();
            graph.repaint();
        }
        else if ( button == revert ) {
            try {
                ShowUser.cursor( Frame.WAIT_CURSOR );
                db.undoAll();
                if ( node_dialog != null ) {
                    node_dialog.hide();
                }
                graph.clear( false );
                loadInitialNodes();
            }
            catch ( Exception exp ) {
                ShowUser.error( "Database error trying to revert chart before unsaved changes." );
            }
        }
        else if ( button == save ) {
            if ( ( node_dialog == null ) ||
                 node_dialog.apply() ) {
                try {
                    db.saveAll();
                }
                catch ( IllegalAccessException iae ) {
                    lostWritability();
                }
                catch ( IOException ioe ) {
                    ShowUser.error( "Database error trying to save." );
                }
            }
        }
        else if ( button == help ) {
            try {
                URL url = new URL( "javascript:NetHelp('intro')" );
                getAppletContext().showDocument( url );
            }
            catch ( MalformedURLException mue ) {
                ShowUser.warning( "JavaScript error. Could not open the help window." );
            }
        }
        else {
            return false;
        }
        return true;
    }

    private void saveTopNodeParent() {
        OrgNode top_node = graph.getTopNode();
        if ( top_node == null ) {
            save_top_node_parentid = 0;
        }
        else {
            OrgNode top_node_parent = top_node.getSupervisor();
            if ( top_node_parent == null ) {
                save_top_node_parentid = 0;
            }
            else {
                save_top_node_parentid = top_node_parent.getId();
            }
        }
    }

	/**
	 * Called by someone that noticed an IllegalAccessException on a write, indicating that
	 * we got kicked out of edit mode.  We should update the user interface to indicate that
	 * and tell the user.
	 *
	 * @param PARAMNAME DESCRIPTION
	 * @return DESCRIPTION
	 * @exception CLASSNAME DESCRIPTION
	 * @see CLASSNAME#METHOD
	 */
    public void lostWritability() {
        ShowUser.warning( "Someone has kicked you out of edit mode.  Your changes should "+
                          "have been saved automatically anyway.  The chart will "+
                          "remain open for viewing only.  Sorry." );
        editable = false;
        updateButtons();
        if ( node_dialog != null ) {
            node_dialog.hide();
            node_dialog = null;
        }
        ShowUser.setDefaultStatus( "" );
        ShowUser.cursor( Frame.WAIT_CURSOR );
        saveTopNodeParent();
        try {
            db.undoAll();
        }
        catch ( IllegalAccessException iae ) {
            lostWritability();
        }
        catch ( IOException ioe ) {
            ShowUser.error( "Database I/O error trying to undo and re-open Org Chart." );
        }
        graph.clear( false );
        loadInitialNodes();
    }

	/**
	 * DESCRIPTION
	 *
	 * @param PARAMNAME DESCRIPTION
	 * @return DESCRIPTION
	 * @exception CLASSNAME DESCRIPTION
	 * @see CLASSNAME#METHOD
	 */
    public boolean keyDown( Event evt, int key ) {
        if ( key == '\t' ) {
            if ( evt.shiftDown() ) {
                tab_group.prevFocus( (Component)evt.target );
            }
            else {
                tab_group.nextFocus( (Component)evt.target );
            }
            return true;
        }
        return false;
    }

	/**
	 * DESCRIPTION
	 *
	 * @param PARAMNAME DESCRIPTION
	 * @return DESCRIPTION
	 * @exception CLASSNAME DESCRIPTION
	 * @see CLASSNAME#METHOD
	 */
    public void requestFocus() {
        super.requestFocus();
        tab_group.resetFocus();
    }
}

class MyButton extends Button
{
    OrgChart chart;

	/**
	 * DESCRIPTION
	 *
	 * @param PARAMNAME DESCRIPTION
	 * @return DESCRIPTION
	 * @exception CLASSNAME DESCRIPTION
	 * @see CLASSNAME#METHOD
	 */
    public MyButton( OrgChart chart, String name ) {
        super( name );
        this.chart = chart;
    }

	/**
	 * DESCRIPTION
	 *
	 * @param PARAMNAME DESCRIPTION
	 * @return DESCRIPTION
	 * @exception CLASSNAME DESCRIPTION
	 * @see CLASSNAME#METHOD
	 */
    public boolean action( Event evt, Object arg ) {
        return chart.buttonPressed( this );
    }
}

class RemoveDialog extends SimpleDialog {

    private Button  remove_all;
    private Button  remove_one;
    private boolean remove_all_selected = false;
    private boolean remove_one_selected = false;

    protected RemoveDialog ( Frame frame ) {
        super ( frame, "Remove Selected Node" );

        addButton ( remove_all = new Button ( "Remove All" ) );
        addButton ( remove_one = new Button ( "Remove This One" ) );
        addButton ( new Button ( "Cancel" ) );

        setMaxLineLength( 60 );

        setMessage( "Do you want to remove this node and all "+
                    "of the nodes under it?  Or do you only "+
                    "want to remove the ONE selected node, and "+
                    "give its subnodes to its supervisor node?" );
    }

    public void show() {
        remove_all_selected = false;
        remove_one_selected = false;
        super.show();
    }

    public boolean action( Event evt, Object arg ) {
        if ( evt.target instanceof Button ) {
            if ( evt.target == remove_all ) {
                remove_all_selected = true;
                notifyObservers();
            }
            else if ( evt.target == remove_one ) {
                remove_one_selected = true;
                notifyObservers();
            }
            hide ();
            return( true );
        }
        return( false );
    }

    protected boolean removeAllSelected() {
        return remove_all_selected;
    }

    protected boolean removeOneSelected() {
        return remove_one_selected;
    }
}

class OverrideDialog extends SimpleDialog {

    private Button  override;
    private boolean override_selected = false;

    protected OverrideDialog ( Frame frame ) {
        super ( frame, "Edit Not Available" );

        addButton ( override = new Button ( "Override" ) );
        addButton ( new Button ( "View Only" ) );

        setMaxLineLength( 60 );

        setMessage(
            "The Org Chart is not available for writing. "+
            "It is probably being editted by another user. "+
            "You can view the chart, but you will not be able "+
            "to make any changes.\n\n"+
            "You can override this if you feel that someone has "+
            "inadvertantly left the Org Chart open for writing, "+
            "or if you think there is a problem with the database.  If someone is "+
            "editting the Org Chart and you override them, their "+
            "current changes will be saved; however, they will "+
            "no longer be able to make changes.\n\n"+
            "Please be sure about a decision to override." );
    }

    public void show() {
        override_selected = false;
        super.show();
    }

    public boolean action( Event evt, Object arg ) {
        if ( evt.target instanceof Button ) {
            if ( evt.target == override ) {
                override_selected = true;
            }
			notifyObservers();
            hide ();
            return( true );
        }
        return( false );
    }

    protected boolean overrideSelected() {
        return override_selected;
    }
}
