/**
 * Copyright(c) 1996 DTAI, Incorporated (http://www.dtai.com)
 *
 *                        All rights reserved
 *
 * Permission to use, copy, modify and distribute this material for
 * any purpose and without fee is hereby granted, provided that the
 * above copyright notice and this permission notice appear in all
 * copies, and that the name of DTAI, Incorporated not be used in
 * advertising or publicity pertaining to this material without the
 * specific, prior written permission of an authorized representative of
 * DTAI, Incorporated.
 *
 * DTAI, INCORPORATED MAKES NO REPRESENTATIONS AND EXTENDS NO WARRANTIES,
 * EXPRESS OR IMPLIED, WITH RESPECT TO THE SOFTWARE, INCLUDING, BUT
 * NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 * FITNESS FOR ANY PARTICULAR PURPOSE, AND THE WARRANTY AGAINST
 * INFRINGEMENT OF PATENTS OR OTHER INTELLECTUAL PROPERTY RIGHTS.  THE
 * SOFTWARE IS PROVIDED "AS IS", AND IN NO EVENT SHALL DTAI, INCORPORATED OR
 * ANY OF ITS AFFILIATES BE LIABLE FOR ANY DAMAGES, INCLUDING ANY
 * LOST PROFITS OR OTHER INCIDENTAL OR CONSEQUENTIAL DAMAGES RELATING
 * TO THE SOFTWARE.
 */

import java.util.*;
import orgdata.*;
import java.io.*;
import COM.dtai.util.*;

/**
 * OrgChartDb - The database interface for the OrgChart applet
 *
 * @see		OrgChart
 * @version	1.0b1
 * @author	DTAI, Incorporated
 *
 * $Id: OrgChartDb.java,v 1.21 1996/08/08 02:58:45 kadel Exp $
 *
 * $Source: /cvs/java/OrgChart/OrgChartDb.java,v $
 */

public class OrgChartDb extends OrgChartDbClient {

    Hashtable employee_hash = new Hashtable();
    Hashtable node_hash = new Hashtable();

	/**
	 * DESCRIPTION
	 *
	 * @param PARAMNAME DESCRIPTION
	 * @return DESCRIPTION
	 * @exception CLASSNAME DESCRIPTION
	 * @see CLASSNAME#METHOD
	 */
    public OrgChartDb( String host, int portnum ) throws IOException {
        super( host, portnum );
    }

    /**
     * dummy db if can't connect
     */

	/**
	 * DESCRIPTION
	 *
	 * @param PARAMNAME DESCRIPTION
	 * @return DESCRIPTION
	 * @exception CLASSNAME DESCRIPTION
	 * @see CLASSNAME#METHOD
	 */
    public OrgChartDb() {
    }

	/**
	 * DESCRIPTION
	 *
	 * @param PARAMNAME DESCRIPTION
	 * @return DESCRIPTION
	 * @exception CLASSNAME DESCRIPTION
	 * @see CLASSNAME#METHOD
	 */
    public synchronized void clear() {
        employee_hash.clear();
        node_hash.clear();
    }

	/**
	 * DESCRIPTION
	 *
	 * @param PARAMNAME DESCRIPTION
	 * @return DESCRIPTION
	 * @exception CLASSNAME DESCRIPTION
	 * @see CLASSNAME#METHOD
	 */
    public synchronized void undoAll() throws IOException, IllegalAccessException {
        super.undoAll();
        clear();
    }

	/**
	 * Updates the hash table
	 *
	 * @param PARAMNAME DESCRIPTION
	 * @return DESCRIPTION
	 * @exception CLASSNAME DESCRIPTION
	 * @see CLASSNAME#METHOD
	 */
    public synchronized void newEmployee( Employee employee ) {
        employee_hash.put( new Integer( employee.getId() ), employee );
    }

	/**
	 * Updates the hash table
	 *
	 * @param PARAMNAME DESCRIPTION
	 * @return DESCRIPTION
	 * @exception CLASSNAME DESCRIPTION
	 * @see CLASSNAME#METHOD
	 */
    public synchronized void newNode( OrgNode node ) {
        node_hash.put( new Integer( node.getId() ), node );
    }

	/**
	 * DESCRIPTION
	 *
	 * @param PARAMNAME DESCRIPTION
	 * @return DESCRIPTION
	 * @exception CLASSNAME DESCRIPTION
	 * @see CLASSNAME#METHOD
	 */
    public synchronized void deleteOrgNode( OrgNodeData ond )
		throws IOException, IllegalAccessException {

		node_hash.remove( new Integer( ond.id ) );
		super.deleteOrgNode( ond );
    }

	/**
	 * DESCRIPTION
	 *
	 * @param PARAMNAME DESCRIPTION
	 * @return DESCRIPTION
	 * @exception CLASSNAME DESCRIPTION
	 * @see CLASSNAME#METHOD
	 */
    public synchronized void deleteEmployee( EmployeeData emp )
		throws IOException, IllegalAccessException {

		employee_hash.remove( new Integer( emp.id ) );
		super.deleteEmployee( emp );
    }

	/**
	 * DESCRIPTION
	 *
	 * @param PARAMNAME DESCRIPTION
	 * @return DESCRIPTION
	 * @exception CLASSNAME DESCRIPTION
	 * @see CLASSNAME#METHOD
	 */
    public synchronized Employee getEmployee( int employee_id ) throws IOException, IllegalAccessException {
        Employee employee = (Employee)employee_hash.get( new Integer( employee_id ) );

        if ( employee == null ) {
            EmployeeData emp_data = loadEmployeeById( employee_id );
            employee = new Employee( this, emp_data );
            employee_hash.put( new Integer( employee_id ), employee );
        }
        return employee;
    }

    private Employee insertEmployee( EmployeeData emp )
		throws IOException, IllegalAccessException {

        Employee employee = (Employee)employee_hash.get( new Integer( emp.id ) );

        if ( employee == null ) {
            employee = new Employee( this, emp );
            employee_hash.put( new Integer( emp.id ), employee );
        }
        return employee;
    }

    private void insertNode( OrgChart chart, OrgNodeData ond )
        throws IOException, IllegalAccessException {

        OrgNode new_node = (OrgNode)node_hash.get( new Integer( ond.id ) );

        if ( new_node == null ) {
            new_node = new OrgNode( chart, ond, getEmployee( ond.employee_id ) );
            node_hash.put( new Integer( ond.id ), new_node );
            OrgChartGraph graph = chart.getGraph();
            if ( graph.getTopNode() == null ) {
                graph.setTopNode( new_node );
            }
            for ( Enumeration e = node_hash.elements(); e.hasMoreElements(); ) {
                OrgNode node = (OrgNode)e.nextElement();
                if ( node.getParentId() == ond.id ) {
                    new_node.loadChildren( 1 );
                    break;
                }
            }
        }
        if ( ond.parent_id > 0 ) {
            OrgNode parent = (OrgNode)node_hash.get( new Integer( ond.parent_id ) );
            if ( parent != null ) {
                parent.addNode( new_node );
            }
        }
    }

    private OrgNode insertNode( OrgChart chart, OrgNodeData ond, EmployeeData emp )
        throws IOException, IllegalAccessException {

        OrgNode new_node = (OrgNode)node_hash.get( new Integer( ond.id ) );

        if ( new_node == null ) {
            new_node = new OrgNode( chart, ond, insertEmployee( emp ) );
            node_hash.put( new Integer( ond.id ), new_node );
            OrgChartGraph graph = chart.getGraph();
            if ( graph.getTopNode() == null ) {
                graph.setTopNode( new_node );
            }
        }
        if ( ond.parent_id > 0 ) {
            OrgNode parent = (OrgNode)node_hash.get( new Integer( ond.parent_id ) );
            if ( parent != null ) {
                parent.addNode( new_node );
            }
        }
		return new_node;
    }

	/**
	 * DESCRIPTION
	 *
	 * @param PARAMNAME DESCRIPTION
	 * @return DESCRIPTION
	 * @exception CLASSNAME DESCRIPTION
	 * @see CLASSNAME#METHOD
	 */
    public synchronized OrgNodeData loadOrgNodeById( OrgChart chart, int id )
        throws IOException, IllegalAccessException {
        OrgNodeData ond = loadOrgNodeById( id );
        insertNode( chart, ond );
        return ond;
    }

	/**
	 * DESCRIPTION
	 *
	 * @param PARAMNAME DESCRIPTION
	 * @return DESCRIPTION
	 * @exception CLASSNAME DESCRIPTION
	 * @see CLASSNAME#METHOD
	 */
    public synchronized void loadOrgNodesBySection( String section, OrgChart chart )
        throws IOException, IllegalAccessException {

        ShowUser.status( "Loading chart for "+section );

        Vector nodes = loadOrgNodesBySection( section );

        for ( Enumeration e = nodes.elements(); e.hasMoreElements(); ) {
            OrgNodeData ond = (OrgNodeData)e.nextElement();
            insertNode( chart, ond );
        }

        ShowUser.status( "" );
    }

	/**
	 * DESCRIPTION
	 *
	 * @param PARAMNAME DESCRIPTION
	 * @return DESCRIPTION
	 * @exception CLASSNAME DESCRIPTION
	 * @see CLASSNAME#METHOD
	 */
    public synchronized void loadOrgNodesByParentId( int parent_id, OrgChart chart )
        throws IOException, IllegalAccessException {

        ShowUser.status( "Loading subordinate nodes" );

        Vector nodes = loadOrgNodesByParentId( parent_id );

        for ( Enumeration e = nodes.elements(); e.hasMoreElements(); ) {
            OrgNodeData ond = (OrgNodeData)e.nextElement();
            insertNode( chart, ond );
        }

        ShowUser.status( "" );
    }
	/**
	 * DESCRIPTION
	 *
	 * @param PARAMNAME DESCRIPTION
	 * @return DESCRIPTION
	 * @exception CLASSNAME DESCRIPTION
	 * @see CLASSNAME#METHOD
	 */
    public synchronized void getOrgChartBranchBySection( String section, OrgChart chart,
											int max_levels )
        throws IOException, IllegalAccessException {

        ShowUser.status( "Loading chart for "+section );

        chart.getGraph().freeze();

        Vector nodes = new Vector();
        Vector employees = new Vector();
        Vector levels = new Vector();

        // first read OrgNodeData objects in
        getOrgChartBranchBySection( section, max_levels, nodes, employees, levels );

		Enumeration ee = employees.elements();
		Enumeration le = levels.elements();
        for ( Enumeration ne = nodes.elements(); ne.hasMoreElements(); ) {
            OrgNodeData ond = (OrgNodeData)ne.nextElement();
            EmployeeData emp = (EmployeeData)ee.nextElement();
            Integer level = (Integer)le.nextElement();
            OrgNode node = insertNode( chart, ond, emp );
			if ( level.intValue() > 1 ) {
				node.setChildren();
			}
			ee.hasMoreElements();
			le.hasMoreElements();
        }

        chart.getGraph().thaw();

        ShowUser.status( "" );
    }

	/**
	 * DESCRIPTION
	 *
	 * @param PARAMNAME DESCRIPTION
	 * @return DESCRIPTION
	 * @exception CLASSNAME DESCRIPTION
	 * @see CLASSNAME#METHOD
	 */
    public synchronized void getOrgChartBranchByParent( int parent_id, OrgChart chart,
										   int max_levels )
        throws IOException, IllegalAccessException {

        ShowUser.status( "Loading subordinate nodes" );

        chart.getGraph().freeze();

        Vector nodes = new Vector();
        Vector employees = new Vector();
        Vector levels = new Vector();

        // first read OrgNodeData objects in
        getOrgChartBranchByParentId( parent_id, max_levels, nodes, employees, levels );

		Enumeration ee = employees.elements();
		Enumeration le = levels.elements();
        for ( Enumeration ne = nodes.elements();
			  ne.hasMoreElements(); ) {
            OrgNodeData ond = (OrgNodeData)ne.nextElement();
            EmployeeData emp = (EmployeeData)ee.nextElement();
            Integer level = (Integer)le.nextElement();
            OrgNode node = insertNode( chart, ond, emp );
			if ( level.intValue() > 1 ) {
				node.setChildren();
			}
			ee.hasMoreElements();
			le.hasMoreElements();
        }

        chart.getGraph().thaw();

        ShowUser.status( "" );
    }

	/**
	 * DESCRIPTION
	 *
	 * @param PARAMNAME DESCRIPTION
	 * @return DESCRIPTION
	 * @exception CLASSNAME DESCRIPTION
	 * @see CLASSNAME#METHOD
	 */
    public synchronized int getNumVisibleNodes() {
        int num_vis = 0;
        for ( Enumeration e = node_hash.elements(); e.hasMoreElements(); ) {
            if ( ! ((OrgNode)e.nextElement()).isHidden() ) {
                num_vis++;
            }
        }
        return num_vis;
    }
}
