/**
 * Copyright(c) 1996 DTAI, Incorporated (http://www.dtai.com)
 *
 *                        All rights reserved
 *
 * Permission to use, copy, modify and distribute this material for
 * any purpose and without fee is hereby granted, provided that the
 * above copyright notice and this permission notice appear in all
 * copies, and that the name of DTAI, Incorporated not be used in
 * advertising or publicity pertaining to this material without the
 * specific, prior written permission of an authorized representative of
 * DTAI, Incorporated.
 *
 * DTAI, INCORPORATED MAKES NO REPRESENTATIONS AND EXTENDS NO WARRANTIES,
 * EXPRESS OR IMPLIED, WITH RESPECT TO THE SOFTWARE, INCLUDING, BUT
 * NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 * FITNESS FOR ANY PARTICULAR PURPOSE, AND THE WARRANTY AGAINST
 * INFRINGEMENT OF PATENTS OR OTHER INTELLECTUAL PROPERTY RIGHTS.  THE
 * SOFTWARE IS PROVIDED "AS IS", AND IN NO EVENT SHALL DTAI, INCORPORATED OR
 * ANY OF ITS AFFILIATES BE LIABLE FOR ANY DAMAGES, INCLUDING ANY
 * LOST PROFITS OR OTHER INCIDENTAL OR CONSEQUENTIAL DAMAGES RELATING
 * TO THE SOFTWARE.
 */

import COM.dtai.net.*;
import COM.dtai.serial.*;
import COM.dtai.util.*;
import java.net.*;
import java.io.*;
import java.util.*;
import orgdata.*;

/**
 * OrgChartDbServer - listens for and accepts clients, then spawns a thread
 * to handle the client requests.
 *
 * @version	1.0b1
 * @author	DTAI, Incorporated
 *
 * $Id: OrgChartDbServer.java,v 1.9 1996/08/08 02:58:47 kadel Exp $
 *
 * $Source: /cvs/java/OrgChartServer/OrgChartDbServer.java,v $
 */

public class OrgChartDbServer extends DbServer
{

	/**
	 * DESCRIPTION
	 *
	 * @param PARAMNAME DESCRIPTION
	 * @return DESCRIPTION
	 * @exception CLASSNAME DESCRIPTION
	 * @see CLASSNAME#METHOD
	 */
    public OrgChartDbServer( int portnum ) {
        super ( portnum );
    }

	/**
	 * DESCRIPTION
	 *
	 * @param PARAMNAME DESCRIPTION
	 * @return DESCRIPTION
	 * @exception CLASSNAME DESCRIPTION
	 * @see CLASSNAME#METHOD
	 */
    public static void main( String argv[] ) {

		new OrgChartDbServer( getPortnum( argv, 9006 ) );

System.err.println( "enabling logging" );
Syslog.enableLogging ( true );
    }

	/**
	 * DESCRIPTION
	 *
	 * @param PARAMNAME DESCRIPTION
	 * @return DESCRIPTION
	 * @exception CLASSNAME DESCRIPTION
	 * @see CLASSNAME#METHOD
	 */
    public void createClientPort( Socket socket ) {
        try {
			new OrgChartDbPortToClient ( this, socket );
        }
        catch( IOException ioe ) {
            Syslog.error( "IO Exception creating client port handler object" );
            Syslog.error( ioe );
        }
    }

	/**
	 * DESCRIPTION
	 *
	 * @param PARAMNAME DESCRIPTION
	 * @return DESCRIPTION
	 * @exception CLASSNAME DESCRIPTION
	 * @see CLASSNAME#METHOD
	 */
    public DbData createDbData( String name ) {
		return new OrgChartDbData( name );
	}
}

class OrgChartDbPortToClient extends DbPortToClient
{
    private OrgNodeData ndata = new OrgNodeData();
    private EmployeeData edata = new EmployeeData();
	private OrgChartDbData dbdata;

	/**
	 * DESCRIPTION
	 *
	 * @param NAME DESCRIPTION
	 * @return DESCRIPTION
	 * @exception DESCRIPTION
	 */
    public OrgChartDbPortToClient( OrgChartDbServer server, Socket socket )
        throws IOException {

        super( server, socket );
    }

	/**
	 * DESCRIPTION
	 *
	 * @param NAME DESCRIPTION
	 * @return DESCRIPTION
	 * @exception DESCRIPTION
	 */
    protected void setDbData( DbData dbdata ) {
		super.setDbData( dbdata );
		this.dbdata = (OrgChartDbData)dbdata;
	}

	void sendBranch(SerialDataIndex sdi, int nlevel)
    throws IOException, SerialDataException
	{
		EmployeeManager emgr = dbdata.getEmployeeManager( this );
		OrgNodeManager nmgr = dbdata.getOrgNodeManager( this );

		if (nlevel == 0) return;

		SerialDataValue sdv = sdi.getValue(ndata);

		if (sdv == null) return;

		nlevel--;
		Enumeration nodes = sdv.elements();
		while ( nmgr.loadNextElement( nodes, ndata ) ) {
			writeInt( OrgChartDbCommands.SendingObject );
			writeInt( nlevel+1 );
			ndata.write ( this );
			edata.clear();
			edata.id = ndata.employee_id;
			Enumeration e = emgr.elements( emgr.eii, edata );
			emgr.loadNextElement( e, edata );
			edata.write ( this );
			ndata.parent_id = ndata.id;
			sendBranch(nmgr.pii, nlevel);
		}
	}

    protected synchronized boolean process( int command )
		throws IOException, SerialDataException, IllegalAccessException {

		if ( super.process( command ) ) {
			return true;
		}

		EmployeeManager emgr = dbdata.getEmployeeManager( this );
		OrgNodeManager nmgr = dbdata.getOrgNodeManager( this );

        if ( command == OrgChartDbCommands.GetOrgNodeById ) {
            ndata.id = readInt();
            Syslog.warning( "GetOrgNodeById: " + ndata.id );

			Enumeration e = nmgr.elements( nmgr.nii, ndata );

			while ( nmgr.loadNextElement( e, ndata ) ) {
				writeInt( OrgChartDbCommands.SendingObject );
				ndata.write ( this );
			}

			writeInt( OrgChartDbCommands.NoMoreObjects );
        }
        else if ( command == OrgChartDbCommands.GetOrgNodeBySection ) {
            ndata.section_title = readUTF();
            Syslog.warning( "GetOrgNodeBySection: " + ndata.section_title );

			Enumeration e = nmgr.elements( nmgr.sti, ndata );

			while ( nmgr.loadNextElement( e, ndata ) ) {
				writeInt( OrgChartDbCommands.SendingObject );
				ndata.write ( this );
			}

			writeInt( OrgChartDbCommands.NoMoreObjects );
        }
        else if ( command == OrgChartDbCommands.GetOrgNodeByParent ) {
            ndata.parent_id = readInt();
            Syslog.warning( "GetOrgNodeByParent: " + ndata.parent_id );

			Enumeration e = nmgr.elements( nmgr.pii, ndata );

			while ( nmgr.loadNextElement( e, ndata ) ) {
				writeInt( OrgChartDbCommands.SendingObject );
				ndata.write ( this );
			}

			writeInt( OrgChartDbCommands.NoMoreObjects );
        }
        else if ( command == OrgChartDbCommands.GetOrgChartBranchByParent ) {
            ndata.parent_id = readInt();
            int nlevel = readInt();
            Syslog.warning( "GetOrgChartBranchByParent: " + ndata.parent_id );

			sendBranch( nmgr.pii, nlevel );

            writeInt( OrgChartDbCommands.NoMoreObjects );
        }
        else if ( command == OrgChartDbCommands.GetOrgChartBranchBySection ) {
            ndata.section_title = readUTF();
            int nlevel = readInt();
            Syslog.warning( "GetOrgChartBranchBySection: " + ndata.section_title );

			sendBranch(nmgr.sti, nlevel);

            writeInt( OrgChartDbCommands.NoMoreObjects );
        }
        else if ( command == OrgChartDbCommands.GetEmployeeById ) {
            edata.id = readInt();
            Syslog.warning( "GetEmployeeById: " + edata.id );

			Enumeration e = emgr.elements( emgr.eii, edata );

			while ( emgr.loadNextElement( e, edata ) ) {
				writeInt( OrgChartDbCommands.SendingObject );
				edata.write ( this );
			}

			writeInt( OrgChartDbCommands.NoMoreObjects );
        }
        else if ( command == OrgChartDbCommands.GetMaxOrgNodeId ) {
            Syslog.warning( "GetMaxOrgNodeId" );

			nmgr.validate(nmgr.nii);

			writeInt( OrgChartDbCommands.Success );
			writeInt( nmgr.nii.max_id );
        }
        else if ( command == OrgChartDbCommands.GetMaxEmployeeId ) {
            Syslog.warning( "GetMaxEmployeeId" );

			emgr.validate(emgr.eii);

			writeInt( OrgChartDbCommands.Success );
			writeInt( emgr.eii.max_id );
        }
        else if ( command == OrgChartDbCommands.DeleteOrgNode ) {
            ndata.read( this );
            Syslog.warning( "DeleteOrgNode: " + ndata.id );

			nmgr.delete(nmgr.nii, ndata);
			writeInt( OrgChartDbCommands.Success );
        }
        else if ( command == OrgChartDbCommands.DeleteEmployee ) {
            edata.read( this );
            Syslog.warning( "DeleteEmployee: " + edata.id );

			emgr.delete(emgr.eii, edata);
			writeInt( OrgChartDbCommands.Success );
        }
        else if ( command == OrgChartDbCommands.SaveOrgNode ) {
            ndata.read( this );
            Syslog.warning( "SaveOrgNode: " + ndata.id );

			nmgr.update(ndata);
			writeInt( OrgChartDbCommands.Success );
        }
        else if ( command == OrgChartDbCommands.SaveEmployee ) {
            edata.read( this );
            Syslog.warning( "SaveEmployee: " + edata.id );

			emgr.update(edata);
			writeInt( OrgChartDbCommands.Success );
        }
        else {
            Syslog.error("Unknown Org Chart Server command");
			return false;
        }
		return true;
    }
}

class OrgChartDbData extends DbData {

	protected EmployeeManager read_em;
	protected OrgNodeManager read_nm;

	protected EmployeeManager edit_em;
	protected OrgNodeManager edit_nm;

	public OrgChartDbData( String name ) {
		super( name );

		addReadManager( read_em = new EmployeeManager(name, false) );
		addReadManager( read_nm = new OrgNodeManager(name, false) );

		addEditManager( edit_em = new EmployeeManager(name, true) );
		addEditManager( edit_nm = new OrgNodeManager(name, true) );
	}

	/**
	 * DESCRIPTION
	 *
	 * @param PARAMNAME DESCRIPTION
	 * @return DESCRIPTION
	 * @exception CLASSNAME DESCRIPTION
	 * @see CLASSNAME#METHOD
	 */
    public EmployeeManager getEmployeeManager( OrgChartDbPortToClient port )
	{
		if (current_editor == port) return edit_em;
		else return read_em;
    }

	/**
	 * DESCRIPTION
	 *
	 * @param PARAMNAME DESCRIPTION
	 * @return DESCRIPTION
	 * @exception CLASSNAME DESCRIPTION
	 * @see CLASSNAME#METHOD
	 */
    public OrgNodeManager getOrgNodeManager( OrgChartDbPortToClient port )
	{
		if (current_editor == port) return edit_nm;
		else return read_nm;
    }
}
