/**
 * Copyright(c) 1996 DTAI, Incorporated (http://www.dtai.com)
 *
 *                        All rights reserved
 *
 * Permission to use, copy, modify and distribute this material for
 * any purpose and without fee is hereby granted, provided that the
 * above copyright notice and this permission notice appear in all
 * copies, and that the name of DTAI, Incorporated not be used in
 * advertising or publicity pertaining to this material without the
 * specific, prior written permission of an authorized representative of
 * DTAI, Incorporated.
 *
 * DTAI, INCORPORATED MAKES NO REPRESENTATIONS AND EXTENDS NO WARRANTIES,
 * EXPRESS OR IMPLIED, WITH RESPECT TO THE SOFTWARE, INCLUDING, BUT
 * NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 * FITNESS FOR ANY PARTICULAR PURPOSE, AND THE WARRANTY AGAINST
 * INFRINGEMENT OF PATENTS OR OTHER INTELLECTUAL PROPERTY RIGHTS.  THE
 * SOFTWARE IS PROVIDED "AS IS", AND IN NO EVENT SHALL DTAI, INCORPORATED OR
 * ANY OF ITS AFFILIATES BE LIABLE FOR ANY DAMAGES, INCLUDING ANY
 * LOST PROFITS OR OTHER INCIDENTAL OR CONSEQUENTIAL DAMAGES RELATING
 * TO THE SOFTWARE.
 */

import java.awt.*;
import java.awt.image.*;
import java.util.*;
import java.io.*;
import java.net.*;
import COM.dtai.gui.*;
import COM.dtai.diagram.*;
import COM.dtai.util.*;

/**
 * OrgChartGraph - The GadgetManager on which the OrgChart links and nodes
 * are drawn.
 *
 * @version	1.0b1
 * @author	DTAI, Incorporated
 *
 * $Id: OrgChartGraph.java,v 1.12 1996/08/01 17:03:12 kadel Exp $
 *
 * $Source: /cvs/java/OrgChart/OrgChartGraph.java,v $
 */

public class OrgChartGraph extends GadgetManager {

    private NodeBackground node_background;
    private Vector node_components = new Vector();

    private GadgetLayer title_layer;
    private GadgetLayer link_layer;
    private GadgetLayer node_layer;

    private TitleGadget title_gadget;

    private OrgChart chart;
    private OrgChartPrefs prefs;

    private OrgNode selected_node;
    private OrgNode top_node;

    private boolean showing_wait_cursor = false;

	/**
	 * DESCRIPTION
	 *
	 * @param PARAMNAME DESCRIPTION
	 * @return DESCRIPTION
	 * @exception CLASSNAME DESCRIPTION
	 * @see CLASSNAME#METHOD
	 */
    public OrgChartGraph( OrgChart chart ) {
        this.chart = chart;
        prefs = chart.getPreferences();

        setBackground( prefs.chart_background_color );

        if ( ( prefs.chart_background_image != null ) &&
             ( ! prefs.chart_background_image.trim().equals( "" ) ) ) {
            URL image_url = null;
            try {
                image_url = new URL( chart.getDocumentBase(), prefs.chart_background_image );
            }
            catch ( MalformedURLException mue ) {
                ShowUser.error ( "Could not form a valid URL from the document base "+
                                 chart.getDocumentBase()+" and "+prefs.chart_background_image );
            }
            if ( image_url != null ) {
                Image image = chart.getImage( image_url );
                setBackgroundImage( image );
            }
        }

        // the background style and
        // components could be setable through
        // preferences at some point

        if ( ! prefs.suppress_node_background ) {
            RectBackground rect = new RectBackground();
            node_background = rect;
            rect.setBorderColor( prefs.node_border_color );
            rect.setBackground( prefs.node_background_color );
            rect.setArmColor( prefs.node_arm_color );
            rect.setHiliteColor( prefs.node_hilite_color );
            rect.setBorderWidth( prefs.node_border_width );
            rect.setShadowColor( prefs.node_shadow_color );
            rect.setShadowOffset( prefs.node_shadow_offset );
            rect.roundCorners( prefs.round_node_corners );
            rect.threeDBorders( prefs.threed_borders );
            rect.transparentBackground( prefs.transparent_node_background );
        }

        TextNodeComponent text;

        node_components.addElement( text = new EmployeeNameComponent() );
        text.setFont( new Font( prefs.node_font_face, prefs.node_font_style,
                        prefs.node_font_size ) );
        text.setColor( prefs.node_text_color );
        text.setHiliteColor( prefs.node_text_hilite_color );

        node_components.addElement( text = new RoleComponent() );
        text.setFont( new Font( prefs.node_font_face, prefs.node_font_style,
                        prefs.node_font_size ) );
        text.setColor( prefs.node_text_color );
        text.setHiliteColor( prefs.node_text_hilite_color );

        title_layer = getLayer( 0 );
        link_layer = getLayer( 1 );
        node_layer = getLayer( 2 );

        link_layer.setClickable( false );
    }

	/**
	 * DESCRIPTION
	 *
	 * @param PARAMNAME DESCRIPTION
	 * @return DESCRIPTION
	 * @exception CLASSNAME DESCRIPTION
	 * @see CLASSNAME#METHOD
	 */
    public OrgChartPrefs getPreferences() {
        return prefs;
    }

	/**
	 * DESCRIPTION
	 *
	 * @param PARAMNAME DESCRIPTION
	 * @return DESCRIPTION
	 * @exception CLASSNAME DESCRIPTION
	 * @see CLASSNAME#METHOD
	 */
    public void repaint () {
        super.repaint();
    }

	/**
	 * DESCRIPTION
	 *
	 * @param PARAMNAME DESCRIPTION
	 * @return DESCRIPTION
	 * @exception CLASSNAME DESCRIPTION
	 * @see CLASSNAME#METHOD
	 */
    public void paint ( Graphics g ) {

        ShowUser.status( "Drawing the org chart" );

        ShowUser.cursor( Frame.WAIT_CURSOR );
        repainting = true;
        initImage();

        node_layer.setDragable( chart.isEditable() );

        link_layer.removeAllGadgets();

        if ( top_node != null ) {
            for ( Enumeration e = node_components.elements(); e.hasMoreElements(); ) {
                NodeComponent nc = (NodeComponent)e.nextElement();
                if ( nc instanceof TextNodeComponent ) {
                    TextNodeComponent text = (TextNodeComponent)nc;
                    text.setFont( new Font( prefs.node_font_face, prefs.node_font_style,
                                    prefs.node_font_size ) );
                    text.setColor( prefs.node_text_color );
                }
            }

            top_node.hide();
            top_node.show( prefs.max_levels );
            top_node.resize( top_node.preferredSize() );
            top_node.setUniformSizes( prefs.max_levels );
            Dimension space = top_node.calcSpace( prefs.max_levels );
            Point upper_left = new Point( 0, 0 );

            Dimension title_size = null;

            if ( top_node.getSectionTitle() == null ) {
                if ( title_gadget != null ) {
                    title_layer.removeGadget( title_gadget );
                    title_gadget = null;
                }
            }
            else {
                if ( title_gadget == null ) {
                    title_gadget = new TitleGadget( this, top_node.getSectionTitle(), top_node.getSectionURL() );
                }
                else {
                    title_gadget.setText( top_node.getSectionTitle() );
                    title_gadget.setURL( top_node.getSectionURL() );
                }
                title_gadget.setFont( new Font(
                    prefs.title_font_face, prefs.title_font_style,
                    prefs.title_font_size ) );
                title_gadget.setColor( prefs.title_text_color );
                title_gadget.move( prefs.horiz_spacing, prefs.vert_spacing );
                title_layer.addGadget( title_gadget );
                title_size = title_gadget.size();
                upper_left.y += title_size.height + prefs.vert_spacing;
            }

            upper_left.x += prefs.horiz_spacing;
            upper_left.y += prefs.vert_spacing;
            top_node.setLocations( upper_left, prefs.max_levels );
            Point start = top_node.location();
            Point min = new Point( start.x, start.y );
            Point max = new Point( start.x, start.y );
            top_node.calcExtents( min, max, prefs.max_levels );

            int x_offset_for_title = 0;
            int y_offset_for_title = 0;
            if ( title_gadget != null ) {
                if ( title_size.width > ( max.x - min.x ) ) {
                    x_offset_for_title = ( title_size.width - ( max.x - min.x ) ) / 2;
                }
                y_offset_for_title = title_size.height + prefs.vert_spacing;
            }

            if ( ( x_offset_for_title != 0 ) ||
                 ( min.x != upper_left.x ) ) {
                top_node.setLocations( new Point( ( ( upper_left.x - min.x ) + x_offset_for_title ),
                                                  upper_left.y ),
                                       prefs.max_levels );
            }

            top_node.addLinks( link_layer, prefs.max_levels );

            int max_width = max.x - min.x;
            int max_height = max.y - min.y;

            if ( title_gadget != null ) {
                max_height += title_size.height + prefs.vert_spacing;
                if ( max_width > title_size.width ) {
                    if ( x_offset_for_title > 0 ) {
                        title_gadget.move( ( ( max_width - title_size.width ) / 2 ) + prefs.horiz_spacing,
                                           prefs.vert_spacing );
                    }
                    else {
                        title_gadget.move( ( top_node.location().x +
                                             ( top_node.size().width / 2 ) -
                                             ( title_size.width / 2 ) ),
                                           prefs.vert_spacing );
/*
                        title_gadget.move( ( ( max_width - title_size.width ) / 2 ),
                                           prefs.vert_spacing );
*/
                    }
                }
                else {
                    max_width = title_size.width;
                }
            }

            Dimension new_image_size = new Dimension( max_width + ( prefs.horiz_spacing * 2 ),
                                                      max_height + ( prefs.vert_spacing * 2 ) );

            setImageSize( new_image_size );
        }
        super.paint( g );
        ShowUser.cursor( Frame.DEFAULT_CURSOR );
        ShowUser.status( "" );
    }

	/**
	 * DESCRIPTION
	 *
	 * @param PARAMNAME DESCRIPTION
	 * @return DESCRIPTION
	 * @exception CLASSNAME DESCRIPTION
	 * @see CLASSNAME#METHOD
	 */
    public void addNode( OrgNode node ) {
        node_layer.addGadget( node );
        if ( isVisible() ) {
            repaint();
        }
    }

	/**
	 * DESCRIPTION
	 *
	 * @param PARAMNAME DESCRIPTION
	 * @return DESCRIPTION
	 * @exception CLASSNAME DESCRIPTION
	 * @see CLASSNAME#METHOD
	 */
    public void removeNode( OrgNode node, boolean permanent )
        throws IOException, IllegalAccessException {

        freeze();
        if ( node == top_node ) {
            setTopNode( null );
        }
        if ( node == selected_node ) {
            deselectAll();
        }
        node_layer.removeGadget( node );
        if ( permanent ) {
            node.getEmployee().delete();
            node.delete();
        }
        if ( isVisible() ) {
            repaint();
        }
        thaw();
    }

	/**
	 * DESCRIPTION
	 *
	 * @param PARAMNAME DESCRIPTION
	 * @return DESCRIPTION
	 * @exception CLASSNAME DESCRIPTION
	 * @see CLASSNAME#METHOD
	 */
    public void clear( boolean permanent ) {
        freeze();
        try {
            removeNode( top_node, permanent );
        }
        catch ( IllegalAccessException iae ) {
            chart.lostWritability();
        }
        catch ( IOException ioe ) {
            ShowUser.error( "Database error trying to remove all nodes from the database." );
        }
        super.clear();
        deselectAll();
        thaw();
    }

	/**
	 * DESCRIPTION
	 *
	 * @param PARAMNAME DESCRIPTION
	 * @return DESCRIPTION
	 * @exception CLASSNAME DESCRIPTION
	 * @see CLASSNAME#METHOD
	 */
    public void setTopNode( OrgNode node ) {
        top_node = node;
        if ( isVisible() ) {
            repaint();
        }
    }

	/**
	 * DESCRIPTION
	 *
	 * @param PARAMNAME DESCRIPTION
	 * @return DESCRIPTION
	 * @exception CLASSNAME DESCRIPTION
	 * @see CLASSNAME#METHOD
	 */
    public OrgNode getTopNode() {
        return top_node;
    }

	/**
	 * DESCRIPTION
	 *
	 * @param PARAMNAME DESCRIPTION
	 * @return DESCRIPTION
	 * @exception CLASSNAME DESCRIPTION
	 * @see CLASSNAME#METHOD
	 */
    public OrgNode getSelectedNode() {
        return selected_node;
    }

    /**
     * this is called by the OrgNode.
     */
    protected void drawNode( Graphics g, OrgNode node,
                          Point location, Dimension size,
                          boolean hilited, boolean disabled, boolean armed ) {

        node_background.draw( g, location, size, hilited, disabled, armed, false );

        Insets insets = node_background.insets();
        int center_x = location.x + insets.left +
                       ( ( size.width - ( insets.left + insets.right ) ) / 2 );
        int next_y = location.y + insets.top;

        for ( Enumeration e = node_components.elements(); e.hasMoreElements(); ) {
            NodeComponent nc = (NodeComponent)e.nextElement();
            nc.draw( g, node, center_x, next_y, hilited, disabled, armed );
            next_y += nc.size( node ).height;
        }
    }

    /**
     * this is called by the OrgNode.
     */
    protected void armNode( Graphics g, OrgNode node,
                            Point location, Dimension size,
                            boolean hilited ) {

        Point offsets = getOffsets();

        Graphics erase_g = node_background.arm( g,
            new Point( location.x + offsets.x, location.y + offsets.y ),
            size, hilited );

        if ( erase_g != null ) {
            Insets insets = node_background.insets();
            int center_x = location.x + insets.left +
                           ( ( size.width - ( insets.left + insets.right ) ) / 2 );
            int next_y = location.y + insets.top;

            for ( Enumeration e = node_components.elements(); e.hasMoreElements(); ) {
                NodeComponent nc = (NodeComponent)e.nextElement();
                nc.draw( erase_g, node, center_x + offsets.x, next_y + offsets.y );
                next_y += nc.size( node ).height;
            }
        }
    }

    /**
     * this is called by the OrgNode.
     */
    protected void disarmNode( Graphics g, OrgNode node,
                            Point location, Dimension size,
                            boolean hilited ) {

        Point offsets = getOffsets();
        Graphics erase_g = node_background.disarm( g,
            new Point( location.x + offsets.x, location.y + offsets.y ),
            size, hilited );

        if ( erase_g != null ) {
            Insets insets = node_background.insets();
            int center_x = location.x + insets.left +
                           ( ( size.width - ( insets.left + insets.right ) ) / 2 );
            int next_y = location.y + insets.top;

            for ( Enumeration e = node_components.elements(); e.hasMoreElements(); ) {
                NodeComponent nc = (NodeComponent)e.nextElement();
                nc.draw( erase_g, node, center_x + offsets.x, next_y + offsets.y );
                next_y += nc.size( node ).height;
            }
        }
    }

    protected Dimension preferredNodeSize( OrgNode node ) {
        Dimension pref = new Dimension( 0, 0 );

        Insets insets = node_background.insets();

        for ( Enumeration e = node_components.elements();
              e.hasMoreElements(); ) {
            Dimension size = ((NodeComponent)e.nextElement()).size( node );
            pref.width = Math.max( pref.width, size.width );
            pref.height += size.height;
        }
        pref.width += insets.left + insets.right;
        pref.height += insets.top + insets.bottom;
        return pref;
    }

	/**
	 * DESCRIPTION
	 *
	 * @param PARAMNAME DESCRIPTION
	 * @return DESCRIPTION
	 * @exception CLASSNAME DESCRIPTION
	 * @see CLASSNAME#METHOD
	 */
    public void deselectAll() {
        super.deselectAll();
        selected_node = null;
        chart.updateSelected( selected_node );
    }

	/**
	 * DESCRIPTION
	 *
	 * @param PARAMNAME DESCRIPTION
	 * @return DESCRIPTION
	 * @exception CLASSNAME DESCRIPTION
	 * @see CLASSNAME#METHOD
	 */
    public void nodeSelected( OrgNode node ) {
        deselectAll();
        selected_node = node;
        chart.updateSelected( selected_node );
    }

	/**
	 * DESCRIPTION
	 *
	 * @param PARAMNAME DESCRIPTION
	 * @return DESCRIPTION
	 * @exception CLASSNAME DESCRIPTION
	 * @see CLASSNAME#METHOD
	 */
    public OrgChartDb getDatabase() {
        return chart.getDatabase();
    }
}
