/**
 * Copyright(c) 1996 DTAI, Incorporated (http://www.dtai.com)
 *
 *                        All rights reserved
 *
 * Permission to use, copy, modify and distribute this material for
 * any purpose and without fee is hereby granted, provided that the
 * above copyright notice and this permission notice appear in all
 * copies, and that the name of DTAI, Incorporated not be used in
 * advertising or publicity pertaining to this material without the
 * specific, prior written permission of an authorized representative of
 * DTAI, Incorporated.
 *
 * DTAI, INCORPORATED MAKES NO REPRESENTATIONS AND EXTENDS NO WARRANTIES,
 * EXPRESS OR IMPLIED, WITH RESPECT TO THE SOFTWARE, INCLUDING, BUT
 * NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 * FITNESS FOR ANY PARTICULAR PURPOSE, AND THE WARRANTY AGAINST
 * INFRINGEMENT OF PATENTS OR OTHER INTELLECTUAL PROPERTY RIGHTS.  THE
 * SOFTWARE IS PROVIDED "AS IS", AND IN NO EVENT SHALL DTAI, INCORPORATED OR
 * ANY OF ITS AFFILIATES BE LIABLE FOR ANY DAMAGES, INCLUDING ANY
 * LOST PROFITS OR OTHER INCIDENTAL OR CONSEQUENTIAL DAMAGES RELATING
 * TO THE SOFTWARE.
 */

import java.applet.*;
import java.awt.*;

/**
 * OrgChartPrefs - the display preferences for the OrgChart
 *
 * @version	1.0b1
 * @author	DTAI, Incorporated
 *
 * $Id: OrgChartPrefs.java,v 1.4 1996/07/27 19:30:27 kadel Exp $
 *
 * $Source: /cvs/java/OrgChart/OrgChartPrefs.java,v $
 */

public class OrgChartPrefs {
    Color foreground_color = Color.black;
    Color background_color = Color.lightGray;
    Color chart_background_color = Color.white;
    String chart_background_image = null;
    int vert_spacing = 20;
    int horiz_spacing = 10;
    int max_levels = 2;
    String node_font_face = "Helvetica";
    int node_font_size = 18;
    int node_font_style = Font.PLAIN;
    String title_font_face = "TimesRoman";
    int title_font_size = 36;
    int title_font_style = Font.BOLD;
    Color node_text_color = Color.black;
    Color title_text_color = Color.black;
    Color link_color = Color.black;
    Color node_border_color = Color.black;
    Color node_background_color = Color.white;
    Color node_arm_color = Color.gray;
    Color node_hilite_color = Color.yellow;
    Color node_text_hilite_color = Color.black;
    Color node_shadow_color = Color.gray;
    int node_shadow_offset = 5;
    int node_border_width = 1;
    int link_thickness = 1;
    boolean round_node_corners = true;
    boolean threed_borders = false;
    boolean suppress_node_background = false;
    boolean transparent_node_background = false;

    private Color getColor( String name ) {
        if ( name.charAt( 0 ) == '#' ) {
            try {
                int r = Integer.valueOf( name.substring( 1, 3 ), 16 ).intValue();
                int g = Integer.valueOf( name.substring( 3, 5 ), 16 ).intValue();
                int b = Integer.valueOf( name.substring( 5, 7 ), 16 ).intValue();
                return new Color( r, g, b );
            }
            catch ( Exception e ) {
                return null;
            }
        }
        else if ( name.equalsIgnoreCase( "White" ) ) {
            return Color.white;
        }
        else if ( name.equalsIgnoreCase( "Black" ) ) {
            return Color.black;
        }
        else if ( name.equalsIgnoreCase( "LightGray" ) ) {
            return Color.lightGray;
        }
        else if ( name.equalsIgnoreCase( "Gray" ) ) {
            return Color.gray;
        }
        else if ( name.equalsIgnoreCase( "DarkGray" ) ) {
            return Color.darkGray;
        }
        else if ( name.equalsIgnoreCase( "Red" ) ) {
            return Color.red;
        }
        else if ( name.equalsIgnoreCase( "Pink" ) ) {
            return Color.pink;
        }
        else if ( name.equalsIgnoreCase( "Orange" ) ) {
            return Color.orange;
        }
        else if ( name.equalsIgnoreCase( "Yellow" ) ) {
            return Color.yellow;
        }
        else if ( name.equalsIgnoreCase( "Green" ) ) {
            return Color.green;
        }
        else if ( name.equalsIgnoreCase( "Magenta" ) ) {
            return Color.magenta;
        }
        else if ( name.equalsIgnoreCase( "Cyan" ) ) {
            return Color.cyan;
        }
        else if ( name.equalsIgnoreCase( "Blue" ) ) {
            return Color.blue;
        }
        return null;
    }

	/**
	 * DESCRIPTION
	 *
	 * @param PARAMNAME DESCRIPTION
	 * @return DESCRIPTION
	 * @exception CLASSNAME DESCRIPTION
	 * @see CLASSNAME#METHOD
	 */
    public OrgChartPrefs( Applet applet ) {

        String param;
        Color color;

        param = applet.getParameter( "ForegroundColor" );
        if ( param != null ) {
            color = Color.getColor( param );
            if ( color == null ) {
                color = getColor( param );
            }
            if ( color != null ) {
                foreground_color = color;
            }
        }

        param = applet.getParameter( "BackgroundColor" );
        if ( param != null ) {
            color = Color.getColor( param );
            if ( color == null ) {
                color = getColor( param );
            }
            if ( color != null ) {
                background_color = color;
            }
        }

        param = applet.getParameter( "ChartBackgroundColor" );
        if ( param != null ) {
            color = Color.getColor( param );
            if ( color == null ) {
                color = getColor( param );
            }
            if ( color != null ) {
                chart_background_color = color;
            }
        }

        param = applet.getParameter( "ChartBackgroundImage" );
        if ( ( param != null ) &&
             ( ! param.equals( "" ) ) ) {
            chart_background_image = param;
        }

        try {
            vert_spacing = Integer.parseInt( applet.getParameter( "VertSpacing" ) );
        }
        catch ( NumberFormatException nfe ) {
        }

        try {
            horiz_spacing = Integer.parseInt( applet.getParameter( "HorizSpacing" ) );
        }
        catch ( NumberFormatException nfe ) {
        }

        try {
            max_levels = Integer.parseInt( applet.getParameter( "MaxLevels" ) );
        }
        catch ( NumberFormatException nfe ) {
        }

        param = applet.getParameter( "NodeFontFace" );
        if ( ( param != null ) &&
             ( ! param.equals( "" ) ) ) {
            node_font_face = param;
        }

        try {
            node_font_size = Integer.parseInt( applet.getParameter( "NodeFontSize" ) );
        }
        catch ( NumberFormatException nfe ) {
        }

        param = applet.getParameter( "NodeFontStyle" );
        if ( param != null ) {
            if ( param.equalsIgnoreCase( "Bold" ) ) {
                node_font_style = Font.BOLD;
            }
            else if ( param.equalsIgnoreCase( "Italic" ) ) {
                node_font_style = Font.ITALIC;
            }
            else if ( param.equalsIgnoreCase( "BoldItalic" ) ) {
                node_font_style = Font.BOLD | Font.ITALIC;
            }
            else {
                node_font_style = Font.PLAIN;
            }
        }

        param = applet.getParameter( "TitleFontFace" );
        if ( ( param != null ) &&
             ( ! param.equals( "" ) ) ) {
            title_font_face = param;
        }

        try {
            title_font_size = Integer.parseInt( applet.getParameter( "TitleFontSize" ) );
        }
        catch ( NumberFormatException nfe ) {
        }

        param = applet.getParameter( "TitleFontStyle" );
        if ( param != null ) {
            if ( param.equalsIgnoreCase( "Bold" ) ) {
                title_font_style = Font.BOLD;
            }
            else if ( param.equalsIgnoreCase( "Italic" ) ) {
                title_font_style = Font.ITALIC;
            }
            else if ( param.equalsIgnoreCase( "BoldItalic" ) ) {
                title_font_style = Font.BOLD | Font.ITALIC;
            }
            else {
                title_font_style = Font.PLAIN;
            }
        }

        param = applet.getParameter( "NodeTextColor" );
        if ( param != null ) {
            color = Color.getColor( param );
            if ( color == null ) {
                color = getColor( param );
            }
            if ( color != null ) {
                node_text_color = color;
            }
        }

        param = applet.getParameter( "TitleTextColor" );
        if ( param != null ) {
            color = Color.getColor( param );
            if ( color == null ) {
                color = getColor( param );
            }
            if ( color != null ) {
                title_text_color = color;
            }
        }

        param = applet.getParameter( "LinkColor" );
        if ( param != null ) {
            color = Color.getColor( param );
            if ( color == null ) {
                color = getColor( param );
            }
            if ( color != null ) {
                link_color = color;
            }
        }

        param = applet.getParameter( "NodeBorderColor" );
        if ( param != null ) {
            color = Color.getColor( param );
            if ( color == null ) {
                color = getColor( param );
            }
            if ( color != null ) {
                node_border_color = color;
            }
        }

        param = applet.getParameter( "NodeBackgroundColor" );
        if ( param != null ) {
            color = Color.getColor( param );
            if ( color == null ) {
                color = getColor( param );
            }
            if ( color != null ) {
                node_background_color = color;
            }
        }

        param = applet.getParameter( "NodeArmColor" );
        if ( param != null ) {
            color = Color.getColor( param );
            if ( color == null ) {
                color = getColor( param );
            }
            if ( color != null ) {
                node_arm_color = color;
            }
        }

        param = applet.getParameter( "NodeHiliteColor" );
        if ( param != null ) {
            color = Color.getColor( param );
            if ( color == null ) {
                color = getColor( param );
            }
            if ( color != null ) {
                node_hilite_color = color;
            }
        }

        param = applet.getParameter( "NodeTextHiliteColor" );
        if ( param != null ) {
            color = Color.getColor( param );
            if ( color == null ) {
                color = getColor( param );
            }
            if ( color != null ) {
                node_text_hilite_color = color;
            }
        }

        try {
            node_border_width = Integer.parseInt( applet.getParameter( "NodeBorderWidth" ) );
        }
        catch ( NumberFormatException nfe ) {
        }

        try {
            link_thickness = Integer.parseInt( applet.getParameter( "LinkThickness" ) );
        }
        catch ( NumberFormatException nfe ) {
        }

        param = applet.getParameter( "NodeShadowColor" );
        if ( param != null ) {
            color = Color.getColor( param );
            if ( color == null ) {
                color = getColor( param );
            }
            if ( color != null ) {
                node_shadow_color = color;
            }
        }

        try {
            node_shadow_offset = Integer.parseInt( applet.getParameter( "NodeShadowOffset" ) );
        }
        catch ( NumberFormatException nfe ) {
        }

        param = applet.getParameter( "RoundNodeCorners" );
        if ( param != null ) {
            if ( param.equalsIgnoreCase( "yes" ) ||
                 param.equalsIgnoreCase( "true" ) ) {
                round_node_corners = true;
            }
            else {
                round_node_corners = false;
            }
        }

        param = applet.getParameter( "ThreeDBorders" );
        if ( param != null ) {
            if ( param.equalsIgnoreCase( "yes" ) ||
                 param.equalsIgnoreCase( "true" ) ) {
                threed_borders = true;
            }
            else {
                threed_borders = false;
            }
        }

        param = applet.getParameter( "SuppressNodeBackground" );
        if ( param != null ) {
            if ( param.equalsIgnoreCase( "yes" ) ||
                 param.equalsIgnoreCase( "true" ) ) {
                suppress_node_background = true;
            }
            else {
                suppress_node_background = false;
            }
        }

        param = applet.getParameter( "TransparentNodeBackground" );
        if ( param != null ) {
            if ( param.equalsIgnoreCase( "yes" ) ||
                 param.equalsIgnoreCase( "true" ) ) {
                transparent_node_background = true;
            }
            else {
                transparent_node_background = false;
            }
        }
    }

    public void shrink() {
        if ( ( node_font_size > 3 ) &&
             ( title_font_size > 3 ) ) {
            node_font_size -= 3;
            title_font_size -= 3;
        }
    }

    public void grow() {
        if ( ( node_font_size < 70 ) &&
             ( title_font_size < 70 ) ) {
            node_font_size += 3;
            title_font_size += 3;
        }
    }

    public void more() {
        max_levels++;
    }

    public void less() {
        if ( max_levels > 1 ) {
            max_levels--;
        }
    }
}
