/**
 * Copyright(c) 1996 DTAI, Incorporated (http://www.dtai.com)
 *
 *                        All rights reserved
 *
 * Permission to use, copy, modify and distribute this material for
 * any purpose and without fee is hereby granted, provided that the
 * above copyright notice and this permission notice appear in all
 * copies, and that the name of DTAI, Incorporated not be used in
 * advertising or publicity pertaining to this material without the
 * specific, prior written permission of an authorized representative of
 * DTAI, Incorporated.
 *
 * DTAI, INCORPORATED MAKES NO REPRESENTATIONS AND EXTENDS NO WARRANTIES,
 * EXPRESS OR IMPLIED, WITH RESPECT TO THE SOFTWARE, INCLUDING, BUT
 * NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 * FITNESS FOR ANY PARTICULAR PURPOSE, AND THE WARRANTY AGAINST
 * INFRINGEMENT OF PATENTS OR OTHER INTELLECTUAL PROPERTY RIGHTS.  THE
 * SOFTWARE IS PROVIDED "AS IS", AND IN NO EVENT SHALL DTAI, INCORPORATED OR
 * ANY OF ITS AFFILIATES BE LIABLE FOR ANY DAMAGES, INCLUDING ANY
 * LOST PROFITS OR OTHER INCIDENTAL OR CONSEQUENTIAL DAMAGES RELATING
 * TO THE SOFTWARE.
 */

import java.awt.*;
import java.util.*;
import java.net.*;
import orgdata.*;
import java.io.*;
import COM.dtai.gui.*;
import COM.dtai.util.*;

/**
 * OrgNode - an employee position node in the OrgChart
 *
 * @version	1.0b1
 * @author	DTAI, Incorporated
 *
 * $Id: OrgNode.java,v 1.20 1996/08/13 06:29:17 kadel Exp $
 *
 * $Source: /cvs/java/OrgChart/OrgNode.java,v $
 */

public class OrgNode extends Gadget {

    private OrgChart chart;
    private OrgChartGraph graph;
    private OrgChartDb db;
    private OrgChartPrefs prefs;
    private Employee employee;
    private OrgNode supervisor;
    private OrgNodeData data;


	/**
	 * DESCRIPTION
	 */
    public static final int ASSISTANT = 0;
	/**
	 * DESCRIPTION
	 */
    public static final int SUBORDINATE = 1;

    private Vector children;
    private int max_order;

    private int cur_level = 0;

    private Point location = new Point( 0, 0 );
    private Dimension size = new Dimension( 0, 0 );

    private int control_base_x;
    private int control_base_y;
    private int last_x;
    private int last_y;
    private boolean dragging = false;
    private boolean showing_hand = false;

    private Dimension space;

    private boolean over_uparrow_indicator = false;
    private boolean over_downarrow_indicator = false;
    private boolean child_indicator = false;
    private boolean parent_indicator = false;
    private boolean is_leaf_node = false;
    private boolean showing_status = false;

    private Point offset = new Point( 0, 0 );

	/**
     * constructor for brand new org node
	 *
	 * @param PARAMNAME DESCRIPTION
	 * @return DESCRIPTION
	 * @exception CLASSNAME DESCRIPTION
	 * @see CLASSNAME#METHOD
	 */
    public OrgNode ( OrgChart chart, Employee employee ) {
        super( chart.getGraph() );
        this.chart = chart;
        this.graph = chart.getGraph();
        this.db = chart.getDatabase();
        prefs = graph.getPreferences();
        children = new Vector();
        data = new OrgNodeData();
        try {
            data.id = db.getNextOrgNodeId();
        }
        catch ( Exception exp ) {
            ShowUser.error( "Database error on getNextOrgNodeId()." );
        }
        setEmployee ( employee );
        setType( OrgNode.SUBORDINATE );
        setOrder( 0 );
        graph.addNode( this );
        hide();
    }

	/**
     * constructor for node from the database
	 *
	 * @param PARAMNAME DESCRIPTION
	 * @return DESCRIPTION
	 * @exception CLASSNAME DESCRIPTION
	 * @see CLASSNAME#METHOD
	 */
    public OrgNode ( OrgChart chart, OrgNodeData data, Employee employee ) {
        super( chart.getGraph() );
        this.chart = chart;
        this.graph = chart.getGraph();
        this.db = chart.getDatabase();
        prefs = graph.getPreferences();
        this.data = data;
        setEmployee( employee );
        graph.addNode( this );
        hide();
    }

	/**
	 * DESCRIPTION
	 *
	 * @param PARAMNAME DESCRIPTION
	 * @return DESCRIPTION
	 * @exception CLASSNAME DESCRIPTION
	 * @see CLASSNAME#METHOD
	 */
    public void setEmployee( Employee employee ) {
        this.employee = employee;
        if ( employee == null ) {
            data.employee_id = 0;
        }
        else {
            data.employee_id = employee.getId();
        }
    }

	/**
	 * DESCRIPTION
	 *
	 * @param PARAMNAME DESCRIPTION
	 * @return DESCRIPTION
	 * @exception CLASSNAME DESCRIPTION
	 * @see CLASSNAME#METHOD
	 */
    public void loadData( OrgNodeData data ) {
        this.data = data;
    }

	/**
	 * DESCRIPTION
	 *
	 * @param PARAMNAME DESCRIPTION
	 * @return DESCRIPTION
	 * @exception CLASSNAME DESCRIPTION
	 * @see CLASSNAME#METHOD
	 */
    public boolean childrenLoaded() {
        return ( children != null );
    }

	/**
	 * DESCRIPTION
	 *
	 * @param PARAMNAME DESCRIPTION
	 * @return DESCRIPTION
	 * @exception CLASSNAME DESCRIPTION
	 * @see CLASSNAME#METHOD
	 */
    public void setChildren() {
        if ( children == null ) {
    		children = new Vector();
    	}
	}

	/**
	 * DESCRIPTION
	 *
	 * @param PARAMNAME DESCRIPTION
	 * @return DESCRIPTION
	 * @exception CLASSNAME DESCRIPTION
	 * @see CLASSNAME#METHOD
	 */
    public void loadChildren( int stop_level ) {
        ShowUser.cursor( Frame.WAIT_CURSOR );
        graph.freeze();

        try {
            if ( children == null ) {
                children = new Vector();
//                db.loadOrgNodesByParentId( data.id, chart );
				if ( supervisor == null ) {
					db.getOrgChartBranchByParent( data.id, chart, stop_level + 2 );
				}
				else {
					db.getOrgChartBranchByParent( supervisor.data.id,
												  chart, stop_level + 3 );
				}
            }
        }
        catch ( IOException ioe ) {
            ShowUser.error( "Database error trying to load the children of a node." );
        }
        catch ( IllegalAccessException iae ) {
            ShowUser.error( "Database access error trying to load the children of a node." );
        }
        finally {
            graph.thaw();
        }
    }

	/**
	 * DESCRIPTION
	 *
	 * @param PARAMNAME DESCRIPTION
	 * @return DESCRIPTION
	 * @exception CLASSNAME DESCRIPTION
	 * @see CLASSNAME#METHOD
	 */
    public void save() throws IOException, IllegalAccessException {
        db.saveOrgNode( data );
    }

	/**
	 * DESCRIPTION
	 *
	 * @param PARAMNAME DESCRIPTION
	 * @return DESCRIPTION
	 * @exception CLASSNAME DESCRIPTION
	 * @see CLASSNAME#METHOD
	 */
    public void load() throws IOException, IllegalAccessException {
        data = db.loadOrgNodeById( data.id );
    }

	/**
	 * DESCRIPTION
	 *
	 * @param PARAMNAME DESCRIPTION
	 * @return DESCRIPTION
	 * @exception CLASSNAME DESCRIPTION
	 * @see CLASSNAME#METHOD
	 */
    public void delete() throws IOException, IllegalAccessException {
        db.deleteOrgNode( data );
    }

	/**
	 * DESCRIPTION
	 *
	 * @param PARAMNAME DESCRIPTION
	 * @return DESCRIPTION
	 * @exception CLASSNAME DESCRIPTION
	 * @see CLASSNAME#METHOD
	 */
    public void removeSelf() throws IOException, IllegalAccessException
    {
        hide();
        for ( Enumeration e = children.elements(); e.hasMoreElements(); ) {
            ((OrgNode)e.nextElement()).removeSelf();
        }
        if ( supervisor != null ) {
            supervisor.removeNode( this );
        }
        graph.removeNode( this, true );
    }

	/**
	 * DESCRIPTION
	 *
	 * @param PARAMNAME DESCRIPTION
	 * @return DESCRIPTION
	 * @exception CLASSNAME DESCRIPTION
	 * @see CLASSNAME#METHOD
	 */
    public void setSupervisor( OrgNode supervisor ) {
        this.supervisor = supervisor;
        if ( supervisor == null ) {
            data.parent_id = 0;
        }
        else {
            data.parent_id = supervisor.getId();
        }
    }

	/**
	 * DESCRIPTION
	 *
	 * @param PARAMNAME DESCRIPTION
	 * @return DESCRIPTION
	 * @exception CLASSNAME DESCRIPTION
	 * @see CLASSNAME#METHOD
	 */
    public OrgNode getSupervisor() {
        return supervisor;
    }

	/**
	 * DESCRIPTION
	 *
	 * @param PARAMNAME DESCRIPTION
	 * @return DESCRIPTION
	 * @exception CLASSNAME DESCRIPTION
	 * @see CLASSNAME#METHOD
	 */
    public final Employee getEmployee() {
        return ( employee );
    }

	/**
	 * DESCRIPTION
	 *
	 * @param PARAMNAME DESCRIPTION
	 * @return DESCRIPTION
	 * @exception CLASSNAME DESCRIPTION
	 * @see CLASSNAME#METHOD
	 */
    public void setRole( String role ) {
        data.role = role;
    }

	/**
	 * DESCRIPTION
	 *
	 * @param PARAMNAME DESCRIPTION
	 * @return DESCRIPTION
	 * @exception CLASSNAME DESCRIPTION
	 * @see CLASSNAME#METHOD
	 */
    public final String getRole() {
        return data.role;
    }

	/**
	 * DESCRIPTION
	 *
	 * @param PARAMNAME DESCRIPTION
	 * @return DESCRIPTION
	 * @exception CLASSNAME DESCRIPTION
	 * @see CLASSNAME#METHOD
	 */
    public final int getId() {
        return data.id;
    }

	/**
	 * DESCRIPTION
	 *
	 * @param PARAMNAME DESCRIPTION
	 * @return DESCRIPTION
	 * @exception CLASSNAME DESCRIPTION
	 * @see CLASSNAME#METHOD
	 */
    public final int getType() {
        return data.type;
    }

	/**
	 * DESCRIPTION
	 *
	 * @param PARAMNAME DESCRIPTION
	 * @return DESCRIPTION
	 * @exception CLASSNAME DESCRIPTION
	 * @see CLASSNAME#METHOD
	 */
    public void setType( int type ) {
        data.type = type;
    }

	/**
	 * DESCRIPTION
	 *
	 * @param PARAMNAME DESCRIPTION
	 * @return DESCRIPTION
	 * @exception CLASSNAME DESCRIPTION
	 * @see CLASSNAME#METHOD
	 */
    public String getSectionTitle() {
        return data.section_title;
    }

	/**
	 * DESCRIPTION
	 *
	 * @param PARAMNAME DESCRIPTION
	 * @return DESCRIPTION
	 * @exception CLASSNAME DESCRIPTION
	 * @see CLASSNAME#METHOD
	 */
    public void setSectionTitle( String section_title ) {
        data.section_title = section_title;
    }

	/**
	 * DESCRIPTION
	 *
	 * @param PARAMNAME DESCRIPTION
	 * @return DESCRIPTION
	 * @exception CLASSNAME DESCRIPTION
	 * @see CLASSNAME#METHOD
	 */
    public URL getSectionURL() {
        URL url;
        try {
            url = new URL( data.section_url );
            if ( url.toString().equals( "http:/" ) ) {
                url = null;
            }
        }
        catch ( MalformedURLException mue ) {
            url = null;
        }
        return url;
    }

	/**
	 * DESCRIPTION
	 *
	 * @param PARAMNAME DESCRIPTION
	 * @return DESCRIPTION
	 * @exception CLASSNAME DESCRIPTION
	 * @see CLASSNAME#METHOD
	 */
    public void setSectionURL( URL section_url ) {
        if ( section_url == null ) {
            data.section_url = "";
        }
        else {
            data.section_url = section_url.toString();
        }
    }

	/**
	 * DESCRIPTION
	 *
	 * @param PARAMNAME DESCRIPTION
	 * @return DESCRIPTION
	 * @exception CLASSNAME DESCRIPTION
	 * @see CLASSNAME#METHOD
	 */
    public final int getParentId() {
        return data.parent_id;
    }

	/**
	 * DESCRIPTION
	 *
	 * @param PARAMNAME DESCRIPTION
	 * @return DESCRIPTION
	 * @exception CLASSNAME DESCRIPTION
	 * @see CLASSNAME#METHOD
	 */
    public final int getOrder() {
        return data.order;
    }

	/**
	 * DESCRIPTION
	 *
	 * @param PARAMNAME DESCRIPTION
	 * @return DESCRIPTION
	 * @exception CLASSNAME DESCRIPTION
	 * @see CLASSNAME#METHOD
	 */
    public void setOrder( int order ) {
        data.order = order;
    }

	/**
	 * DESCRIPTION
	 *
	 * @param PARAMNAME DESCRIPTION
	 * @return DESCRIPTION
	 * @exception CLASSNAME DESCRIPTION
	 * @see CLASSNAME#METHOD
	 */
    public void addNode( OrgNode node ) {
        if ( children == null ) {
            children = new Vector();
        }
        node.setSupervisor( this );
        children.removeElement( node );
        if ( node.getOrder() == 0 ) {
            if ( children.size() > 0 ) {
                node.setOrder( ((OrgNode)children.lastElement()).getOrder() + 1 );
            }
            else {
                node.setOrder( 1 );
            }
            children.addElement( node );
            max_order++;
            node.data.order = max_order;
        }
        else {
            max_order = Math.max( max_order, node.getOrder() );
            int idx = 0;
            for ( Enumeration e = children.elements(); e.hasMoreElements(); ) {
                OrgNode sibling = (OrgNode)e.nextElement();
                if ( sibling.getOrder() > node.getOrder() ) {
                    children.insertElementAt( node, idx );
                    return;
                }
                idx++;
            }
            children.addElement( node );
        }
    }

	/**
	 * DESCRIPTION
	 *
	 * @param PARAMNAME DESCRIPTION
	 * @return DESCRIPTION
	 * @exception CLASSNAME DESCRIPTION
	 * @see CLASSNAME#METHOD
	 */
    public void removeNode( OrgNode node ) {
        children.removeElement( node );
    }

	/**
	 * DESCRIPTION
	 *
	 * @param PARAMNAME DESCRIPTION
	 * @return DESCRIPTION
	 * @exception CLASSNAME DESCRIPTION
	 * @see CLASSNAME#METHOD
	 */
    public OrgChartGraph getGraph() {
        return graph;
    }

	/**
	 * DESCRIPTION
	 *
	 * @param PARAMNAME DESCRIPTION
	 * @return DESCRIPTION
	 * @exception CLASSNAME DESCRIPTION
	 * @see CLASSNAME#METHOD
	 */
    public Graphics getGraphics() {
        return graph.getPermGraphics();
    }

	/**
	 * DESCRIPTION
	 *
	 * @param PARAMNAME DESCRIPTION
	 * @return DESCRIPTION
	 * @exception CLASSNAME DESCRIPTION
	 * @see CLASSNAME#METHOD
	 */
    public void draw( Graphics g ) {
        if ( ! isHidden() ) {
            graph.drawNode( g, this, location, size(),
                            isHilited(), isDisabled(), isArmed() );
        }
    }

	/**
	 * DESCRIPTION
	 *
	 * @param PARAMNAME DESCRIPTION
	 * @return DESCRIPTION
	 * @exception CLASSNAME DESCRIPTION
	 * @see CLASSNAME#METHOD
	 */
    public void refresh() {
        size = preferredSize();
        graph.repaint();
    }

	/**
	 * DESCRIPTION
	 *
	 * @param PARAMNAME DESCRIPTION
	 * @return DESCRIPTION
	 * @exception CLASSNAME DESCRIPTION
	 * @see CLASSNAME#METHOD
	 */
    public void calcExtents( Point min, Point max, int stop_level ) {
        stop_level--;
        if ( ( size.width == 0 ) && ( size.height == 0 ) ) {
            size = preferredSize();
        }

        min.x = Math.min( min.x, location.x );
        min.y = Math.min( min.y, location.y );
        max.x = Math.max( max.x, ( location.x + size.width ) );
        max.y = Math.max( max.y, ( location.y + size.height ) );
        if ( stop_level > 0 ) {
            for ( Enumeration e = children.elements(); e.hasMoreElements(); ) {
                ((OrgNode)e.nextElement()).calcExtents( min, max, stop_level );
            }
        }
    }

	/**
	 * DESCRIPTION
	 *
	 * @param PARAMNAME DESCRIPTION
	 * @return DESCRIPTION
	 * @exception CLASSNAME DESCRIPTION
	 * @see CLASSNAME#METHOD
	 */
    public void calcDefaultExtents( Point min, Point max, int stop_level ) {
        stop_level--;
        if ( ( size.width == 0 ) && ( size.height == 0 ) ) {
            size = preferredSize();
        }
        min.x = Math.min( min.x, location.x - offset.x );
        min.y = Math.min( min.y, location.y - offset.y );
        max.x = Math.max( max.x, ( location.x + size.width - offset.x ) );
        max.y = Math.max( max.y, ( location.y + size.height - offset.y ) );
        if ( stop_level > 0 ) {
            for ( Enumeration e = children.elements(); e.hasMoreElements(); ) {
                ((OrgNode)e.nextElement()).calcExtents( min, max, stop_level );
            }
        }
    }

	/**
	 * DESCRIPTION
	 *
	 * @param PARAMNAME DESCRIPTION
	 * @return DESCRIPTION
	 * @exception CLASSNAME DESCRIPTION
	 * @see CLASSNAME#METHOD
	 */
    public Point getInitMinExtent() {
        return new Point( location.x, location.y );
    }

	/**
	 * DESCRIPTION
	 *
	 * @param PARAMNAME DESCRIPTION
	 * @return DESCRIPTION
	 * @exception CLASSNAME DESCRIPTION
	 * @see CLASSNAME#METHOD
	 */
    public Point getInitMaxExtent() {
        return new Point( location.x, location.y );
    }

	/**
	 * DESCRIPTION
	 *
	 * @param PARAMNAME DESCRIPTION
	 * @return DESCRIPTION
	 * @exception CLASSNAME DESCRIPTION
	 * @see CLASSNAME#METHOD
	 */
    public Point getInitDefaultMinExtent() {
        return new Point( location.x - offset.x, location.y - offset.y );
    }

	/**
	 * DESCRIPTION
	 *
	 * @param PARAMNAME DESCRIPTION
	 * @return DESCRIPTION
	 * @exception CLASSNAME DESCRIPTION
	 * @see CLASSNAME#METHOD
	 */
    public Point getInitDefaultMaxExtent() {
        return new Point( location.x - offset.x, location.y - offset.y );
    }

	/**
	 * DESCRIPTION
	 *
	 * @param PARAMNAME DESCRIPTION
	 * @return DESCRIPTION
	 * @exception CLASSNAME DESCRIPTION
	 * @see CLASSNAME#METHOD
	 */
    public int getCurrentLevel() {
        if ( isHidden() ) {
            return 0;
        }
        else {
            return cur_level;
        }
    }

	/**
	 * DESCRIPTION
	 *
	 * @param PARAMNAME DESCRIPTION
	 * @return DESCRIPTION
	 * @exception CLASSNAME DESCRIPTION
	 * @see CLASSNAME#METHOD
	 */
    public void setUniformSizes( int stop_level ) {
        stop_level--;
        if ( stop_level == 0 ) {
            return;
        }

        int max_subord_width = 0;
        int max_subord_height = 0;
        int max_assist_width = 0;
        int max_assist_height = 0;

        for ( Enumeration e = children.elements(); e.hasMoreElements(); ) {
            OrgNode child = (OrgNode)e.nextElement();
            child.setUniformSizes( stop_level );
            Dimension child_size = child.preferredSize();
            if ( child.getType() == OrgNode.SUBORDINATE ) {
                max_subord_width = Math.max( max_subord_width, child_size.width );
                max_subord_height = Math.max( max_subord_height, child_size.height );
            }
            else {
                max_assist_width = Math.max( max_assist_width, child_size.width );
                max_assist_height = Math.max( max_assist_height, child_size.height );
            }
        }

        for ( Enumeration e = children.elements(); e.hasMoreElements(); ) {
            OrgNode child = (OrgNode)e.nextElement();
            child.setUniformSizes( stop_level );
            Dimension child_size = child.preferredSize();
            if ( child.getType() == OrgNode.SUBORDINATE ) {
                child.resize( max_subord_width, max_subord_height );
            }
            else {
                child.resize( max_assist_width, max_assist_height );
            }
        }
    }

	/**
	 * DESCRIPTION
	 *
	 * @param PARAMNAME DESCRIPTION
	 * @return DESCRIPTION
	 * @exception CLASSNAME DESCRIPTION
	 * @see CLASSNAME#METHOD
	 */
    public void setLocations( Point upper_left, int stop_level ) {
        stop_level--;
        cur_level = prefs.max_levels - stop_level;

        if ( graph.getTopNode() != this ) {
            upper_left = new Point( upper_left.x + offset.x, upper_left.y + offset.y );
        }

        if ( ( stop_level == 0 ) ||
             ( children.size() == 0 ) ) {
            move( upper_left.x, upper_left.y );
        }
        else {

            int min_subnode_x = Integer.MAX_VALUE;
            int max_subnode_x = Integer.MIN_VALUE;
            int max_right_width = 0;
            int max_left_width = 0;

            int next_right_assistant_y = upper_left.y + size.height + prefs.vert_spacing;
            int next_left_assistant_y = next_right_assistant_y;
            int subordinate_y = next_right_assistant_y;
            int next_subordinate_x = upper_left.x;

            int idx = 0;
            for ( Enumeration e = children.elements(); e.hasMoreElements(); ) {
                OrgNode child = (OrgNode)e.nextElement();
                if ( child.getType() == OrgNode.SUBORDINATE ) {
                    child.setLocations( new Point( next_subordinate_x, 0 ), stop_level );
                    Point min = child.getInitDefaultMinExtent();
                    Point max = child.getInitDefaultMaxExtent();
                    child.calcDefaultExtents( min, max, stop_level );
                    next_subordinate_x = max.x + prefs.horiz_spacing ;//- child.offset.x;
                    Point child_loc = child.location();
                    Dimension child_size = child.size();
                    min_subnode_x = Math.min( min_subnode_x, child_loc.x );
                    max_subnode_x = Math.max( max_subnode_x, ( child_loc.x + child_size.width ) );
                }
                else {
                    if ( ( idx % 2 ) == 0 ) {
                        child.setLocations( new Point( 0, next_right_assistant_y ), stop_level );
                        Point min = child.getInitDefaultMinExtent();
                        Point max = child.getInitDefaultMaxExtent();
                        child.calcDefaultExtents( min, max, stop_level );
                        next_right_assistant_y = max.y + prefs.vert_spacing;
                        max_right_width = Math.max( max_right_width, ( max.x - min.x ) );
                    }
                    else {
                        child.setLocations( new Point( 0, next_left_assistant_y ), stop_level );
                        Point min = child.getInitDefaultMinExtent();
                        Point max = child.getInitDefaultMaxExtent();
                        child.calcDefaultExtents( min, max, stop_level );
                        next_left_assistant_y = max.y + prefs.vert_spacing;
                        max_left_width = Math.max( max_left_width, ( max.x - min.x ) );
                    }
                    Point min = child.getInitMinExtent();
                    Point max = child.getInitMaxExtent();
                    child.calcExtents( min, max, stop_level );
                    subordinate_y = Math.max( subordinate_y, max.y + prefs.vert_spacing );
                    idx++;
                }
            }
            int center_x = upper_left.x + ( size.width / 2 );

            if ( ( min_subnode_x != Integer.MAX_VALUE ) &&
                 ( ( size.width < ( max_subnode_x - min_subnode_x ) ) ||
                   ( min_subnode_x > upper_left.x ) ) ) {
                center_x = min_subnode_x + ( ( max_subnode_x - min_subnode_x ) / 2 );
            }
            next_subordinate_x = upper_left.x;
            if ( ( max_left_width + prefs.horiz_spacing ) > ( center_x - upper_left.x ) ) {
                center_x = upper_left.x + ( max_left_width + prefs.horiz_spacing );
                //next_subordinate_x = center_x - ( size.width / 2 );
                if ( min_subnode_x != Integer.MAX_VALUE ) {
                    next_subordinate_x = center_x - ( ( max_subnode_x - min_subnode_x ) / 2 );
                }
            }
            else {
                if ( ( min_subnode_x != Integer.MAX_VALUE ) &&
                     ( size.width > ( max_subnode_x - min_subnode_x ) ) ) {
                    next_subordinate_x = center_x - ( ( max_subnode_x - min_subnode_x ) / 2 );
                }
            }
            int right_assistant_x = center_x + prefs.horiz_spacing;
            int num_subs = 0;
            OrgNode last_sub = null;

            idx = 0;
            for ( Enumeration e = children.elements(); e.hasMoreElements(); ) {
                OrgNode child = (OrgNode)e.nextElement();
                if ( child.getType() == OrgNode.SUBORDINATE ) {
                    num_subs++;
                    last_sub = child;
                    child.setLocations( new Point( next_subordinate_x, subordinate_y ), stop_level );
                    Point min = child.getInitDefaultMinExtent();
                    Point max = child.getInitDefaultMaxExtent();
                    child.calcDefaultExtents( min, max, stop_level );
                    next_subordinate_x = max.x + prefs.horiz_spacing ;//- child.offset.x;
                }
                else {
                    if ( ( idx % 2 ) == 0 ) {
                        child.setLocations( new Point( right_assistant_x,
                                                       child.location().y - child.offset.y ), stop_level );
                    }
                    else {
                        Point min = child.getInitDefaultMinExtent();
                        Point max = child.getInitDefaultMaxExtent();
                        child.calcDefaultExtents( min, max, stop_level );
                        child.setLocations(
                            new Point( center_x - prefs.horiz_spacing - ( max.x - min.x ),
                                       child.location().y - child.offset.y ), stop_level );
                    }
                    idx++;
                }
            }

            if ( children.size() == 1 ) {
                OrgNode child = (OrgNode)children.firstElement();
                if ( child.data.type == SUBORDINATE ) {
                //    center_x -= child.offset.x;
                    center_x = child.location.x + ( child.size.width / 2 ) - child.offset.x;
                }
                else {
                    center_x = child.location.x - prefs.horiz_spacing - child.offset.x;
                }
            }
            else if ( num_subs == 1 ) {
                center_x = last_sub.location.x + ( last_sub.size.width / 2 ) - last_sub.offset.x;
            }

            move( ( center_x - ( size.width / 2 ) ),
                  upper_left.y );
        }
    }

	/**
	 * DESCRIPTION
	 *
	 * @param PARAMNAME DESCRIPTION
	 * @return DESCRIPTION
	 * @exception CLASSNAME DESCRIPTION
	 * @see CLASSNAME#METHOD
	 */
    public Dimension calcSpace( int stop_level ) {
        setLocations( new Point( 0, 0 ), stop_level );
        Point min = getInitMinExtent();
        Point max = getInitMaxExtent();
        calcExtents( min, max, stop_level );
        Dimension space = new Dimension();
        space.width = max.x - min.x;
        space.height = max.y - min.y;
        return space;
    }

	/**
	 * DESCRIPTION
	 *
	 * @param PARAMNAME DESCRIPTION
	 * @return DESCRIPTION
	 * @exception CLASSNAME DESCRIPTION
	 * @see CLASSNAME#METHOD
	 */
    public void addLinks( GadgetLayer link_layer, int stop_level ) {
        stop_level--;

        LineGadget line;

        if ( ( stop_level > 0 ) &&
             ( children.size() > 0 ) ) {
            int center_x = location.x + ( size.width / 2 );
            int start_y = location.y + size.height;
            int max_line_y = start_y;
            int min_x = center_x;
            int max_x = center_x;
            int num_subs = 0;

            int max_y = Integer.MIN_VALUE;
            for ( Enumeration e = children.elements(); e.hasMoreElements(); ) {
                OrgNode child = (OrgNode)e.nextElement();
                if ( child.getType() == OrgNode.ASSISTANT ) {
                    child.addLinks( link_layer, stop_level );
                    Point min = child.getInitMinExtent();
                    Point max = child.getInitMaxExtent();
                    child.calcExtents( min, max, stop_level );
                    max_y = Math.max( max_y, max.y );
                    Point child_loc = child.location();
                    Dimension child_size = child.size();
                    int line_y = child_loc.y + ( child_size.height / 2 );
                    link_layer.addGadget(
                        line = new LineGadget( graph, center_x, line_y,
                                        child_loc.x, line_y ) );
                    line.setColor( prefs.link_color );
                    line.setThickness( prefs.link_thickness );
                    max_line_y = Math.max( max_line_y, line_y );
                }
            }

            int horiz_y = start_y + ( prefs.vert_spacing / 2 );
            if ( max_y != Integer.MIN_VALUE ) {
                horiz_y = max_y + ( prefs.vert_spacing / 2 );
            }

            for ( Enumeration e = children.elements(); e.hasMoreElements(); ) {
                OrgNode child = (OrgNode)e.nextElement();
                if ( child.getType() == OrgNode.SUBORDINATE ) {
                    num_subs++;
                    child.addLinks( link_layer, stop_level );
                    Point child_loc = child.location();
                    Dimension child_size = child.size();
                    int mid_x = child_loc.x + ( child_size.width / 2 );
                    link_layer.addGadget ( line = new LineGadget( graph, mid_x, horiz_y, mid_x, child_loc.y ) );
                    line.setColor( prefs.link_color );
                    line.setThickness( prefs.link_thickness );
                    min_x = Math.min( min_x, mid_x );
                    max_x = Math.max( max_x, mid_x );
                }
            }
            if ( num_subs > 0 ) {
                link_layer.addGadget( line = new LineGadget( graph, center_x, start_y, center_x, horiz_y ) );
                line.setColor( prefs.link_color );
                line.setThickness( prefs.link_thickness );
                link_layer.addGadget( line = new LineGadget( graph, min_x, horiz_y, max_x, horiz_y ) );
                line.setColor( prefs.link_color );
                line.setThickness( prefs.link_thickness );
            }
            else if ( max_line_y != start_y ) {
                link_layer.addGadget( line = new LineGadget( graph, center_x, start_y, center_x, max_line_y ) );
                line.setColor( prefs.link_color );
                line.setThickness( prefs.link_thickness );
            }
        }
    }

	/**
	 * DESCRIPTION
	 *
	 * @param PARAMNAME DESCRIPTION
	 * @return DESCRIPTION
	 * @exception CLASSNAME DESCRIPTION
	 * @see CLASSNAME#METHOD
	 */
    public Dimension getSpace() {
        return new Dimension( space.width, space.height );
    }

	/**
	 * DESCRIPTION
	 *
	 * @param PARAMNAME DESCRIPTION
	 * @return DESCRIPTION
	 * @exception CLASSNAME DESCRIPTION
	 * @see CLASSNAME#METHOD
	 */
    public Dimension preferredSize() {
        return( graph.preferredNodeSize( this ) );
    }

	/**
	 * DESCRIPTION
	 *
	 * @param PARAMNAME DESCRIPTION
	 * @return DESCRIPTION
	 * @exception CLASSNAME DESCRIPTION
	 * @see CLASSNAME#METHOD
	 */
    public final Dimension size() {
        if ( ( size.width == 0 ) && ( size.height == 0 ) ) {
            size = preferredSize();
        }
        return new Dimension( size.width, size.height );
    }

	/**
	 * DESCRIPTION
	 *
	 * @param PARAMNAME DESCRIPTION
	 * @return DESCRIPTION
	 * @exception CLASSNAME DESCRIPTION
	 * @see CLASSNAME#METHOD
	 */
    public void resize( int width, int height ) {
        size.width = width;
        size.height = height;
        offset.x = (int)( data.offset_x * width );
        offset.y = (int)( data.offset_y * height );
    }

	/**
	 * DESCRIPTION
	 *
	 * @param PARAMNAME DESCRIPTION
	 * @return DESCRIPTION
	 * @exception CLASSNAME DESCRIPTION
	 * @see CLASSNAME#METHOD
	 */
    public void resize( Dimension size ) {
        resize( size.width, size.height );
    }

	/**
	 * DESCRIPTION
	 *
	 * @param PARAMNAME DESCRIPTION
	 * @return DESCRIPTION
	 * @exception CLASSNAME DESCRIPTION
	 * @see CLASSNAME#METHOD
	 */
    public final Point location() {
        return new Point( location.x, location.y );
    }

	/**
	 * DESCRIPTION
	 *
	 * @param PARAMNAME DESCRIPTION
	 * @return DESCRIPTION
	 * @exception CLASSNAME DESCRIPTION
	 * @see CLASSNAME#METHOD
	 */
    public void move( int x, int y ) {
        location.x = x;
        location.y = y;
    }

    protected void drawOverUpArrowIndicator() {
        Graphics g = graph.getGraphics();
        Color background = graph.getBackground();
        if ( background.equals( Color.black ) ) {
            g.setColor( Color.white );
        }
        else {
            g.setColor( Color.black );
        }
        g.setXORMode( background );

        int arrow_height = prefs.vert_spacing;
        int arrow_width = 16;
        Point graph_offset = graph.getOffsets();
        int arrow_x = ( location.x + ( size.width / 2 ) - 8 ) + graph_offset.x;
        int arrow_y = ( location.y - prefs.vert_spacing ) + graph_offset.y;

        g.drawLine( arrow_x - 1, arrow_y - 1, arrow_x + arrow_width, arrow_y - 1 );
        g.drawLine( arrow_x - 1, arrow_y, arrow_x - 1, arrow_y + arrow_height - 1 );
        g.drawLine( arrow_x, arrow_y + arrow_height - 1, arrow_x + 5, arrow_y + arrow_height - 1 );
        g.drawLine( arrow_x + arrow_width - 6, arrow_y + arrow_height - 1, arrow_x + arrow_width, arrow_y + arrow_height - 1 );
        g.drawLine( arrow_x + arrow_width, arrow_y, arrow_x + arrow_width, arrow_y + arrow_height - 2 );
    }

    protected void drawOverDownArrowIndicator() {
        Graphics g = graph.getGraphics();
        Color background = graph.getBackground();
        if ( background.equals( Color.black ) ) {
            g.setColor( Color.white );
        }
        else {
            g.setColor( Color.black );
        }
        g.setXORMode( background );

        int arrow_height = prefs.vert_spacing;
        int arrow_width = 16;
        Point graph_offset = graph.getOffsets();
        int arrow_x = ( location.x + ( size.width / 2 ) - 8 ) + graph_offset.x;
        int arrow_y = ( location.y + size.height ) + graph_offset.y;

        g.drawLine( arrow_x - 1, arrow_y + arrow_height + 1, arrow_x + arrow_width, arrow_y + arrow_height + 1 );
        g.drawLine( arrow_x - 1, arrow_y + arrow_height, arrow_x - 1, arrow_y );
        g.drawLine( arrow_x, arrow_y, arrow_x + 5, arrow_y );
        g.drawLine( arrow_x + arrow_width - 6, arrow_y, arrow_x + arrow_width, arrow_y );
        g.drawLine( arrow_x + arrow_width, arrow_y + arrow_height, arrow_x + arrow_width, arrow_y + 1 );
    }

	/**
	 * DESCRIPTION
	 *
	 * @param PARAMNAME DESCRIPTION
	 * @return DESCRIPTION
	 * @exception CLASSNAME DESCRIPTION
	 * @see CLASSNAME#METHOD
	 */
    public boolean inUpArrow( int x, int y ) {
        int arrow_height = prefs.vert_spacing;
        int arrow_width = 16;
        int arrow_x = location.x + ( size.width / 2 ) - 8;
        if ( ( graph.getTopNode() == this ) &&
             ( data.parent_id > 0 ) ) {
            int arrow_y = location.y - prefs.vert_spacing;
            if ( ( x >= arrow_x ) &&
                 ( x < ( arrow_x + arrow_width ) ) &&
                 ( y >= arrow_y ) &&
                 ( y < ( arrow_y + arrow_height ) ) ) {
                return true;
            }
        }
        return false;
    }

	/**
	 * DESCRIPTION
	 *
	 * @param PARAMNAME DESCRIPTION
	 * @return DESCRIPTION
	 * @exception CLASSNAME DESCRIPTION
	 * @see CLASSNAME#METHOD
	 */
    public boolean inDownArrow( int x, int y ) {
        int arrow_height = prefs.vert_spacing;
        int arrow_width = 16;
        int arrow_x = location.x + ( size.width / 2 ) - 8;
        if ( is_leaf_node &&
             hasChildren() ) {
            int arrow_y = location.y + size.height;
            if ( ( x >= arrow_x ) &&
                 ( x < ( arrow_x + arrow_width ) ) &&
                 ( y >= arrow_y ) &&
                 ( y < ( arrow_y + arrow_height ) ) ) {
                return true;
            }
        }
        return false;
    }

	/**
	 * DESCRIPTION
	 *
	 * @param PARAMNAME DESCRIPTION
	 * @return DESCRIPTION
	 * @exception CLASSNAME DESCRIPTION
	 * @see CLASSNAME#METHOD
	 */
    public boolean inArrow( int x, int y ) {
        return ( inUpArrow( x, y ) || inDownArrow( x, y ) );
    }

	/**
	 * DESCRIPTION
	 *
	 * @param PARAMNAME DESCRIPTION
	 * @return DESCRIPTION
	 * @exception CLASSNAME DESCRIPTION
	 * @see CLASSNAME#METHOD
	 */
    public boolean contains( int x, int y ) {
        if ( ( x >= location.x ) &&
             ( x < ( location.x + size.width ) ) &&
             ( y >= location.y ) &&
             ( y < ( location.y + size.height ) ) ) {
            return true;
        }
        return inArrow( x, y );
    }

	/**
	 * DESCRIPTION
	 *
	 * @param PARAMNAME DESCRIPTION
	 * @return DESCRIPTION
	 * @exception CLASSNAME DESCRIPTION
	 * @see CLASSNAME#METHOD
	 */
    public void select( Event evt, int x, int y ) {
        graph.nodeSelected( this );
        hilite();
        if ( inUpArrow( x, y ) ) {
            if ( supervisor == null ) {
                try {
                    db.loadOrgNodeById( chart, data.parent_id );
                    if ( supervisor == null ) {
                        data.parent_id = 0;
                        ShowUser.error( "Supervisor not found in database." );
                    }
                }
                catch ( IllegalAccessException iae ) {
                    ShowUser.error( "Database error trying to load the supervisor." );
                }
                catch ( IOException exp ) {
                    ShowUser.error( "Database error trying to load the supervisor." );
                }
            }
            if ( supervisor != null ) {
                supervisor.becomeGraphTop();
            }
        }
        else if ( inDownArrow( x, y ) ) {
            becomeGraphTop();
        }
    }

	/**
	 * DESCRIPTION
	 *
	 * @param PARAMNAME DESCRIPTION
	 * @return DESCRIPTION
	 * @exception CLASSNAME DESCRIPTION
	 * @see CLASSNAME#METHOD
	 */
    public void deselect() {
        unhilite();
    }

	/**
	 * DESCRIPTION
	 *
	 * @param PARAMNAME DESCRIPTION
	 * @return DESCRIPTION
	 * @exception CLASSNAME DESCRIPTION
	 * @see CLASSNAME#METHOD
	 */
    public boolean isDescendent( OrgNode poss_ancestor ) {
        if ( supervisor == null ) {
            return false;
        }
        else if ( poss_ancestor == supervisor ) {
            return true;
        }
        else {
            return supervisor.isDescendent( poss_ancestor );
        }
    }

    protected void drawDragBox( int x, int y ) {
        Graphics g = graph.getGraphics();
        Color background = graph.getBackground();
        if ( background.equals( Color.black ) ) {
            g.setColor( Color.white );
        }
        else {
            g.setColor( Color.black );
        }
        g.setXORMode( background );

        Point graph_offset = graph.getOffsets();
        g.drawRect( ( x + graph_offset.x ), ( y + graph_offset.y ),
                    ( size.width - 1 ), ( size.height - 1 ) );
        g.setPaintMode();
    }

	/**
	 * DESCRIPTION
	 *
	 * @param PARAMNAME DESCRIPTION
	 * @return DESCRIPTION
	 * @exception CLASSNAME DESCRIPTION
	 * @see CLASSNAME#METHOD
	 */
    public final void drawLastDragBox() {
        drawDragBox( last_x, last_y );
    }

    protected void beginDrag( Event evt, int x, int y ) {
        if ( evt.controlDown() &&
             ( graph.getTopNode() != this ) ) {
            dragging = true;
            control_base_x = x;
            control_base_y = y;
            last_x = location.x;
            last_y = location.y;
            drawDragBox( last_x, last_y );
        }
    }

    protected void drag( Event evt, int x, int y ) {
        if ( dragging &&
             ( graph.getTopNode() != this ) ) {
            drawLastDragBox();
            last_x = location.x + ( x - control_base_x );
            last_y = location.y + ( y - control_base_y );
            if ( evt.shiftDown() ) {
                if ( Math.abs( x - control_base_x ) > Math.abs( y - control_base_y ) ) {
                    last_y = location.y;
                }
                else {
                    last_x = location.x;
                }
            }
            drawDragBox( last_x, last_y );
        }
    }

    protected void drop( Event evt, int x, int y ) {
        if ( dragging &&
             ( graph.getTopNode() != this ) ) {
            dragging = false;
            drawLastDragBox();
            if ( evt.shiftDown() ) {
                if ( Math.abs( x - control_base_x ) > Math.abs( y - control_base_y ) ) {
                    offset.x += x - control_base_x;
                }
                else {
                    offset.y += y - control_base_y;
                }
            }
            else {
                offset.x += x - control_base_x;
                offset.y += y - control_base_y;
            }
            data.offset_x = (float)offset.x / (float)size.width;
            data.offset_y = (float)offset.y / (float)size.height;
            graph.repaint();
            try {
                save();
            }
            catch ( IllegalAccessException iae ) {
                chart.lostWritability();
            }
            catch ( IOException exp ) {
                ShowUser.error( "Database error trying to save the new location." );
            }
        }
    }

	/**
	 * DESCRIPTION
	 *
	 * @param PARAMNAME DESCRIPTION
	 * @return DESCRIPTION
	 * @exception CLASSNAME DESCRIPTION
	 * @see CLASSNAME#METHOD
	 */
    public void reparent( OrgNode newparent ) {
        if ( ( newparent != this ) &&
             ( ! newparent.isDescendent( this ) ) ) {
            if ( supervisor != null ) {
                supervisor.removeNode( this );
            }

            if ( newparent != null ) {
                supervisor = newparent;
                data.order = 0;
                offset.x = 0;
                offset.y = 0;
                data.offset_x = (float)0.0;
                data.offset_y = (float)0.0;
                supervisor.addNode( this );
            }
            else {
                supervisor = null;
            }
            graph.repaint();
            try {
                save();
            }
            catch ( IllegalAccessException iae ) {
                chart.lostWritability();
            }
            catch ( IOException exp ) {
                ShowUser.error( "Database error trying to reparent the node." );
            }
        }
    }

	/**
	 * DESCRIPTION
	 *
	 * @param PARAMNAME DESCRIPTION
	 * @return DESCRIPTION
	 * @exception CLASSNAME DESCRIPTION
	 * @see CLASSNAME#METHOD
	 */
    public void giveChildrenToParent() {
        if ( supervisor != null ) {
            graph.freeze();
            while ( children.size() > 0 ) {
                ((OrgNode)children.firstElement()).reparent( supervisor );
            }
            graph.thaw();
        }
    }

    protected boolean dropOn( Gadget over ) {
        if ( dragging &&
             ( over != this ) &&
             ( over instanceof OrgNode ) ) {
            reparent( (OrgNode)over );
            dragging = false;
            return true;
        }
        else {
            return false;
        }
    }

    protected void clearDrag() {
        if ( dragging ) {
            drawLastDragBox();
        }
    }
/*
    protected void tempHilite() {
        Graphics g = graph.getGraphics();
        Color background = graph.getBackground();
        if ( background.equals( Color.black ) ) {
            g.setColor( Color.white );
        }
        else {
            g.setColor( Color.black );
        }
        g.setXORMode( background );

        Point graph_offset = graph.getOffsets();
        int x = location.x + graph_offset.x;
        int y = location.y + graph_offset.y;
        g.drawRect( ( x - 2 ), ( y - 2 ),
                    ( size.width + 3 ), ( size.height + 3 ) );
        g.drawRect( ( x - 1 ), ( y - 1 ),
                    ( size.width + 1 ), ( size.height + 1 ) );
        g.drawRect( ( x + 1 ), ( y + 1 ),
                    ( size.width - 3 ), ( size.height - 3 ) );
        g.drawRect( ( x + 2 ), ( y + 2 ),
                    ( size.width - 5 ), ( size.height - 5 ) );
        g.setPaintMode();
    }
*/
    protected void isUnder( Gadget under ) {
        if ( ( under != this ) &&
             ( ( ! ( under instanceof OrgNode ) ) ||
               ( ! isDescendent( (OrgNode)under ) ) ) ) {
            if ( ((OrgNode)under).dragging ) {
                graph.armNode( graph.getGraphics(), this, location, size(),
                               isHilited() );
            }
        }
    }

    protected void notUnder( Gadget under ) {
        if ( ( under != this ) &&
             ( ( ! ( under instanceof OrgNode ) ) ||
               ( ! isDescendent( (OrgNode)under ) ) ) ) {
            if ( ((OrgNode)under).dragging ) {
                graph.armNode( graph.getGraphics(), this, location, size(),
                               isHilited() );
            }
        }
    }

	/**
	 * DESCRIPTION
	 *
	 * @param PARAMNAME DESCRIPTION
	 * @return DESCRIPTION
	 * @exception CLASSNAME DESCRIPTION
	 * @see CLASSNAME#METHOD
	 */
    public boolean hasChildren() {
        if ( children == null ) {
            loadChildren( 1 );
        }
        return ( children.size() > 0 );
    }

    protected void drawChildIndicator() {
        Graphics g = graph.getGraphics();
        Color background = graph.getBackground();
        if ( background.equals( Color.black ) ) {
            g.setColor( Color.white );
        }
        else {
            g.setColor( Color.black );
        }
        g.setXORMode( background );

        Point graph_offset = graph.getOffsets();
        int x = location.x + graph_offset.x;
        int y = location.y + graph_offset.y;
        y += size.height;
        x += size.width / 2;
        Polygon poly = new Polygon();
        poly.addPoint( x - 2, y );
        poly.addPoint( x - 2, y + ( prefs.vert_spacing / 2 ) );
        poly.addPoint( x - 8, y + ( prefs.vert_spacing / 2 ) );
        poly.addPoint( x, y + prefs.vert_spacing );
        poly.addPoint( x + 8, y + ( prefs.vert_spacing / 2 ) );
        poly.addPoint( x + 2, y + ( prefs.vert_spacing / 2 ) );
        poly.addPoint( x + 2, y );
        poly.addPoint( x - 2, y );
        g.fillPolygon( poly );
        g.setPaintMode();
    }

    protected void drawParentIndicator() {
        Graphics g = graph.getGraphics();
        Color background = graph.getBackground();
        if ( background.equals( Color.black ) ) {
            g.setColor( Color.white );
        }
        else {
            g.setColor( Color.black );
        }
        g.setXORMode( background );

        Point graph_offset = graph.getOffsets();
        int x = location.x + graph_offset.x;
        int y = location.y + graph_offset.y;
        x += size.width / 2;
        Polygon poly = new Polygon();
        poly.addPoint( x - 2, y );
        poly.addPoint( x - 2, y - ( prefs.vert_spacing / 2 ) );
        poly.addPoint( x - 8, y - ( prefs.vert_spacing / 2 ) );
        poly.addPoint( x, y - prefs.vert_spacing );
        poly.addPoint( x + 8, y - ( prefs.vert_spacing / 2 ) );
        poly.addPoint( x + 2, y - ( prefs.vert_spacing / 2 ) );
        poly.addPoint( x + 2, y );
        poly.addPoint( x - 2, y );
        g.fillPolygon( poly );
        g.setPaintMode();
    }

	/**
	 * DESCRIPTION
	 *
	 * @param PARAMNAME DESCRIPTION
	 * @return DESCRIPTION
	 * @exception CLASSNAME DESCRIPTION
	 * @see CLASSNAME#METHOD
	 */
    protected void mouseMove( Event evt, int x, int y ) {
        if ( inArrow( x, y ) ) {
            if ( ! showing_hand ) {
                ShowUser.cursor( Frame.HAND_CURSOR );
                showing_hand = true;
            }
            if ( inUpArrow( x, y ) ) {
                if ( ! over_uparrow_indicator ) {
                    drawOverUpArrowIndicator();
                    over_uparrow_indicator = true;
                    ShowUser.status( "Click up-arrow to show the supervisor" );
                    showing_status = true;
                }
            }
            else {
                if ( ! over_downarrow_indicator ) {
                    drawOverDownArrowIndicator();
                    over_downarrow_indicator = true;
                    ShowUser.status( "Click down-arrow to show the subordinates" );
                    showing_status = true;
                }
            }
        }
        else {
            if ( ( data.section_title != null ) &&
                 ( ! data.section_title.equals( "" ) ) ) {
                ShowUser.status( data.section_title );
                showing_status = true;
            }
            else if ( showing_status ) {
                ShowUser.status( "" );
                showing_status = false;
            }
            if ( showing_hand ) {
                ShowUser.cursor( Frame.DEFAULT_CURSOR );
                showing_hand = false;

                if ( over_uparrow_indicator ) {
                    drawOverUpArrowIndicator();
                    over_uparrow_indicator = false;
                }
                if ( over_downarrow_indicator ) {
                    drawOverDownArrowIndicator();
                    over_downarrow_indicator = false;
                }
            }
        }
    }

	/**
	 * DESCRIPTION
	 *
	 * @param PARAMNAME DESCRIPTION
	 * @return DESCRIPTION
	 * @exception CLASSNAME DESCRIPTION
	 * @see CLASSNAME#METHOD
	 */
    protected void mouseEnter() {
        if ( ( data.section_title != null ) &&
             ( ! data.section_title.equals( "" ) ) ) {
            ShowUser.status( data.section_title );
            showing_status = true;
        }
        if ( hasChildren() && is_leaf_node ) {
            drawChildIndicator();
            child_indicator = true;
        }
        if ( ( data.parent_id > 0 ) &&
             ( graph.getTopNode() == this ) ) {
            drawParentIndicator();
            parent_indicator = true;
        }
    }

	/**
	 * DESCRIPTION
	 *
	 * @param PARAMNAME DESCRIPTION
	 * @return DESCRIPTION
	 * @exception CLASSNAME DESCRIPTION
	 * @see CLASSNAME#METHOD
	 */
    protected void mouseExit() {
        if ( showing_status ) {
            ShowUser.status( "" );
            showing_status = false;
        }
        if ( child_indicator ) {
            drawChildIndicator();
            child_indicator = false;
        }
        if ( parent_indicator ) {
            drawParentIndicator();
            parent_indicator = false;
        }
        if ( showing_hand ) {
            ShowUser.cursor( Frame.DEFAULT_CURSOR );
            showing_hand = false;
            if ( over_uparrow_indicator ) {
                drawOverUpArrowIndicator();
                over_uparrow_indicator = false;
            }
            if ( over_downarrow_indicator ) {
                drawOverDownArrowIndicator();
                over_downarrow_indicator = false;
            }
        }
    }

	/**
	 * DESCRIPTION
	 *
	 * @param PARAMNAME DESCRIPTION
	 * @return DESCRIPTION
	 * @exception CLASSNAME DESCRIPTION
	 * @see CLASSNAME#METHOD
	 */
    public void becomeGraphTop() {
        if ( graph.getTopNode() != null ) {
            graph.getTopNode().hide();
        }
        graph.setTopNode( this );
        show();
    }

	/**
	 * DESCRIPTION
	 *
	 * @param PARAMNAME DESCRIPTION
	 * @return DESCRIPTION
	 * @exception CLASSNAME DESCRIPTION
	 * @see CLASSNAME#METHOD
	 */
    public void doubleClick() {
        chart.getNodeDialog().editNode( this );
    }

	/**
	 * DESCRIPTION
	 *
	 * @param PARAMNAME DESCRIPTION
	 * @return DESCRIPTION
	 * @exception CLASSNAME DESCRIPTION
	 * @see CLASSNAME#METHOD
	 */
    public void hide() {
        if ( children != null ) {
            for ( Enumeration e = children.elements(); e.hasMoreElements(); ) {
                ((OrgNode)e.nextElement()).hide();
            }
        }
        super.hide();
    }

	/**
	 * DESCRIPTION
	 *
	 * @param PARAMNAME DESCRIPTION
	 * @return DESCRIPTION
	 * @exception CLASSNAME DESCRIPTION
	 * @see CLASSNAME#METHOD
	 */
    public void show() {
        if ( children != null ) {
            for ( Enumeration e = children.elements(); e.hasMoreElements(); ) {
                ((OrgNode)e.nextElement()).show();
            }
        }
        super.show();
    }

	/**
	 * DESCRIPTION
	 *
	 * @param PARAMNAME DESCRIPTION
	 * @return DESCRIPTION
	 * @exception CLASSNAME DESCRIPTION
	 * @see CLASSNAME#METHOD
	 */
    public void show( int stop_level ) {
        stop_level--;
        if ( stop_level >= 0 ) {
            if ( children == null ) {
                loadChildren( stop_level );
            }
        }
        if ( stop_level > 0 ) {
            for ( Enumeration e = children.elements(); e.hasMoreElements(); ) {
                ((OrgNode)e.nextElement()).show( stop_level );
            }
            is_leaf_node = false;
        }
        else {
            is_leaf_node = true;
        }
        if ( child_indicator ) {
            drawChildIndicator();
            child_indicator = false;
        }
        if ( parent_indicator ) {
            drawParentIndicator();
            parent_indicator = false;
        }
        if ( over_uparrow_indicator ) {
            drawOverUpArrowIndicator();
            over_uparrow_indicator = false;
        }
        if ( over_downarrow_indicator ) {
            drawOverDownArrowIndicator();
            over_downarrow_indicator = false;
        }
        super.show();
    }

	/**
	 * DESCRIPTION
	 *
	 * @param PARAMNAME DESCRIPTION
	 * @return DESCRIPTION
	 * @exception CLASSNAME DESCRIPTION
	 * @see CLASSNAME#METHOD
	 */
    public void arm() {
        if ( ! armed ) {
            armed = true;
            if ( ! isHidden() ) {
                graph.armNode( graph.getGraphics(), this, location, size(),
                               isHilited() );
            }
        }
    }

	/**
	 * DESCRIPTION
	 *
	 * @param PARAMNAME DESCRIPTION
	 * @return DESCRIPTION
	 * @exception CLASSNAME DESCRIPTION
	 * @see CLASSNAME#METHOD
	 */
    public void disarm() {
        if ( armed ) {
            armed = false;
            if ( ! isHidden() ) {
                graph.disarmNode( graph.getGraphics(), this, location, size(),
                                  isHilited() );
            }
        }
    }
}
