/**
 * Copyright(c) 1996 DTAI, Incorporated (http://www.dtai.com)
 *
 *                        All rights reserved
 *
 * Permission to use, copy, modify and distribute this material for
 * any purpose and without fee is hereby granted, provided that the
 * above copyright notice and this permission notice appear in all
 * copies, and that the name of DTAI, Incorporated not be used in
 * advertising or publicity pertaining to this material without the
 * specific, prior written permission of an authorized representative of
 * DTAI, Incorporated.
 *
 * DTAI, INCORPORATED MAKES NO REPRESENTATIONS AND EXTENDS NO WARRANTIES,
 * EXPRESS OR IMPLIED, WITH RESPECT TO THE SOFTWARE, INCLUDING, BUT
 * NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 * FITNESS FOR ANY PARTICULAR PURPOSE, AND THE WARRANTY AGAINST
 * INFRINGEMENT OF PATENTS OR OTHER INTELLECTUAL PROPERTY RIGHTS.  THE
 * SOFTWARE IS PROVIDED "AS IS", AND IN NO EVENT SHALL DTAI, INCORPORATED OR
 * ANY OF ITS AFFILIATES BE LIABLE FOR ANY DAMAGES, INCLUDING ANY
 * LOST PROFITS OR OTHER INCIDENTAL OR CONSEQUENTIAL DAMAGES RELATING
 * TO THE SOFTWARE.
 */

import java.awt.*;
import COM.dtai.gui.*;

/**
 * TextNodeComponent - superclass for individual text components of an
 * OrgNode
 *
 * @see		COM.dtai.gui.DrawText
 * @see		OrgNode
 * @author	DTAI, Incorporated
 * @version	1.0b1
 *
 * $Id: TextNodeComponent.java,v 1.5 1996/07/27 03:27:07 kadel Exp $
 *
 * $Source: /cvs/java/OrgChart/TextNodeComponent.java,v $
 */

public abstract class TextNodeComponent extends NodeComponent {

    private DrawText draw_text = new DrawText();
    Color normal_color = Color.black;
    Color hilite_color = Color.black;

/**
 * returns the current font
 *
 * @return      the font
 * @exception CLASSNAME DESCRIPTION
 * @see CLASSNAME#METHOD
 */
    public Font getFont() {
        return draw_text.getFont();
    }

/**
 * Changes the font for the next time it is drawn
 *
 * @param       font        the new font
 * @return DESCRIPTION
 * @exception CLASSNAME DESCRIPTION
 * @see CLASSNAME#METHOD
 */
    public void setFont( Font font ) {
        draw_text.setFont( font );
    }

/**
 * returns the current color
 *
 * @return      the color
 * @exception CLASSNAME DESCRIPTION
 * @see CLASSNAME#METHOD
 */
    public Color getColor() {
        return normal_color;
    }

/**
 * Changes the text color for the next time it is drawn
 *
 * @param       color        the new color
 */
    public void setColor( Color color ) {
        normal_color = color;
    }

/**
 * Changes the text color for the next time it is drawn hilited
 *
 * @param       color        the new color
 */
    public void setHiliteColor( Color color ) {
        hilite_color = color;
    }

    protected abstract String getText( OrgNode node );

    protected Dimension size( Graphics g, OrgNode node ) {
        draw_text.setText( getText( node ) );
        return draw_text.size( g );
    }

    protected Dimension size( OrgNode node ) {
        Graphics g = node.getGraphics();
        if ( g == null ) {
            return new Dimension( 0, 0 );
        }
        else {
            return size( g, node );
        }
    }

    protected void draw( Graphics g, OrgNode node,
                         int center_x, int top_y,
                         boolean hilited, boolean disabled, boolean armed ) {
        draw_text.setText( getText( node ) );
        Dimension size = draw_text.size( g );
        FontMetrics metrics = g.getFontMetrics();
        if ( hilited ) {
            draw_text.setColor( hilite_color );
        }
        else {
            draw_text.setColor( normal_color );
        }
        draw_text.draw( g, new Point( ( center_x - ( size.width / 2 ) ), top_y ) );
    }

    protected void draw( Graphics erase_g, OrgNode node,
                         int center_x, int top_y ) {
        Color save_color = draw_text.getColor();
        draw_text.setColor( null );
        draw_text.setText( getText( node ) );
        Dimension size = draw_text.size( erase_g );
        FontMetrics metrics = erase_g.getFontMetrics();
        draw_text.draw( erase_g, new Point( ( center_x - ( size.width / 2 ) ), top_y ) );
        draw_text.setColor( save_color );
    }
}
