/**
 * Copyright(c) 1996 DTAI, Incorporated (http://www.dtai.com)
 *
 *                        All rights reserved
 *
 * Permission to use, copy, modify and distribute this material for
 * any purpose and without fee is hereby granted, provided that the
 * above copyright notice and this permission notice appear in all
 * copies, and that the name of DTAI, Incorporated not be used in
 * advertising or publicity pertaining to this material without the
 * specific, prior written permission of an authorized representative of
 * DTAI, Incorporated.
 *
 * DTAI, INCORPORATED MAKES NO REPRESENTATIONS AND EXTENDS NO WARRANTIES,
 * EXPRESS OR IMPLIED, WITH RESPECT TO THE SOFTWARE, INCLUDING, BUT
 * NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 * FITNESS FOR ANY PARTICULAR PURPOSE, AND THE WARRANTY AGAINST
 * INFRINGEMENT OF PATENTS OR OTHER INTELLECTUAL PROPERTY RIGHTS.  THE
 * SOFTWARE IS PROVIDED "AS IS", AND IN NO EVENT SHALL DTAI, INCORPORATED OR
 * ANY OF ITS AFFILIATES BE LIABLE FOR ANY DAMAGES, INCLUDING ANY
 * LOST PROFITS OR OTHER INCIDENTAL OR CONSEQUENTIAL DAMAGES RELATING
 * TO THE SOFTWARE.
 */

package orgdata;

import java.io.*;
import COM.dtai.serial.*;

/**
 * EmployeeData - holds the saved information about an employee and assists
 * in reading and writing across network connections and to/from a file.
 *
 * @see OrgNodeData
 * @version	1.0b1
 * @author	DTAI, Incorporated
 *
 * $Id: EmployeeData.java,v 1.8 1996/08/06 02:54:56 kadel Exp $
 *
 * $Source: /cvs/java/classes/orgdata/EmployeeData.java,v $
 */

public class EmployeeData implements SerialData
{
	private static final int magic = 1234;

	/**
	 * The employee's id
	 */
    public int id;

	/**
	 * The employee's first name
	 */
    public String first_name;

	/**
	 * The employee's last name
	 */
    public String last_name;

	/**
	 * The company Employee Number
	 */
    public String number;

	/**
	 * The employee's phone number
	 */
    public String phone;

	/**
	 * The employee's email address
	 */
    public String email;

	/**
	 * The employee's personal web page
	 */
    public String web_page;

	/**
	 * A URL to a picture of the employee
	 */
    public String image;

	/**
	 * DESCRIPTION
	 * 
	 * @param PARAMNAME DESCRIPTION
	 * @return DESCRIPTION
	 * @exception CLASSNAME DESCRIPTION
	 * @see CLASSNAME#METHOD
	 */
    public EmployeeData()
    {
		clear();
    }

	/**
	 * DESCRIPTION
	 * 
	 * @param PARAMNAME DESCRIPTION
	 * @return DESCRIPTION
	 * @exception CLASSNAME DESCRIPTION
	 * @see CLASSNAME#METHOD
	 */
    public EmployeeData(int id,
						String first_name,
						String last_name,
						String number,
						String phone,
						String email,
						String web_page,
						String image)
    {
        this.id = id;
        this.first_name = first_name;
        this.last_name = last_name;
        this.number = number;
        this.phone = phone;
        this.email = email;
        this.web_page = web_page;
        this.image = image;
    }

	/**
	 * DESCRIPTION
	 * 
	 * @param PARAMNAME DESCRIPTION
	 * @return DESCRIPTION
	 * @exception CLASSNAME DESCRIPTION
	 * @see CLASSNAME#METHOD
	 */
    public final void setValues(int id,
								String first_name,
								String last_name,
								String number,
								String phone,
								String email,
								String web_page,
								String image)
    {
        this.id = id;
        this.first_name = first_name;
        this.last_name = last_name;
        this.number = number;
        this.phone = phone;
        this.email = email;
        this.web_page = web_page;
        this.image = image;
    }

	/**
	 * DESCRIPTION
	 * 
	 * @param PARAMNAME DESCRIPTION
	 * @return DESCRIPTION
	 * @exception CLASSNAME DESCRIPTION
	 * @see CLASSNAME#METHOD
	 */
    public final void clear()
    {
        id = 0;
        first_name = "";
        last_name = "";
        number = "";
        phone = "";
        email = "";
        web_page = "";
        image = "";
    }

	/**
	 * DESCRIPTION
	 * 
	 * @param PARAMNAME DESCRIPTION
	 * @return DESCRIPTION
	 * @exception CLASSNAME DESCRIPTION
	 * @see CLASSNAME#METHOD
	 */
    public final void read(DataInput din) throws IOException
    {
		if (din.readInt() != magic)
			throw new IOException("Bad magic value in OrgChartData");
        id = din.readInt();
        first_name = din.readUTF();
        last_name = din.readUTF();
        number = din.readUTF();
        phone = din.readUTF();
        email = din.readUTF();
        web_page = din.readUTF();
        image = din.readUTF();
    }

	/**
	 * DESCRIPTION
	 * 
	 * @param PARAMNAME DESCRIPTION
	 * @return DESCRIPTION
	 * @exception CLASSNAME DESCRIPTION
	 * @see CLASSNAME#METHOD
	 */
    public final void write(DataOutput dout) throws IOException
    {
		dout.writeInt(magic);
        dout.writeInt(id);
        dout.writeUTF(first_name);
        dout.writeUTF(last_name);
        dout.writeUTF(number);
        dout.writeUTF(phone);
        dout.writeUTF(email);
        dout.writeUTF(web_page);
        dout.writeUTF(image);
    }

	/**
	 * DESCRIPTION
	 * 
	 * @param PARAMNAME DESCRIPTION
	 * @return DESCRIPTION
	 * @exception CLASSNAME DESCRIPTION
	 * @see CLASSNAME#METHOD
	 */
    public final void print(PrintStream ps)
    {
        String[] labels = { "id",
                            "first_name",
                            "last_name",
                            "number",
                            "phone",
                            "email",
                            "web_page",
                            "image"
                          };
        String[] values = new String[labels.length];
        int maxlen = 0, i = 0;

        values[i] = labels[i] + ": " + id;						i++;
        values[i] = labels[i] + ": " + '\'' + first_name + '\'';		i++;
        values[i] = labels[i] + ": " + '\'' + last_name + '\'';		i++;
        values[i] = labels[i] + ": " + '\'' + number + '\'';		i++;
        values[i] = labels[i] + ": " + '\'' + phone + '\'';		i++;
        values[i] = labels[i] + ": " + '\'' + email + '\'';		i++;
        values[i] = labels[i] + ": " + '\'' + web_page + '\'';	i++;
        values[i] = labels[i] + ": " + '\'' + image + '\'';		i++;

        for (i = 0; i < labels.length; i++) {
            ps.println(values[i]);
            maxlen = Math.max(maxlen, values[i].length());
        }

        for (i = 0; i < maxlen; i++) ps.print('-');

        ps.print('\n');
    }

	/**
	 * DESCRIPTION
	 * 
	 * @param PARAMNAME DESCRIPTION
	 * @return DESCRIPTION
	 * @exception CLASSNAME DESCRIPTION
	 * @see CLASSNAME#METHOD
	 */
    public final void print()
    {
		print(System.out);
	}
}
