/**
 * Copyright(c) 1996 DTAI, Incorporated (http://www.dtai.com)
 *
 *                        All rights reserved
 *
 * Permission to use, copy, modify and distribute this material for
 * any purpose and without fee is hereby granted, provided that the
 * above copyright notice and this permission notice appear in all
 * copies, and that the name of DTAI, Incorporated not be used in
 * advertising or publicity pertaining to this material without the
 * specific, prior written permission of an authorized representative of
 * DTAI, Incorporated.
 *
 * DTAI, INCORPORATED MAKES NO REPRESENTATIONS AND EXTENDS NO WARRANTIES,
 * EXPRESS OR IMPLIED, WITH RESPECT TO THE SOFTWARE, INCLUDING, BUT
 * NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 * FITNESS FOR ANY PARTICULAR PURPOSE, AND THE WARRANTY AGAINST
 * INFRINGEMENT OF PATENTS OR OTHER INTELLECTUAL PROPERTY RIGHTS.  THE
 * SOFTWARE IS PROVIDED "AS IS", AND IN NO EVENT SHALL DTAI, INCORPORATED OR
 * ANY OF ITS AFFILIATES BE LIABLE FOR ANY DAMAGES, INCLUDING ANY
 * LOST PROFITS OR OTHER INCIDENTAL OR CONSEQUENTIAL DAMAGES RELATING
 * TO THE SOFTWARE.
 */

package orgdata;

import java.net.*;
import java.util.*;
import java.io.*;
import COM.dtai.net.*;
import COM.dtai.serial.*;

/**
 * OrgChartDbClient - a client-side API to the OrgChartDbServer database server
 * for OrgChart data
 *
 * @version	1.0b1
 * @author	DTAI, Incorporated
 *
 * $Id: OrgChartDbClient.java,v 1.17 1996/08/08 02:58:51 kadel Exp $
 *
 * $Source: /cvs/java/classes/orgdata/OrgChartDbClient.java,v $
 */

public class OrgChartDbClient extends DbClient {

    int max_org_node_id = -1;
    int max_employee_id = -1;

	/**
	 * DESCRIPTION
	 *
	 * @param PARAMNAME DESCRIPTION
	 * @return DESCRIPTION
	 * @exception CLASSNAME DESCRIPTION
	 * @see CLASSNAME#METHOD
	 */
    public OrgChartDbClient( String host, int portnum ) throws IOException {
        super( host, portnum );
    }

    /**
     * dummy db if can't connect
	 *
	 * @param PARAMNAME DESCRIPTION
	 * @return DESCRIPTION
	 * @exception CLASSNAME DESCRIPTION
	 * @see CLASSNAME#METHOD
	 */
    public OrgChartDbClient() {
        port = null;
    }

	/**
	 * DESCRIPTION
	 *
	 * @param PARAMNAME DESCRIPTION
	 * @return DESCRIPTION
	 * @exception CLASSNAME DESCRIPTION
	 * @see CLASSNAME#METHOD
	 */
    public synchronized int getNextOrgNodeId()
		throws IOException, IllegalAccessException {

        if ( port == null ) {
            max_org_node_id = 0;
        }
        if ( max_org_node_id == -1 ) {
            port.writeInt( OrgChartDbCommands.GetMaxOrgNodeId );

			if ( port.readInt() == OrgChartDbCommands.Failure ) {
				throw new IllegalAccessException(
					"Unable to get max org node id");
			}

            max_org_node_id = port.readInt();
        }
        max_org_node_id++;
        return max_org_node_id;
    }

	/**
	 * DESCRIPTION
	 *
	 * @param PARAMNAME DESCRIPTION
	 * @return DESCRIPTION
	 * @exception CLASSNAME DESCRIPTION
	 * @see CLASSNAME#METHOD
	 */
    public synchronized int getNextEmployeeId()
		throws IOException, IllegalAccessException {

        if ( port == null ) {
            max_employee_id = 0;
        }
        if ( max_employee_id == -1 ) {
            port.writeInt( OrgChartDbCommands.GetMaxEmployeeId );

			if ( port.readInt() == OrgChartDbCommands.Failure ) {
				throw new IllegalAccessException(
					"Unable to get max employee id");
			}

            max_employee_id = port.readInt();
        }
        max_employee_id++;
        return max_employee_id;
    }

	/**
	 * DESCRIPTION
	 *
	 * @param PARAMNAME DESCRIPTION
	 * @return DESCRIPTION
	 * @exception CLASSNAME DESCRIPTION
	 * @see CLASSNAME#METHOD
	 */
    public synchronized void deleteOrgNode( OrgNodeData ond )
		throws IOException, IllegalAccessException {

        if ( port == null ) return;
        port.writeInt( OrgChartDbCommands.DeleteOrgNode );
        ond.write( port );
        port.flush();

        if ( port.readInt() == OrgChartDbCommands.Failure ) {
            throw new IllegalAccessException(
                "Unable to delete org node");
        }
    }

	/**
	 * DESCRIPTION
	 *
	 * @param PARAMNAME DESCRIPTION
	 * @return DESCRIPTION
	 * @exception CLASSNAME DESCRIPTION
	 * @see CLASSNAME#METHOD
	 */
    public synchronized void saveOrgNode ( OrgNodeData ond )
		throws IOException, IllegalAccessException {

        if ( port == null ) return;
        port.writeInt( OrgChartDbCommands.SaveOrgNode );
        ond.write( port );
        port.flush();

        if ( port.readInt() == OrgChartDbCommands.Failure ) {
            throw new IllegalAccessException(
                "Unable to save org node");
        }
    }

	/**
	 * DESCRIPTION
	 *
	 * @param PARAMNAME DESCRIPTION
	 * @return DESCRIPTION
	 * @exception CLASSNAME DESCRIPTION
	 * @see CLASSNAME#METHOD
	 */
    public synchronized void getOrgChartBranchBySection(
		String section, int max_levels,
		Vector nodes, Vector employees, Vector levels )
			throws IOException {

        if ( port == null ) return;
        port.writeInt( OrgChartDbCommands.GetOrgChartBranchBySection );
        port.writeUTF( section );
		port.writeInt( max_levels );
        port.flush();

		int response;
        while( ( response = port.readInt() ) ==
			   OrgChartDbCommands.SendingObject ) {
			levels.addElement( new Integer( port.readInt() ) );
            OrgNodeData ond = new OrgNodeData();
            ond.read( port );
            nodes.addElement( ond );
            EmployeeData emp = new EmployeeData();
            emp.read( port );
            employees.addElement( emp );
        }

        if ( response != OrgChartDbCommands.NoMoreObjects ) {
            throw new IOException(
                "Failure trying to get Org Chart branch by section "+section );
        }
    }

	/**
	 * DESCRIPTION
	 *
	 * @param PARAMNAME DESCRIPTION
	 * @return DESCRIPTION
	 * @exception CLASSNAME DESCRIPTION
	 * @see CLASSNAME#METHOD
	 */
    public synchronized void getOrgChartBranchByParentId(
		int parent_id, int max_levels,
		Vector nodes, Vector employees, Vector levels )
			throws IOException {

        if ( port == null ) return;
        port.writeInt( OrgChartDbCommands.GetOrgChartBranchByParent );
        port.writeInt( parent_id );
		port.writeInt( max_levels );
        port.flush();

		int response;
        while( ( response = port.readInt() ) ==
			   OrgChartDbCommands.SendingObject ) {
			levels.addElement( new Integer( port.readInt() ) );
            OrgNodeData ond = new OrgNodeData();
            ond.read( port );
            nodes.addElement( ond );
            EmployeeData emp = new EmployeeData();
            emp.read( port );
            employees.addElement( emp );
        }

        if ( response != OrgChartDbCommands.NoMoreObjects ) {
            throw new IOException(
                "Failure trying to get Org Chart branch by parent id "+parent_id );
        }
    }

	/**
	 * DESCRIPTION
	 *
	 * @param PARAMNAME DESCRIPTION
	 * @return DESCRIPTION
	 * @exception CLASSNAME DESCRIPTION
	 * @see CLASSNAME#METHOD
	 */
    public synchronized Vector loadOrgNodesBySection( String section )
        throws IOException {

        if ( port == null ) return null;
        port.writeInt( OrgChartDbCommands.GetOrgNodeBySection );
        port.writeUTF( section );
        port.flush();

		Vector nodes = new Vector();
		int response;
        while( ( response = port.readInt() ) ==
			   OrgChartDbCommands.SendingObject ) {
            OrgNodeData ond = new OrgNodeData();
            ond.read( port );
            nodes.addElement( ond );
        }

        if ( response != OrgChartDbCommands.NoMoreObjects ) {
            throw new IOException(
                "Failure trying to get Org Chart nodes by section "+section );
        }

		return nodes;
    }

	/**
	 * DESCRIPTION
	 *
	 * @param PARAMNAME DESCRIPTION
	 * @return DESCRIPTION
	 * @exception CLASSNAME DESCRIPTION
	 * @see CLASSNAME#METHOD
	 */
    public synchronized Vector loadOrgNodesByParentId( int parent_id )
		throws IOException {

        if ( port == null ) return null;
        port.writeInt( OrgChartDbCommands.GetOrgNodeByParent );
        port.writeInt( parent_id );
        port.flush();

		Vector nodes = new Vector();
		int response;
        while( ( response = port.readInt() ) ==
			   OrgChartDbCommands.SendingObject ) {
            OrgNodeData ond = new OrgNodeData();
            ond.read( port );
            nodes.addElement( ond );
        }

        if ( response != OrgChartDbCommands.NoMoreObjects ) {
            throw new IOException(
                "Failure trying to get Org Chart nodes by parent id "+parent_id );
        }

		return nodes;
    }

	/**
	 * DESCRIPTION
	 *
	 * @param PARAMNAME DESCRIPTION
	 * @return DESCRIPTION
	 * @exception CLASSNAME DESCRIPTION
	 * @see CLASSNAME#METHOD
	 */
    public synchronized OrgNodeData loadOrgNodeById( int id )
		throws IOException {

        if ( port == null ) return null;
        port.writeInt( OrgChartDbCommands.GetOrgNodeById );
        port.writeInt( id );
        port.flush();

		OrgNodeData ond = null;
		int response;
        while( ( response = port.readInt() ) ==
			   OrgChartDbCommands.SendingObject ) {
			ond = new OrgNodeData();
            ond.read( port );
        }

        if ( response != OrgChartDbCommands.NoMoreObjects ) {
            throw new IOException(
                "Failure trying to get Org Chart node by id "+id );
        }

		return ond;
    }

	/**
	 * DESCRIPTION
	 *
	 * @param PARAMNAME DESCRIPTION
	 * @return DESCRIPTION
	 * @exception CLASSNAME DESCRIPTION
	 * @see CLASSNAME#METHOD
	 */
    public synchronized void saveEmployee( EmployeeData emp )
		throws IOException, IllegalAccessException {

        if ( port == null ) return;
        port.writeInt( OrgChartDbCommands.SaveEmployee );
        emp.write( port );
        port.flush();

        if ( port.readInt() == OrgChartDbCommands.Failure ) {
            throw new IllegalAccessException(
                "Unable to save employee");
        }
    }

	/**
	 * DESCRIPTION
	 *
	 * @param PARAMNAME DESCRIPTION
	 * @return DESCRIPTION
	 * @exception CLASSNAME DESCRIPTION
	 * @see CLASSNAME#METHOD
	 */
    public synchronized void deleteEmployee( EmployeeData emp )
		throws IOException, IllegalAccessException {

        if ( port == null ) return;
        port.writeInt( OrgChartDbCommands.DeleteEmployee );
        emp.write( port );
        port.flush();

        if ( port.readInt() == OrgChartDbCommands.Failure ) {
            throw new IllegalAccessException(
                "Unable to delete employee");
        }
    }

	/**
	 * DESCRIPTION
	 *
	 * @param PARAMNAME DESCRIPTION
	 * @return DESCRIPTION
	 * @exception CLASSNAME DESCRIPTION
	 * @see CLASSNAME#METHOD
	 */
    public synchronized EmployeeData loadEmployeeById( int id )
		throws IOException {

        if ( port == null ) return null;
        port.writeInt( OrgChartDbCommands.GetEmployeeById );
        port.writeInt( id );
        port.flush();

		EmployeeData emp = null;
		int response;
        while( ( response = port.readInt() ) ==
			   OrgChartDbCommands.SendingObject ) {
			emp = new EmployeeData();
            emp.read( port );
        }

        if ( response != OrgChartDbCommands.NoMoreObjects ) {
            throw new IOException(
                "Failure trying to get employee by id "+id );
        }

		return emp;
    }
}
