/**
 * Copyright(c) 1996 DTAI, Incorporated (http://www.dtai.com)
 *
 *                        All rights reserved
 *
 * Permission to use, copy, modify and distribute this material for
 * any purpose and without fee is hereby granted, provided that the
 * above copyright notice and this permission notice appear in all
 * copies, and that the name of DTAI, Incorporated not be used in
 * advertising or publicity pertaining to this material without the
 * specific, prior written permission of an authorized representative of
 * DTAI, Incorporated.
 *
 * DTAI, INCORPORATED MAKES NO REPRESENTATIONS AND EXTENDS NO WARRANTIES,
 * EXPRESS OR IMPLIED, WITH RESPECT TO THE SOFTWARE, INCLUDING, BUT
 * NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 * FITNESS FOR ANY PARTICULAR PURPOSE, AND THE WARRANTY AGAINST
 * INFRINGEMENT OF PATENTS OR OTHER INTELLECTUAL PROPERTY RIGHTS.  THE
 * SOFTWARE IS PROVIDED "AS IS", AND IN NO EVENT SHALL DTAI, INCORPORATED OR
 * ANY OF ITS AFFILIATES BE LIABLE FOR ANY DAMAGES, INCLUDING ANY
 * LOST PROFITS OR OTHER INCIDENTAL OR CONSEQUENTIAL DAMAGES RELATING
 * TO THE SOFTWARE.
 */

package orgdata;

import java.io.*;
import COM.dtai.serial.*;

/**
 * OrgNodeData - holds the saved information about an orgnode and assists
 * in reading and writing across network connections and to/from a file.
 *
 * @version	1.0b1
 * @author	DTAI, Incorporated
 *
 * $Id: OrgNodeData.java,v 1.11 1996/08/06 02:54:57 kadel Exp $
 *
 * $Source: /cvs/java/classes/orgdata/OrgNodeData.java,v $
 */

public class OrgNodeData implements SerialData
{
	private static final int magic = 5678;

	/**
	 * The record id of this OrgNode
	 */
    public int id;

	/**
	 * The record id of the OrgNode that is the supervisor to this one
	 */
    public int parent_id;

	/**
	 * The record id of the Employee record that goes with this node
	 */
    public int employee_id;

	/**
	 * A position among the node's siblings
	 */
    public int order;

	/**
	 * The job title
	 */
    public String role;

	/**
	 * a numeric value to indicate that this employee is a subordinate or
	 * an assistant (determines how it should be drawn).
	 */
    public int type;

	/**
	 * Normally the name of the company, division, or department that this
	 * orgnode's employee is in charge of.
	 */
    public String section_title;

	/**
	 * A URL for a web page about the company, division, or department over
	 * which this orgnode's employee is in charge
	 */
    public String section_url;

	/**
	 * The display offset on the chart away from its calculated default
	 * (normal) position in percent of the node's width
	 */
    public float offset_x;

	/**
	 * The display offset on the chart away from its calculated default
	 * (normal) position in percent of the node's width
	 */
    public float offset_y;

	/**
	 * DESCRIPTION
	 *
	 * @param PARAMNAME DESCRIPTION
	 * @return DESCRIPTION
	 * @exception CLASSNAME DESCRIPTION
	 * @see CLASSNAME#METHOD
	 */
    public OrgNodeData()
    {
		clear();
    }

	/**
	 * DESCRIPTION
	 *
	 * @param PARAMNAME DESCRIPTION
	 * @return DESCRIPTION
	 * @exception CLASSNAME DESCRIPTION
	 * @see CLASSNAME#METHOD
	 */
    public OrgNodeData(int id,
					   int parent_id,
					   int employee_id,
					   int order,
					   String role,
					   int type,
					   String section_title,
					   String section_url,
					   float offset_x,
					   float offset_y)
    {
		this.id = id;
		this.parent_id = parent_id;
		this.employee_id = employee_id;
		this.order = order;
		this.role = role;
		this.type = type;
		this.section_title = section_title;
		this.section_url = section_url;
		this.offset_x = offset_x;
		this.offset_y = offset_y;
    }

	/**
	 * DESCRIPTION
	 *
	 * @param PARAMNAME DESCRIPTION
	 * @return DESCRIPTION
	 * @exception CLASSNAME DESCRIPTION
	 * @see CLASSNAME#METHOD
	 */
    public final void setValues(int id,
							    int parent_id,
							    int employee_id,
							    int order,
							    String role,
							    int type,
							    String section_title,
							    String section_url,
							    int offset_x,
							    int offset_y)
    {
		this.id = id;
		this.parent_id = parent_id;
		this.employee_id = employee_id;
		this.order = order;
		this.role = role;
		this.type = type;
		this.section_title = section_title;
		this.section_url = section_url;
		this.offset_x = offset_x;
		this.offset_y = offset_y;
    }

	/**
	 * DESCRIPTION
	 * 
	 * @param PARAMNAME DESCRIPTION
	 * @return DESCRIPTION
	 * @exception CLASSNAME DESCRIPTION
	 * @see CLASSNAME#METHOD
	 */
    public final void clear()
    {
		id = 0;
		parent_id = 0;
		employee_id = 0;
		order = 0;
		role = "";
		type = 0;
		section_title = "";
		section_url = "";
		offset_x = (float)0.0;
		offset_y = (float)0.0;
    }

	/**
	 * DESCRIPTION
	 *
	 * @param PARAMNAME DESCRIPTION
	 * @return DESCRIPTION
	 * @exception CLASSNAME DESCRIPTION
	 * @see CLASSNAME#METHOD
	 */
    public final void read(DataInput din) throws IOException
    {
		if (din.readInt() != magic)
			throw new IOException("Bad magic value in OrgChartData");
		id = din.readInt();
		parent_id = din.readInt();
		employee_id = din.readInt();
		order = din.readInt();
		role = din.readUTF();
		type = din.readInt();
		section_title = din.readUTF();
		section_url = din.readUTF();
		offset_x = din.readFloat();
		offset_y = din.readFloat();
    }

	/**
	 * DESCRIPTION
	 *
	 * @param PARAMNAME DESCRIPTION
	 * @return DESCRIPTION
	 * @exception CLASSNAME DESCRIPTION
	 * @see CLASSNAME#METHOD
	 */
    public final void write(DataOutput dout) throws IOException
    {
		dout.writeInt(magic);
		dout.writeInt(id);
		dout.writeInt(parent_id);
		dout.writeInt(employee_id);
		dout.writeInt(order);
		dout.writeUTF(role);
		dout.writeInt(type);
		dout.writeUTF(section_title);
		dout.writeUTF(section_url);
		dout.writeFloat(offset_x);
		dout.writeFloat(offset_y);
    }

	/**
	 * DESCRIPTION
	 *
	 * @param PARAMNAME DESCRIPTION
	 * @return DESCRIPTION
	 * @exception CLASSNAME DESCRIPTION
	 * @see CLASSNAME#METHOD
	 */
    public final void print(PrintStream ps)
    {
		String[] labels = { "id",
							"parent_id",
							"employee_id",
							"order",
							"role",
							"type",
							"section_title",
							"section_url",
							"offset_x",
							"offset_y"
						  };
		String[] values = new String[labels.length];
		int maxlen = 0, i = 0;

		values[i] = labels[i] + ": " + id;							i++;
		values[i] = labels[i] + ": " + parent_id;					i++;
		values[i] = labels[i] + ": " + employee_id;					i++;
		values[i] = labels[i] + ": " + order;						i++;
		values[i] = labels[i] + ": " + '\'' + role + '\'';			i++;
		values[i] = labels[i] + ": " + type;						i++;
		values[i] = labels[i] + ": " + '\'' + section_title + '\'';	i++;
		values[i] = labels[i] + ": " + '\'' + section_url + '\'';	i++;
		values[i] = labels[i] + ": " + offset_x;					i++;
		values[i] = labels[i] + ": " + offset_y;					i++;

		for (i = 0; i < labels.length; i++) {
			ps.println(values[i]);
			maxlen = Math.max(maxlen, values[i].length());
		}

		for (i = 0; i < maxlen; i++) ps.print('-');

		ps.print('\n');
    }

	/**
	 * DESCRIPTION
	 * 
	 * @param PARAMNAME DESCRIPTION
	 * @return DESCRIPTION
	 * @exception CLASSNAME DESCRIPTION
	 * @see CLASSNAME#METHOD
	 */
    public final void print()
    {
		print(System.out);
	}
}
